// $Id: Automaton.hpp,v 1.2 2001/05/01 10:12:26 mitja Exp $

/* Automaton.hpp, 1.3, 2001-04-06, Mitja Lustrek
 *
 * This class represent a finite state automaton that controls a group of soldiers. It contains an 
 * array of states and an array of weights for each state. States switch to other states themselves
 * if update is called each turn. When requesting a weight the one for the current state is chosen
 * automatically. Weights are read from file.
 */

#ifndef __AUTOMATON__
#define __AUTOMATON__

class State;
class Group;
class Environment;

class Automaton
{
public:
	enum tWeight {WEIGHT_SAFETY, WEIGHT_DISTANCE, WEIGHT_PROGRESSION};

protected:
	char fileName [100]; // file where weights are stored
	int maxStates;       // number of states
	float **weights;     // array of weights for each state
	State **states;      // array of states
	int currentState;    // index of current state

public:
	Automaton (Group *grp, Environment *env, char fileName [], int maxStates); // construct automaton for given group and environment that reads weights from given file and has given number of states
	~Automaton ();                                                              // destructor
	void readWeightsFromFile ();                                                // read weights from file
	float getWeight (int which);                                                // get given weight for current state
	void update ();                                                             // switch state
};

#endif //__AUTOMATON__