
// $Id: auxiliary.cpp,v 1.11 2001/04/08 15:42:52 mitja Exp $

/* Auxiliary functions implementation
 *
 * Created by: Mitja Lustrek
 * Created on: 23.11.2000
 */

#include "stdafx.h"
#include <math.h>
#include "Auxiliary.hpp"
#include "Vector.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



float angleFormat(float angle) 
{
	float result;
	
	result = (float)(angle - ((int)(angle/(2.0*PI)))*2.0*PI);
	if (result < (float)(0.0))
		result += (float)(2.0*PI);
	return result;
}


float angleDifference(float angle1, float angle2) 
{
	angle1 = angleFormat(angle1);
	angle2 = angleFormat(angle2);
	
	if (fabs(angle1 - angle2) <= PI) 
		return angle1 - angle2;
	else
		if (angle1>angle2)
			return angle1 - angle2 - (float)(2.0*PI);
		else
			return angle1 - angle2 + (float)(2.0*PI);
}


float angleOfVector(Vector from, Vector to) 
{
	float result = 0;
	
	if (to.x == from.x)
		if (to.y>from.y)
			result = (float)(0.5*PI);
		else
			result = (float)(1.5*PI);
		else 
		{
			result = (float)(atan((to.y - from.y)/(to.x - from.x)));
			if ((to.x>from.x) &&(to.y < from.y))
				result = angleFormat(result);
			else
				if ((to.x < from.x) &&(to.y>from.y))
					result = angleFormat(result) - PI;
				else
					if ((to.x < from.x) &&(to.y <= from.y))
						result += PI;
		}
		return result;
}


int circleIntersectsCircle(Vector C1, float R1, Vector C2, float R2) 
{
	Vector D = C1 - C2;
	if (D.lengthSquared() < ((R1 + R2)*(R1 + R2)))
	{
		return 1;
	}
	else return 0;
}


int circleIntersectsLine(Vector C, float R, Vector Dot1, Vector Dot2) 
{
	Vector Diff = Dot1 - C;
	Vector Dir = Dot2 - Dot1;
	
	float fA0 = Diff.lengthSquared() - R*R;
	float fA1 = Dir*Diff;
	float fA2 = Dir.lengthSquared();
	float fDiscr = fA1*fA1 - fA0*fA2;
	float afT[2];
	
	if (fDiscr>0.0)
	{
		float fInvA2 =(float) 1.0/fA2;
		fDiscr =(float) sqrt(fDiscr);
		afT[0] =(-fA1 - fDiscr)*fInvA2;
		afT[1] =(-fA1 + fDiscr)*fInvA2;
		
		if ((afT[0] < 1.0 && afT[0] > 0.0) || (afT[1] < 1.0 && afT[1] > 0.0))
		{
			return 1;
		}
		else return 0;
	}
	else return 0;
}


/* circleHitByLineSegment()
 *
 * Parameters:
 *    center : center of the circle
 *    ardius : radius of the circle
 *    start  : beginning of the line segment
 *    end    : end of the line segment
 *
 * Result:
 *    float : hit accuracy
 *
 * The function calculates how well a line segment hits a circle. It returns 0
 * if it doesn't hit it at all or a value from(0, 1] if it hits. The interval
 * linearly describes how much the center was missed(1 means a direct hit, values
 * near 0 mean a near miss).
 *
 * The function uses the circleIntersectsLine() function by Uros which is heavily
 * based on the Magic Siftware implementation. It seems to work although I don't
 * understand the calculations completely. Check http:// www.magicsoftware.com for
 * a more detailed algorithm description.
 */
float circleHitByLineSegment(Vector center, float radius, Vector start, Vector end) 
{
	Vector diff = start - center;
	Vector dir = end - start;     // line segment direction
	
	// some cryptic factors to calculate the intersections
	float fA0 = diff.lengthSquared() - radius * radius;
	float fA1 = dir * diff;
	float fA2 = dir.lengthSquared();
	float fDiscr = fA1 * fA1 - fA0 * fA2;
	float afT[2];
	
	//  line would intersect the circle (?)
	if (fDiscr>0.0)
	{
		// line and circle intersections calculation (?)
		float fInvA2 =(float) 1.0 / fA2;
		fDiscr =(float) sqrt(fDiscr);
		afT[0] =(-fA1 - fDiscr) * fInvA2; // first intersection should be Dir * aft[0]
		afT[1] =(-fA1 + fDiscr) * fInvA2; // second intersection should be Dir * aft[1]
		
		// line segment actually intersects the circle (?)
		if (((afT[0] < 1.0) &&(afT[0] > 0.0)) ||((afT[1] < 1.0) &&(afT[1] > 0.0)))
		{
			Vector dist =(start + dir *((afT[1] + afT[0]) / 2)) - center; // vector to line distance vector
			return (radius - dist.length()) / radius;
		}
		else return 0;
	}
	else return 0;
}


/* randomGauss()
 *
 * Result:
 *    float : a random number with normal Gaussian distribution
 *
 * The function generates a random number with normal Gaussian distribution(N(0, 1)). You
 * only need to shift and scale it according to your needs, nothing more. Of course don't
 * forget to set the seed for the rand() function as the routine uses it internally.
 *
 * The random number is generated with the use of the inverse of the cumulative normal
 * distribution function(a random number from[0, 1] is a point where the cumulative
 * normal gets inverted). This function is approximated with the method described in
 * Abramowitz and Stegun, "Handbook of Mathematical Functions", Dover Publications, 1965.
 * The routine was written with the help of Gaussian Distribution FAQ by John D'Errico
 * at http://www.pitt.edu/~wpilib/gaussfaq.html.
 */
float randomGauss() 
{
	float p = RANDOM(1);
	float t =(float) sqrt(log(1/(p*p)));
	float x = t -(2.515517f + 0.802853f*t + 0.010328f*t*t) /(1 + 1.432788f*t + 0.189269f*t*t + 0.001308f*t*t*t);
	return x;
}



Vector rotateVector(Vector v,  float angle)
{
	float r = (float) sqrt(v.x*v.x + v.y*v.y);
	float fi = (float) atan2(v.y, v.x); 
	
	fi = fi + angle;
	
	v.x = (float)(r * cos(fi));
	v.y = (float)(r * sin(fi));
	
	return v;
}
