
// $Id$

/* AviSaver.cpp, 1.0, 2001-05-15, Damir Arh
 *
 * Details in the header file.
 */

#include "AVISaver.hpp"

AVISTREAMINFO aviStreamInfo;
PAVIFILE      pAviFile = NULL;
PAVISTREAM    psStream = NULL;
PAVISTREAM    psCompressed = NULL;

AVICOMPRESSOPTIONS FAR * lpOptions;

//Width of video
int gAVIWidth  = 0;
//Height of video
int gAVIHeight = 0;
//Fps of the video
int gAVIfps    = 0;
//pixel depth of the video
int gAVIBpp    = 0;
//current video frame
long gVideoFrame = 0;

BOOL bAVIFileOpened = FALSE;

#define ERROR_LOG(x) MessageBox(NULL, x,"Fatal Error",MB_OK)

int WriteAVIFile( char *fname, SDL_Surface *lpSurf, int FramesPerSecond )
{
	HRESULT        hr;

	//See if the vfw is new enough
	WORD wVer = HIWORD( VideoForWindowsVersion() );
	if (wVer < 0x010a)
	{
		ERROR_LOG("VideoForWindows too old.\n");
		return AVI_ERROR_OTHERERROR;   
	}

	gAVIWidth  = lpSurf->w;
	gAVIHeight = lpSurf->h;
	gAVIBpp    = lpSurf->format->BitsPerPixel;
	gAVIfps    = (FramesPerSecond > 0 ? FramesPerSecond : 0); //If some wiseguy tries a negative fps :)

	if( gAVIBpp <= 8 )
	{
		ERROR_LOG("Unsupported Pixel Format.\n");
		return AVI_ERROR_UNSUPPORTEDPIXELFORMAT;
	}

	AVIFileInit();

	hr = AVIFileOpen( &pAviFile, fname, OF_WRITE | OF_CREATE, NULL );
	if( hr != AVIERR_OK )
	{
		ERROR_LOG("AVIFileOpen failed.\n");
		return AVI_ERROR_OTHERERROR;
	}

	BITMAPINFO bmpVideoFormat = 
	{
		{ 
			sizeof( BITMAPINFO ), 
			gAVIWidth, 
			gAVIHeight, 
			1, 
			24, 
			BI_RGB, 
			gAVIWidth * gAVIHeight * 3, 
			0,
			0,
			0,
			0 
		}
	};  


    memset(&aviStreamInfo, 0, sizeof( aviStreamInfo ));

	aviStreamInfo.fccType                = streamtypeVIDEO;
	aviStreamInfo.fccHandler             = 0;
	aviStreamInfo.dwScale                = 1;
	aviStreamInfo.dwRate                 = gAVIfps;		    
	aviStreamInfo.dwSuggestedBufferSize  = 0;
	aviStreamInfo.dwQuality              = -1;
	SetRect(&aviStreamInfo.rcFrame, 0, 0, (int) bmpVideoFormat.bmiHeader.biWidth, (int) bmpVideoFormat.bmiHeader.biHeight);

	hr = AVIFileCreateStream( pAviFile, &psStream, &aviStreamInfo );
	if( hr != AVIERR_OK )
	{ 
		ERROR_LOG("Couldn't create AVI stream.\n");
		CloseAVIFile();
		return AVI_ERROR_OTHERERROR;
	}

	lpOptions = new AVICOMPRESSOPTIONS;	
	memset( lpOptions, 0, sizeof( AVICOMPRESSOPTIONS ));

	if( !AVISaveOptions( NULL , ICMF_CHOOSE_KEYFRAME|ICMF_CHOOSE_DATARATE , 1, &psStream, &lpOptions))
	{
		ERROR_LOG("AVISaveOptions failed.\n");
		CloseAVIFile();
		return AVI_ERROR_OTHERERROR;
	}

	hr = AVIMakeCompressedStream( &psCompressed, psStream, lpOptions, NULL);
	if( hr != AVIERR_OK )
	{ 
		ERROR_LOG("Couldn't create compressed stream.\n");
		CloseAVIFile();
		return AVI_ERROR_OTHERERROR;
	}

	hr = AVIStreamSetFormat( psCompressed, 0, &bmpVideoFormat, sizeof( bmpVideoFormat ));
	if( hr != AVIERR_OK )
	{ 
		ERROR_LOG("AVIStreamSetFormat returned an ERROR_LOG.\n");
		CloseAVIFile();
		return AVI_ERROR_OTHERERROR;
	}

	gVideoFrame = 0;
	bAVIFileOpened = TRUE;

	delete lpOptions;
	return 1;
}

void CloseAVIFile()
{
	if ( psStream )
		AVIStreamClose( psStream );

	if ( psCompressed )
		AVIStreamClose( psCompressed );

	if ( pAviFile )
		AVIFileClose( pAviFile );

	psStream = psCompressed = NULL;
	pAviFile = NULL;
	AVIFileExit();
	bAVIFileOpened = FALSE;
}

int WriteFrame( SDL_Surface *lpSurf )
{
   BYTE           *bSurface;
   BYTE           *bAVIData;
   WORD            wPitch;
   HRESULT         hr;
   DWORD           dwIndex, dwPixel;
   int             iX, iY;

   if( !bAVIFileOpened )
	   return AVI_ERROR_OTHERERROR;

   if( (gAVIWidth != (int)lpSurf->w) || (gAVIHeight != (int)lpSurf->h) || (gAVIBpp != (int)lpSurf->format->BitsPerPixel ))
   {
	   ERROR_LOG("Wrong Surface.\n");
	   return AVI_ERROR_WRONGSURFACE;
   }


   hr = SDL_LockSurface( lpSurf ); 
   if(hr == -1)
   {
	   ERROR_LOG("Couldn't lock surface.\n");
	   return AVI_ERROR_OTHERERROR;
   }
   bSurface = (BYTE *)lpSurf->pixels;
   wPitch   = (WORD)lpSurf->pitch;

   if( gAVIBpp == 16 ) wPitch /= 2; 

   dwIndex  = 0;
   bAVIData = new BYTE[ gAVIWidth * gAVIHeight * 3 ];

	for( iY = gAVIHeight - 1; iY >= 0; iY -- )
	{
		for( iX = 0; iX < gAVIWidth; iX ++ )
		{

			int bpp = lpSurf->format->BytesPerPixel;
			/* Here p is the address to the pixel we want to retrieve */
			Uint8 *p = (Uint8 *)lpSurf->pixels + iY * lpSurf->pitch + iX * bpp;

			switch(bpp) {
			case 1:
				dwPixel = *p;
				break;

			case 2:
				dwPixel = *(Uint16 *)p;
				break;

			case 3:
/*				if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
					dwPixel = p[0] << 16 | p[1] << 8 | p[2];
				else*/
					dwPixel = p[0] | p[1] << 8 | p[2] << 16;
				break;
					
			case 4:
				dwPixel = *(Uint32 *)p;
				break;

			default:
				dwPixel = 0;       /* shouldn't happen, but avoids warnings */
			}

/*			int bpp = lpSurf->format->BytesPerPixel;
			Uint8 *bits = ((Uint8 *)lpSurf->pixels) + iY*lpSurf->pitch + iX*bpp;
			dwPixel = *((Uint32 *)bits);			//use the largest for operations
*/
			Uint32 temp;
			/* Get Blue component */
			temp=dwPixel&lpSurf->format->Bmask; /* Isolate blue component */
			temp=temp>>lpSurf->format->Bshift;/* Shift it down to 8-bit */
			temp=temp<<lpSurf->format->Bloss; /* Expand to a full 8-bit number */
			bAVIData[ dwIndex ++ ]=(Uint8)temp;

			/* Get Green component */
			temp=dwPixel&lpSurf->format->Gmask; /* Isolate green component */
			temp=temp>>lpSurf->format->Gshift;/* Shift it down to 8-bit */
			temp=temp<<lpSurf->format->Gloss; /* Expand to a full 8-bit number */
			bAVIData[ dwIndex ++ ]=(Uint8)temp;

			/* Get Red component */
			temp=dwPixel&lpSurf->format->Rmask; /* Isolate red component */
			temp=temp>>lpSurf->format->Rshift;/* Shift it down to 8-bit */
			temp=temp<<lpSurf->format->Rloss; /* Expand to a full 8-bit number */
			bAVIData[ dwIndex ++ ]=(Uint8)temp;

		}
	}
/*   switch( gAVIBpp )
   {
      case 32:

          for( iY = gAVIHeight - 1; iY >= 0; iY -- )
		  {
	         for( iX = 0; iX < gAVIWidth; iX ++ )
			 {

		        dwPixel = *((DWORD *)( bSurface + iX * 4 + iY * wPitch ));

  		        iByte                  = ( dwPixel       ) & 0x000000ff; //Blue
		        bAVIData[ dwIndex ++ ] = iByte;
		        iByte                  = ( dwPixel >> 8  ) & 0x000000ff; //Green
		        bAVIData[ dwIndex ++ ] = iByte; 
		        iByte                  = ( dwPixel >> 16 ) & 0x000000ff; //Red
 		        bAVIData[ dwIndex ++ ] = iByte; 

			 }
		  }

	  break;

	  case 24:

          for( iY = gAVIHeight - 1; iY >= 0; iY -- )
		  {
	         for( iX = 0; iX < gAVIWidth; iX ++ )
			 {		        
		        bAVIData[ dwIndex ++ ] = *( bSurface + iX * 3 + iY * wPitch + 2 );
		        bAVIData[ dwIndex ++ ] = *( bSurface + iX * 3 + iY * wPitch + 1 ); 
 		        bAVIData[ dwIndex ++ ] = *( bSurface + iX * 3 + iY * wPitch + 0 ); 
			 }
		  }

	  break;

	  case 16:

 	      if( ddsd.ddpfPixelFormat.dwGBitMask == 0x07E0 ) // 565 mode
		  {
            for( iY = gAVIHeight - 1; iY >= 0; iY -- )
			{
	           for( iX = 0; iX < gAVIWidth; iX ++ )
			   {                    
                   wPixel = *((WORD *)( bSurface + iX * 2 + iY * wPitch * 2));

                   iByte                  = ( 8 * (( wPixel ) & 0x001f ));
		           bAVIData[ dwIndex ++ ] = iByte;

                   iByte                  = ( 4 * (( wPixel >> 5 ) & 0x003f )); 
		           bAVIData[ dwIndex ++ ] = iByte;

                   iByte                  = ( 8 * (( wPixel >> 11 ) & 0x001f ));
		           bAVIData[ dwIndex ++ ] = iByte;
			   }
			}
		  }                           
          else // 5550 (1) mode
		  {
            for( iY = gAVIHeight - 1; iY >= 0; iY -- )
			{
	           for( iX = 0; iX < gAVIWidth; iX ++ )
			   {                    
                   wPixel = *((WORD *)( bSurface + iX * 2 + iY * wPitch * 2));

                   iByte                  = ( 8 * (( wPixel ) & 0x001f ));
		           bAVIData[ dwIndex ++ ] = iByte;

                   iByte                  = ( 8 * (( wPixel >> 5 ) & 0x001f )); 
		           bAVIData[ dwIndex ++ ] = iByte;

                   iByte                  = ( 8 * (( wPixel >> 10 ) & 0x001f ));
 	               bAVIData[ dwIndex ++ ] = iByte;
			   }
			}
		  }

       break;

   } //switch*/

   hr = AVIStreamWrite( psCompressed, gVideoFrame, 1, bAVIData, gAVIWidth * gAVIHeight * 3, 0, NULL, NULL );
   if (hr != AVIERR_OK)
   { 
      ERROR_LOG("Couldn't write video data.\n");
 	  CloseAVIFile();
      return AVI_ERROR_OTHERERROR;
   }

   gVideoFrame ++;
   delete bAVIData;
   SDL_UnlockSurface( lpSurf );

   return 1;
}

