
// $Id: Bullet.cpp,v 1.6 2001/04/08 15:42:52 mitja Exp $

/* Bullet class implementation
 *
 * Created by: Martin Znidarsic
 * Created on: 23.11.2000
 */

#include "stdafx.h"
#include <math.h>
#include "Bullet.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


// starting bulletID
int Bullet::nextBulletID = 0;


/* Bullet(Vector start, float angle, float time)
 *
 * Parameters:
 *    start : starting position of the bullet
 *    angle : angle of movement
 *    time  : timestamp of the movement start
 *
 * Creates a bullet with unique ID and given starting parameters.
 */
Bullet::Bullet(Vector start, float angle1, float time1) 
{
	coords = start;            // Sets bullet's coordinates.
	angle = angle1;
	timeLastTurn = time1;
	bulletID = nextBulletID++;
	speed = 10;                // This value can be changed to suit our needs.
}


/* getCoords()
 *
 * Result:
 *    Vector : current position of the bullet
 *
 * Returns current bullet position.
 */
Vector Bullet::getCoords() 
{
	return coords;
}


/* getNewCoords(float time)
 *
 * Parameters:
 *    time : time for which the position is requested
 *
 * Result:
 *    Vector : bullet coordinates at the given time
 *
 * Returns the position of the bullet at the given time and updates
 * its actual position to this one.
 */
Vector Bullet::getNewCoords(float time1)
{
	float movetime, a, b, dist;
	Vector vnew;
	
	movetime =(time1 - timeLastTurn); // Total moving time
	dist = speed * movetime;
	b = dist *(float)sin(angle);      // Vertical and
	a = dist *(float)cos(angle);      // horizontal change
	
	// set new position and time
	coords.y += b;
	coords.x += a;
	timeLastTurn = time1;
	
	return coords;
}


/* get getID()
 *
 * Result:
 *    int : unique bulletID
 *
 * Returns the unique bulletID.
 */
int Bullet::getID() 
{
	return bulletID;
}
