
// $Id: Bullet.hpp,v 1.8 2001/04/08 15:40:52 mitja Exp $

/* Bullet class
 *
 * This is the class for all projectiles in the game. It contains all the information about
 * the bullet and the getCoords() / getNewCoords() pair which returns relevant movement
 * info to the environment class.
 *
 * The class might get extended in the future, e.g. to enable  different types of bullets.
 *
 * Created by: Damir Arh
 * Created on: 17.11.2000
 */

#ifndef __BULLET__
#define __BULLET__

#include "Vector.hpp"

class Bullet
{
private:
	// bullet internal values (default values or initialized with constructor, no changes)
	float speed;        // speed of movement [m/s]
	float angle;        // angle of movement [rad]
	float timeLastTurn; // time of previous update

	// unique ID to identify the bullet
	int bulletID;
	static int nextBulletID;

	// current bullet's state (updated every turn)
	Vector coords; // bullet coordinates [m]

public:
	// constructors and destructors
	Bullet(Vector coords, float angle, float time);

	// methods to obtain info about the bullet
	Vector getCoords();              // get current coordinates
	Vector getNewCoords(float time); // get desired coordinates
	int getID();                     // get the unique ID of the bullet
};

#endif // __BULLET__
