
// $Id: BulletClient.cpp,v 1.4 2001/04/08 15:42:52 mitja Exp $


#include "stdafx.h"
#include "BulletClient.hpp"
#include "TuiGraphics.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/*
* BulletClient::BulletClient(PicCollection *aMaps, int *aPics[], float ax,
*                            float ay, int aBulletId)
* Desc: Main constructor
* In:   aMaps is the piccollection of textures for bullets
*       aPics is the array of picture indexes in piccollection that are used
*       ax, ay are the starting coordinates of bullet
*       aBulletId is the unique id of the bullet
*/
BulletClient::BulletClient(PicCollection *aMaps, int *aPics[], float ax,
						   float ay, int aBulletId)
{
	x = ax;
	y = ay;
	lastX = ax;
	lastY = ay;
	bulletId = aBulletId;
	updated = true;
}


/* 
* void BulletClient::DrawBullet(Viewport* viewport, SDL_Surface *surface) 
* Desc: Draws bullet onto surface using some viewport properties
* In:   viewport is the base viewport to draw the bullet
*       surface is the destination screen
*/
void BulletClient::DrawBullet(Viewport* viewport, SDL_Surface *surface)
{
	TUI_rectStruct sol = TUI_Rect(
		x -(float)BULLET_CLIENT_RADIUS, 
		y -(float) BULLET_CLIENT_RADIUS, 
		2*(float)BULLET_CLIENT_RADIUS, 
		2*(float)BULLET_CLIENT_RADIUS);

	// check if it is out of clipped area
	if (viewport->RectInViewport(sol) != VIEWPORT_RECT_INSIDE)
		return;
	
	float r =((float)BULLET_CLIENT_RADIUS * viewport->GetZoom());
	int i = 0;
	float drawX =(x - viewport->posX) * viewport->GetZoom() + viewport->left;
	float drawY =(y - viewport->posY) * viewport->GetZoom() + viewport->top;
	
	const Uint32 color = MapColor(clRed, surface->format);
	for (i=0; i<(int)(r); i+=1)
		TUI_Circle(surface, TUI_Point(drawX, drawY), (float)i, color, fxOverride); // draw a circle
	
	
	TUI_Circle(surface, TUI_Point(drawX, drawY), r, MapColor(clBlack, surface->format), fxOverride); // draw a circle
}


/* These are some Get/Set properties for protected members. Set methods save
*  previous values so they can later be used for interpolation
*/
float BulletClient::GetX()
{
	return x;
}


void BulletClient::SetX(float value)
{
	if (x != value)
	{
		lastX = x;
		x = value;
	}	// if
}


float BulletClient::GetY()
{
	return y;
}


void BulletClient::SetY(float value)
{
	if (y != value)
	{
		lastY = y;
		y = value;
	}	// if
}


int BulletClient::GetId()
{
	return bulletId;
}
