// CollAvoidance.cpp: implementation of the CCollAvoidance class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "Square.hpp"
#include "Vector.hpp"
#include "CollAvoidance.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

#define INDEX(Y, X) (Y + 1) + (X + 1)*(m_iXSize+2)
#define NUM_FIELDS	2
#define max(a, b)  (((a) > (b)) ? (a) : (b)) 
#define min(a, b)  (((a) < (b)) ? (a) : (b)) 


CCollAvoidance::CCollAvoidance()
{
	m_aVectors = NULL;
	m_iXSize = m_iYSize = 0;
}

CCollAvoidance::~CCollAvoidance()
{
	ClearMap();
}

int CCollAvoidance::CreateVectMap(Square **map, int iXSize, int iYSize)
{
	ClearMap();
	if (!map) return 0;

	// allocation
	m_aVectors = (Vector **) malloc (sizeof(Vector *) * iXSize);
	for (int i=0; i < iXSize; i++) m_aVectors[i] = (Vector *) malloc (sizeof(Vector) * iYSize);

	m_iXSize = iXSize;
	m_iYSize = iYSize;
	
	// init values of vectors to value NONE
	for (i = 0; i < m_iXSize; i++)
	{
		for (int j=0; j < m_iYSize; j++)
		{
			m_aVectors[i][j].x = 0;
			m_aVectors[i][j].y = 0;
		}
	}

	//
	// calculate map vals - ugly, I know...
	//

/*
    --------------- vzporedne silnice ---------------------
	for (i=0; i < m_iXSize; i++)
	{
		for (int j=0; j < m_iYSize; j++)
		{
			//
			// ce je kvadratek prezen, pogledamo kaksne so prehodnosti njegovih sosedov in glede na to nastavimo
			// smer vektorja 
			//
			if (GetPassability(map, i, j))	// vrednosti racunamo samo za prehodne kvadratke!!!!
			{
				if (!GetPassability(map, i, j-1))	// zgornji kvadratek ni passable
				{
					if (!GetPassability(map, i+1, j))	// desni kvadratek ni passable
					{
						if (!GetPassability(map, i, j+1))	// spodnji kvadratek ni passable
						{
							if (!GetPassability(map, i-1, j)){	m_aVectors[i][j].x = 0; m_aVectors[i][j].y = 0;	}
							else							{	m_aVectors[i][j].x = 1; m_aVectors[i][j].y = 0;	}
						}
						else								// spodnji kvadratek JE passable
						{
							if (!GetPassability(map, i-1, j)){	m_aVectors[i][j].x = 0; m_aVectors[i][j].y = 1;	}
							else							{	m_aVectors[i][j].x = 0; m_aVectors[i][j].y = 0;	}
						}
					}
					else								// desni kvadratek JE passable
					{
						if (!GetPassability(map, i, j+1))	// spodnji kvadratek ni passable
						{
							if (!GetPassability(map, i-1, j)){	m_aVectors[i][j].x = 1; m_aVectors[i][j].y = 0;	}
							else							{	m_aVectors[i][j].x = 1; m_aVectors[i][j].y = 0;	}
						}
						else								// spodnji kvadratek JE passable
						{
							if (!GetPassability(map, i-1, j)){	m_aVectors[i][j].x = 0; m_aVectors[i][j].y = 0;	}
							else							{	m_aVectors[i][j].x = 1; m_aVectors[i][j].y = 0;	}
						}
					}
				}
				else							// zgornji kvadratek JE passable
				{
					if (!GetPassability(map, i+1, j))	// desni kvadratek ni passable
					{
						if (!GetPassability(map, i, j+1))	// spodnji kvadratek ni passable
						{
							if (!GetPassability(map, i-1, j)){	m_aVectors[i][j].x = 0; m_aVectors[i][j].y = 1;	}
							else							{	m_aVectors[i][j].x = 0; m_aVectors[i][j].y = 0;	}
						}
						else								// spodnji kvadratek JE passable
						{
							if (!GetPassability(map, i-1, j)){	m_aVectors[i][j].x = 0; m_aVectors[i][j].y = 1;	}
							else							{	m_aVectors[i][j].x = 0; m_aVectors[i][j].y = 1;	}
						}
					}
					else								// desni kvadratek JE passable
					{
						if (!GetPassability(map, i, j+1))	// spodnji kvadratek ni passable
						{
							if (!GetPassability(map, i-1, j)){	m_aVectors[i][j].x = 0; m_aVectors[i][j].y = 0;	}
							else							{	m_aVectors[i][j].x = 1; m_aVectors[i][j].y = 0;	}
						}
						else								// spodnji kvadratek JE passable
						{
							if (!GetPassability(map, i-1, j)){	m_aVectors[i][j].x = 0; m_aVectors[i][j].y = 1;	}
							else							{	m_aVectors[i][j].x = 0; m_aVectors[i][j].y = 0;	}
						}
					}
				}
			}
		}
	}
	-----------------------------------
*/
	// ------------ odbijajoce silnice ------------------------//
	for (i=0; i < m_iXSize; i++)
	{
		for (int j=0; j < m_iYSize; j++)
		{
			m_aVectors[i][j].x = 0;
			m_aVectors[i][j].y = 0;
			
			// 0 pomeni neprehodno, 1 pomeni prehodno

			// posevne variante
			if      (GetSurrounding(map, i, j, 0, 0, 1, 1)) { m_aVectors[i][j].x = -1; m_aVectors[i][j].y = +1;}
			else if (GetSurrounding(map, i, j, 1, 0, 0, 1)) { m_aVectors[i][j].x = -1; m_aVectors[i][j].y = -1;}
			else if (GetSurrounding(map, i, j, 1, 1, 0, 0)) { m_aVectors[i][j].x = +1; m_aVectors[i][j].y = -1;}
			else if (GetSurrounding(map, i, j, 0, 1, 1, 0)) { m_aVectors[i][j].x = +1; m_aVectors[i][j].y = +1;}

			// samo 1 celica zasedena
			else if (GetSurrounding(map, i, j, 0, 1, 1, 1)) { m_aVectors[i][j].x =  0; m_aVectors[i][j].y = +1;}
			else if (GetSurrounding(map, i, j, 1, 0, 1, 1)) { m_aVectors[i][j].x = -1; m_aVectors[i][j].y =  0;}
			else if (GetSurrounding(map, i, j, 1, 1, 0, 1)) { m_aVectors[i][j].x =  0; m_aVectors[i][j].y = -1;}
			else if (GetSurrounding(map, i, j, 1, 1, 1, 0)) { m_aVectors[i][j].x =  1; m_aVectors[i][j].y =  0;}

			// 3 celice zasedene
			else if (GetSurrounding(map, i, j, 0, 0, 0, 1)) { m_aVectors[i][j].x = -1; m_aVectors[i][j].y =  0;}
			else if (GetSurrounding(map, i, j, 1, 0, 0, 0)) { m_aVectors[i][j].x =  0; m_aVectors[i][j].y = -1;}
			else if (GetSurrounding(map, i, j, 0, 1, 0, 0)) { m_aVectors[i][j].x = +1; m_aVectors[i][j].y =  0;}
			else if (GetSurrounding(map, i, j, 0, 0, 1, 0)) { m_aVectors[i][j].x =  0; m_aVectors[i][j].y = +1;}

			// ce je samo en od robov neprehoden
			else if (GetEdges(map, i,j, 0, 1, 1, 1)) { m_aVectors[i][j].x = +1; m_aVectors[i][j].y = 1;}
			else if (GetEdges(map, i,j, 1, 0, 1, 1)) { m_aVectors[i][j].x = -1; m_aVectors[i][j].y = 1;}
			else if (GetEdges(map, i,j, 1, 1, 0, 1)) { m_aVectors[i][j].x = -1; m_aVectors[i][j].y = -1;}
			else if (GetEdges(map, i,j, 1, 1, 1, 0)) { m_aVectors[i][j].x = +1; m_aVectors[i][j].y = -1;}

		}
	}
	//-----------------------------------------------------------//		


	return 1;
}

bool CCollAvoidance::GetEdges(Square **map, int x, int y, short TopLeft, short TopRight, short BottomRight, short BottomLeft)
{
	if (TopLeft == GetPassability(map, x-1, y-1) &&
		TopRight == GetPassability(map, x+1,y-1) &&
		BottomRight == GetPassability (map, x+1,y+1) &&
		BottomLeft == GetPassability (map, x-1, y+1)) return true;
	else return false;
}



bool CCollAvoidance::GetSurrounding(Square **map, int x, int y, short top, short right, short bottom, short left)
{
	if (top    == GetPassability(map, x, y-1) &&
		right  == GetPassability(map, x+1, y) &&
		bottom == GetPassability (map, x, y+1) &&
		left   == GetPassability (map, x-1, y)) return true;
	else return false;
}



void CCollAvoidance::ClearMap()
{
	if (!m_aVectors) return;

	for (int i=0; i < m_iXSize; i++)	delete m_aVectors[i];

	delete m_aVectors;
	m_aVectors = NULL;

	m_iXSize = m_iYSize = 0;
}



int CCollAvoidance::GetPassability(Square ** map, int x, int y)
{
	if (x < 0) return 0;
	if (y < 0) return 0;
	if (x >= m_iXSize) return 0;
	if (y >= m_iYSize) return 0;
	int ret = map[INDEX(x, y)]->getPassability();
	if (ret) return 1;
	else return 0;
}


void CCollAvoidance::PrintToFile(Square **map, char *name)
{
	FILE *f = fopen (name, "wt");
	
	char buff[100];
	for (int j=0; j < m_iYSize; j++)
	{
		for (int i=0; i < m_iXSize; i++)
		{
			if (GetPassability(map, i, j))
			{
				int x = 0, y = 0;
				if (m_aVectors[i][j].x > 0.5) x = 1;
				if (m_aVectors[i][j].y > 0.5) y = 1;
				if (m_aVectors[i][j].x <-0.5) x = -1;
				if (m_aVectors[i][j].y <-0.5) y = -1;


//				sprintf(buff, "%d-%d\t", x, y);
				if ((x == 1) && (y == 1)) sprintf(buff, "\\.\t");
				else if ((x == 1) && (y == -1)) sprintf(buff, "/\t");
				else if ((x == -1) && (y == 1)) sprintf(buff, "./\t");
				else if ((x == -1) && (y == -1)) sprintf(buff, "\\\t");
				else if (x) sprintf (buff, "---\t");
				else if (y) sprintf(buff, "I\t");
				else sprintf(buff, "\t");

				fwrite(buff, 1, strlen(buff), f);
			}
			else
			{
				sprintf(buff, "XXX\t");
				fwrite(buff, 1, strlen(buff), f);
			}
		}
		sprintf(buff, "\n");
		fwrite(buff, 1, strlen(buff), f);
	}

	fclose(f);
}



Vector CCollAvoidance::GetCollAvoidanceVector(Vector soldierPosition, Vector direction)
{
	Vector dir = direction;
	Vector start = soldierPosition;
	Vector finish = start + direction;

	// normiramo dolzino
//	dir.x = dir.x / dir.length();
//	dir.y = dir.y / dir.length();

	// multiply with a constant that will influence the number of fields calculated
//	dir.x = dir.x * NUM_FIELDS;
//	dir.y = dir.y * NUM_FIELDS;


	int signX = 1;  if (dir.x < 0) signX = -1;
	int signY = 1;  if (dir.y < 0) signY = -1;

/*
	// guaranty the smallest number of fields
	if (fabs(dir.x) < NUM_FIELDS) 
	{
//		start.x =  floor(soldierPosition.x - (NUM_FIELDS / 2)); if (start.x < 0) start.x = 0;
//		finish.x = floor(soldierPosition.x + (NUM_FIELDS / 2)); if (finish.x >= m_iXSize) finish.x = m_iXSize-1;
		start.x =  floor(soldierPosition.x);   if (start.x < 0) start.x = 0;
		finish.x = floor(soldierPosition.x + 1 + (dir.x)); if (finish.x >= m_iXSize) finish.x = m_iXSize-1;
	}
	if (fabs(dir.y) < NUM_FIELDS)
	{
//		start.y  = floor(soldierPosition.y - (NUM_FIELDS / 2)); if (start.y < 0) start.y = 0;
//		finish.y = floor(soldierPosition.y + (NUM_FIELDS / 2)); if (finish.y >= m_iYSize) finish.y = m_iYSize-1;
		start.y  = floor(soldierPosition.y); if (start.y < 0) start.y = 0;
		finish.y = floor(soldierPosition.y + 1 + (dir.y)); if (finish.y >= m_iYSize) finish.y = m_iYSize-1;
	}
*/
	
	start.x = (float) floor (start.x);
	start.y = (float) floor (start.y);
	finish.x = (float) floor (finish.x);  if (finish.x >= m_iXSize) finish.x = (float) m_iXSize-1;
	finish.y = (float) floor (finish.x);  if (finish.y >= m_iYSize) finish.y = (float) m_iYSize-1;


	// naredimo tako, da je vedno start visje in bolj levo kot finish
	Vector beg(start.x, start.y);
	Vector end(finish.x, finish.y);

	start.x = min(beg.x, end.x);
	start.y = min(beg.y, end.y);
	finish.x = max (beg.x, end.x);
	finish.y = max (beg.y, end.y);


	Vector ret(0.0,0.0);
	int posX, posY;
	posX = (int) floor(start.x);
	posY = (int) floor(start.y);

	for (int i=0; i <= abs((int)floor(start.x - finish.x)); i++)
	{
		for (int j=0; j <= abs((int)floor(start.y - finish.y)); j++)
		{
			// vrednost vektorja v nekem kvadratku
			Vector temp = GetVectorAt(posX + i, posY + j);

			
			// vrednost utezimo z razdaljo med soldierPos in kvadratkom
			float weight;
			if ( (floor(soldierPosition.x) == posX+i) && (floor(soldierPosition.y) == posY+j) ) weight = 1.0;
			else 
			{
				Vector diff (soldierPosition.x-posX+i, soldierPosition.y-posY+j);
				weight = 1 / diff.lengthSquared();
			}
			
			// utezimo
			temp = temp * weight;

			/*
			// pristejemo koncnemu vektorju
			float scalar = (dir.x * temp.x) + (dir.y * temp.y);
			if (scalar > 0) ret = ret +temp;
			else ret = ret -temp;
			*/
			ret = ret + temp;
		}
	}
	return ret;
}

Vector CCollAvoidance::GetVectorAt(int x, int y)
{
	Vector ret (0.0,0.0);
	if (x < 0) return ret;
	if (y < 0) return ret;
	if (x >= m_iXSize) return ret;
	if (y >= m_iYSize) return ret;
	
	if (!m_aVectors) return ret;

	ret.x = m_aVectors[x][y].x;
	ret.y = m_aVectors[x][y].y;

	return ret;
}


