
// $Id: CommandPanel.cpp,v 1.6 2001-04-08 19:51:43+02 damir Exp damir $

#include "stdafx.h"
#include "CommandPanel.hpp"
#include "TuiGraphics.hpp"
#include "Viewport.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CommandPanel::CommandPanel(Console* aConsole)
{	
	left = 0;
	top = 0;
	width = 0;
	height = 0;

	OnSelectShoot = NULL;
	OnSelectMove = NULL;
	OnSelectAddWaypoint = NULL;
	OnRemoveWaypoint = NULL;
	OnExecuteWaypoint = NULL;
	OnChangeGroup = NULL;
	OnSelectAddAOD = NULL;
	OnRemoveAOD = NULL;
	OnChangeAgr = NULL;
	area = NULL;


	buttonEnabled = cpNone;
	console = aConsole;
}


bool CommandPanel::ProcessEvent(const SDL_Event *event, SDL_Surface* display)
{
	bool value = false;
	area = NULL;
	
	if (event->type == SDL_KEYDOWN)
	{
		switch (event->key.keysym.sym)
		{
		case COMMAND_PANEL_KEY_SHOOT:			//shoot
			if (buttonEnabled == cpShoot)
			{
				buttonEnabled = cpNone;
				value = false;
			}
			else
			{
				buttonEnabled = cpShoot;
				value = true;
			}
			
			if (OnSelectShoot != NULL)
				OnSelectShoot(value);
			
			return true;
			break;

		case COMMAND_PANEL_KEY_MOVE:		//move
			if (buttonEnabled == cpMove)
			{
				buttonEnabled = cpNone;
				value = false;				
			}
			else
			{
				buttonEnabled = cpMove;
				value = true;				
			}
						
			if (OnSelectMove != NULL)
				OnSelectMove(value);

			return true;
			break;

		case COMMAND_PANEL_KEY_ADDWP:		//waypoint
			if (buttonEnabled == cpAddWaypoint)
			{
				buttonEnabled = cpNone;
				value = false;
				console->displayWaypoints = false;
			}
			else
			{
				buttonEnabled = cpAddWaypoint;
				value = true;
				console->displayWaypoints = true;
			}
			
			if (OnSelectAddWaypoint != NULL)
				OnSelectAddWaypoint(value);

			return true;
			break;
		case COMMAND_PANEL_KEY_REMOVEWP:	//remove
			if (OnRemoveWaypoint != NULL)
				OnRemoveWaypoint();

			return true;
			break;
		case COMMAND_PANEL_KEY_EXECWP:		//execute
			if (OnExecuteWaypoint != NULL)
				OnExecuteWaypoint();

			return true;
			break;
		case COMMAND_PANEL_KEY_GROUP_0 :
		case COMMAND_PANEL_KEY_GROUP_1 :
		case COMMAND_PANEL_KEY_GROUP_2 :
		case COMMAND_PANEL_KEY_GROUP_3 :
		case COMMAND_PANEL_KEY_GROUP_4 :
		case COMMAND_PANEL_KEY_GROUP_5 :
		case COMMAND_PANEL_KEY_GROUP_6 :
		case COMMAND_PANEL_KEY_GROUP_7 :
		case COMMAND_PANEL_KEY_GROUP_8 :
		case COMMAND_PANEL_KEY_GROUP_9 :
			if ((event->key.keysym.mod == KMOD_LSHIFT) || (event->key.keysym.mod == KMOD_RSHIFT)){			
				if (OnChangeAgr != NULL)
					OnChangeAgr(
						console->groupID,
						(float)(event->key.keysym.sym - SDLK_0)/10.0f
					);
			}else{
				if (OnChangeGroup != NULL){
					bool set;

					set = (event->key.keysym.mod == KMOD_LCTRL) ||
						(event->key.keysym.mod == KMOD_RCTRL);
					
					OnChangeGroup(event->key.keysym.sym - COMMAND_PANEL_KEY_GROUP_0, set); //index of a group
				} //else
			}  //case
			return true;
			break;
		case COMMAND_PANEL_KEY_ADDAOD:		//waypoint
			if (buttonEnabled == cpAddAOD)
			{
				buttonEnabled = cpNone;
				value = false;				
			}
			else
			{
				buttonEnabled = cpAddAOD;
				value = true;				
			}
			
			if (OnSelectAddAOD != NULL)
				OnSelectAddAOD(value);

			return true;
			break;
		case COMMAND_PANEL_KEY_REMOVEAOD:	//remove
			if (OnRemoveAOD != NULL)
				OnRemoveAOD();

			return true;
			break;
		
		}	// switch
	}	// if
	
	UpdateDisplay(display);
	return false;
}


CPButton CommandPanel::GetButtonEnabled()
{
	return buttonEnabled;
}

void CommandPanel::UpdateDisplay(SDL_Surface* screen)
{
	const CP_BORDER = 5;
	const ICON_SIZE = 25;

	clipRect.x = float(left);
	clipRect.y = float(top);
	clipRect.w = float(width);
	clipRect.h = float(height);


	if (buttonEnabled == cpMove)
		DrawMove(screen, left + CP_BORDER, top + CP_BORDER, ICON_SIZE, true);
	else
		DrawMove(screen, left + CP_BORDER, top + CP_BORDER, ICON_SIZE, false);

	if (buttonEnabled == cpShoot)
		DrawShoot(screen, left + 2*CP_BORDER + ICON_SIZE, top + CP_BORDER, ICON_SIZE, true);
	else
		DrawShoot(screen, left + 2*CP_BORDER + ICON_SIZE, top + CP_BORDER, ICON_SIZE, false);

	if (buttonEnabled == cpAddWaypoint)
		DrawAddWaypoint(screen, left + 3*CP_BORDER + 2*ICON_SIZE, top + CP_BORDER, ICON_SIZE, true);
	else
		DrawAddWaypoint(screen, left + 3*CP_BORDER + 2*ICON_SIZE, top + CP_BORDER, ICON_SIZE, false);

	if (buttonEnabled == cpAddAOD)
		DrawAddAOD(screen, left + 4*CP_BORDER + 3*ICON_SIZE, top + CP_BORDER, ICON_SIZE, true);
	else
		DrawAddAOD(screen, left + 4*CP_BORDER + 3*ICON_SIZE, top + CP_BORDER, ICON_SIZE, false);


	SDL_UpdateRect(screen, left, top, width, height);
}

void CommandPanel::DrawMove(SDL_Surface* screen, int x, int y, int size, bool on)
{
	int border = (int)(size * 0.1);
	TUI_pointStruct p1, p2;
	//move

	if (!on){
		SDL_Rect r;
		r.x = x;
		r.y = y;
		r.w = size + 1;
		r.h = size + 1;

		SDL_FillRect(screen, &r, MapColor(clBlack, screen->format));
		return;
	}

	p1.x = (float)x;
	p1.y = (float)y;
	p2.x = (float)(x + size);
	p2.y = (float)(y + size);
	TUI_Rect(screen, p1, p2, MapColor(clGreen, screen->format), fxOverride, false);

	p1.x = (float)(x + (int)(size / 2));
	p1.y = (float)(y + border);
	p2.x = p1.x;
	p2.y = (float)(y + size - border);
	TUI_Line(p1, p2, MapColor(clGreen, screen->format), screen, fxOverride);

	p2.x = (float)(p1.x - (int)(size / 4));
	p2.y = (float)(p1.y + (int)(size / 4));
	TUI_Line(p1, p2, MapColor(clGreen, screen->format), screen, fxOverride);

	p2.x = (float)(p1.x + (int)(size / 4));
	p2.y = (float)(p1.y + (int)(size / 4));
	TUI_Line(p1, p2, MapColor(clGreen, screen->format), screen, fxOverride);
}

void CommandPanel::DrawShoot(SDL_Surface* screen, int x, int y, int size, bool on)
{
	int					border	= (int)(size * 0.1);
	TUI_pointStruct		p1, p2;
	float				r;
	//move

	if (!on){
		SDL_Rect r;
		r.x = x;
		r.y = y;
		r.w = size + 1;
		r.h = size + 1;

		SDL_FillRect(screen, &r, MapColor(clBlack, screen->format));
		return;
	}

	p1.x = (float)x;
	p1.y = (float)y;
	p2.x = (float)(x + size);
	p2.y = (float)(y + size);
	TUI_Rect(screen, p1, p2, MapColor(clRed, screen->format), fxOverride, false);

	r = size / 2 - (float)border;
	p1.x = (float)(x + (int)(size / 2));
	p1.y = (float)(y + (int)(size / 2));
	TUI_Circle(screen, p1, r, MapColor(clRed, screen->format), fxOverride);

	r = size / 4 - (float)border;
	TUI_Circle(screen, p1, r, MapColor(clRed, screen->format), fxOverride);
	
	TUI_SetPixel(p1, MapColor(clRed, screen->format), screen, fxOverride);
}

void CommandPanel::DrawAddWaypoint(SDL_Surface* screen, int x, int y, int size, bool on)
{
	int					border	= (int)(size * 0.1);
	TUI_pointStruct		p1, p2;
	float				r;
	//move

	if (!on){
		SDL_Rect r;
		r.x = x;
		r.y = y;
		r.w = size + 1;
		r.h = size + 1;

		SDL_FillRect(screen, &r, MapColor(clBlack, screen->format));
		return;
	}

	p1.x = (float)x;
	p1.y = (float)y;
	p2.x = (float)(x + size);
	p2.y = (float)(y + size);
	TUI_Rect(screen, p1, p2, MapColor(clCyan, screen->format), fxOverride, false);

	r = size / 2 - (float)border;
	p1.x = (float)(x + border);
	p1.y = (float)(y + (int)(size / 2));
	p2.x = (float)(x + (int)(size / 2));
	p2.y = (float)(y + size - border);
	TUI_Line(p1, p2, MapColor(clCyan, screen->format), screen, fxOverride);

	p1.x = (float)(x + (int)(size / 2));
	p1.y = (float)(y + size - border);
	p2.x = (float)(x + (int)(size / 2));
	p2.y = (float)(y + border);
	TUI_Rect(screen, p1, p2, MapColor(clCyan, screen->format), fxOverride, false);

	p1.x = (float)(x + (int)(size / 2));
	p1.y = (float)(y + border);
	p2.x = (float)(x + size - border);
	p2.y = (float)(y + (int)(size / 2));
	TUI_Line(p1, p2, MapColor(clCyan, screen->format), screen, fxOverride);
}

void CommandPanel::DrawAddAOD(SDL_Surface* screen, int x, int y, int size, bool on)
{
	int					border	= (int)(size * 0.1);
	TUI_pointStruct		p1, p2;
	float				r;
	//move

	if (!on){
		SDL_Rect r;
		r.x = x;
		r.y = y;
		r.w = size + 1;
		r.h = size + 1;

		SDL_FillRect(screen, &r, MapColor(clBlack, screen->format));
		return;
	}

	p1.x = (float)x;
	p1.y = (float)y;
	p2.x = (float)(x + size);
	p2.y = (float)(y + size);
	TUI_Rect(screen, p1, p2, MapColor(clRed, screen->format), fxOverride, false);

	r = size / 2 - (float)border;
	p1.x = (float)(x + (int)(size / 2));
	p1.y = (float)(y + (int)(size / 2));
	TUI_Circle(screen, p1, r, MapColor(clRed, screen->format), fxOverride);

	TUI_SetPixel(p1, MapColor(clRed, screen->format), screen, fxOverride);
}

