
// $Id: CommandPanel.hpp,v 1.5 2001-04-08 19:51:45+02 damir Exp damir $

/* CommandPanel class
 *
 * Interface with the user containing status of the units, orders, maps... 
 * Created by: Matija Jekovec
 * Created on: 30.11.2000
 */

#ifndef __COMMANDPANEL__
#define __COMMANDPANEL__

class CommandPanel;

#define COMMAND_PANEL_KEY_SHOOT SDLK_s
#define COMMAND_PANEL_KEY_MOVE SDLK_m
#define COMMAND_PANEL_KEY_ADDWP SDLK_w
#define COMMAND_PANEL_KEY_REMOVEWP SDLK_r
#define COMMAND_PANEL_KEY_EXECWP SDLK_e
#define COMMAND_PANEL_KEY_GROUP_0 SDLK_0
#define COMMAND_PANEL_KEY_GROUP_1 SDLK_1
#define COMMAND_PANEL_KEY_GROUP_2 SDLK_2
#define COMMAND_PANEL_KEY_GROUP_3 SDLK_3
#define COMMAND_PANEL_KEY_GROUP_4 SDLK_4
#define COMMAND_PANEL_KEY_GROUP_5 SDLK_5
#define COMMAND_PANEL_KEY_GROUP_6 SDLK_6
#define COMMAND_PANEL_KEY_GROUP_7 SDLK_7
#define COMMAND_PANEL_KEY_GROUP_8 SDLK_8
#define COMMAND_PANEL_KEY_GROUP_9 SDLK_9
#define COMMAND_PANEL_KEY_ADDAOD SDLK_a
#define COMMAND_PANEL_KEY_REMOVEAOD SDLK_d

#include "SDL.h"
#include "Console.hpp"
#include "AreaOfDanger.hpp"

enum CPButton {
	cpNone,
	cpShoot,
	cpAddWaypoint,
	cpMove,
	cpAddAOD
};

class CommandPanel
{
private:	
	CPButton	buttonEnabled;
	Console*	console;

	void		UpdateDisplay(SDL_Surface* screen);	
	void		DrawMove(SDL_Surface* screen, int x, int y, int size, bool on);
	void		DrawShoot(SDL_Surface* screen, int x, int y, int size, bool on);
	void		DrawAddWaypoint(SDL_Surface* screen, int x, int y, int size, bool on);
	void		DrawAddAOD(SDL_Surface* screen, int x, int y, int size, bool on);
public:
	int			left;		//on screen position
	int			top;
	int			width;
	int			height;
	AreaOfDanger *area;

	
	CommandPanel(Console* aConsole);	
	bool     ProcessEvent(const SDL_Event *event, SDL_Surface* display);		// processes events like hot-keys and click on buttons
	CPButton GetButtonEnabled(); // returns buttonEnabled	
	/* the button or hot-key for moving was selected or ESC or hot-key for
	   the second time to cancel it */
	void (*OnSelectMove)(bool selected);
	void (*OnSelectShoot)(bool selected); // same for shooting
	void (*OnSelectAddWaypoint)(bool selected);
	void (*OnRemoveWaypoint)();
	void (*OnExecuteWaypoint)();
	void (*OnSelectAddAOD)(bool selected);
	void (*OnRemoveAOD)();
	void (*OnChangeGroup)(int groupIndex, bool set); /*group index is the key pressed 0 through 
	* 9, if set is true, the group was set else it was selected*/
	void (*OnChangeAgr)(int groupIndex, float value);

};

#endif // __COMMANDPANEL__
