
// $Id: Connection.cpp,v 1.9 2001/04/15 15:06:31 mitja Exp $

//////////////////////////////////////////////////////////////////////
//
// Connection.cpp: implementation of the CConnection class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <stdlib.h>
#include "Connection.h"
#include "Log.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



LRESULT CALLBACK WinProc(HWND hWnd, UINT message, UINT wParam, LONG lParam);

// flooding
#define FLOOD_CHECK				1000	// time interval to check buffer sizes
#define FLOOD_SIZE				1000	// if received buffer is longer than this, we log it to file



#define SOCKET_PORT             1666

#define ERR_INVALID_IP_ADDRESS  1
#define WSA_EVENT               (WM_USER + 10)
#define MAX_RECV                10000
#define BLOCK_SIZE              100000

#define CLASSNAME               "CONN"


CConnection *pConnection;
char        recv_buff[MAX_RECV];
bool        bThreadCanRun;
bool        bThreadRuning;
HANDLE      hThread;


////////////////////////////////////////////////////////////////////////////////////////////
////                         ///////////////////////////////////////////////////////////////
//// CLASS  :   CCONNECTION  ///////////////////////////////////////////////////////////////
////                         ///////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////

CConnection::CConnection()
{
	m_ConnType = -1;
	m_bInitialized = false;
	m_bSocketCreated = false;
	m_bConnected = false;
	m_iLastError = 0;
	m_iClientCount = 0;
	m_iCurrentClient = 0;
	m_hWnd = NULL;
	m_Socket = NULL;
	m_ListenSocket = NULL;
	
	m_pClients = NULL;
	m_pBuffers = NULL;
	
	pConnection = this;
	
	bThreadRuning = false;
	bThreadCanRun = true;

	m_pLog = new Log ("connectionInfo.txt", true);
}

CConnection::~CConnection()
{
	Finish();
	
	m_bInitialized = false;
	m_bConnected = false;
	m_iLastError = 0;
	m_iClientCount = 0;
	m_iCurrentClient = 0;
	m_pClients = NULL;
	m_Socket = NULL;
	m_ListenSocket = NULL;
	pConnection = this;	

	KillTimer(m_hWnd, 0);
	if (m_pLog) delete m_pLog;
	m_pLog = NULL;
}


////////////////////////////////////////////////////////////////////////////////////////////
////   INITIALIZE           ////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
bool CConnection::Initialize(conn_type type)
{
	WSADATA WSAStartupData;
	SOCKADDR_IN sockAddr;
	
	m_iLastError = 0;
	if (m_bInitialized)
		return true;
	
	// 0 if successful
	int failed = WSAStartup(MAKEWORD(2, 0), &WSAStartupData);
	if (failed)
		return false;	
	
	m_bInitialized = true;
	m_ConnType = type;
	
	// create invisible window to receive notifications on new packets
	bool ok = CreateMyWindow();
	if (!ok)
		return false;
	
	HOSTENT *hostEnt;
	char szName[256];
	
	// get host info
	failed = gethostname(szName, 256);
	if (failed)
	{
		m_iLastError = WSAGetLastError();
		return false; 
	}
	
	strcpy(m_szMyCompName, szName);
	
	hostEnt = gethostbyname(szName);
	if (hostEnt == NULL)
	{
		m_iLastError = WSAGetLastError();
		return false; 
	}
	
	memcpy((char FAR *) &sockAddr.sin_addr, hostEnt->h_addr, hostEnt->h_length);
	sockAddr.sin_family = AF_INET;
	sockAddr.sin_port = htons(SOCKET_PORT);
	
	m_bSocketCreated = true;
	
	if (type == SERVER)
	{
		// kreiraj listening socket
		m_ListenSocket = WSASocket(AF_INET, SOCK_STREAM, 0, NULL, 0, WSA_FLAG_OVERLAPPED);
		if (m_ListenSocket == INVALID_SOCKET)
		{
			m_iLastError = WSAGetLastError();
			return false; 
		}
		
		// bind it
		failed = bind(m_ListenSocket, (struct sockaddr FAR *) &sockAddr, sizeof(sockAddr));
		if (failed)
		{
			m_iLastError = WSAGetLastError();
			return false; 
		}
		
		// doloci za katere evente prejme message
		failed = WSAAsyncSelect(m_ListenSocket, m_hWnd, WSA_EVENT, FD_READ | FD_OOB | FD_ACCEPT | FD_CONNECT | FD_CLOSE);
		if (failed)
		{
			m_iLastError = WSAGetLastError();
			return false; 
		}
	}
	
	return true;
}


////////////////////////////////////////////////////////////////////////////////////////////
////   LISTEN        ///////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
bool CConnection::Listen()
{
	m_iLastError = 0;
	if (!m_bInitialized)
		return false;
	
	if (m_ConnType == CLIENT)
		return false;
	
	// poslusaj
	int failed = listen(m_ListenSocket, 1);
	if (failed)
	{
		m_iLastError = WSAGetLastError();
		return false; 
	}
	
	return true;
}


////////////////////////////////////////////////////////////////////////////////////////////
////   CALL   - client calls the IP address ////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
bool CConnection::Call(char *szIPAddr)
{
	int failed;
	SOCKADDR_IN sockAddr;
	HOSTENT *hostEnt;
	
	m_iLastError = 0;
	if (!m_bInitialized)
		return false;
	if (m_ConnType == SERVER)
		return false;
	
	// kreiramo socket
	m_Socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (m_Socket == INVALID_SOCKET)
	{
		m_iLastError = WSAGetLastError();
		return false; 
	}
	
	// kreiraj ustrezno stevilo bufferjev
	//	ASSERT (!m_pBuffers);	
	m_pBuffers =(CBuffer **) realloc(m_pBuffers, sizeof(CBuffer));
	m_pBuffers[0] = new CBuffer;
	
	m_iClientCount = 1;
	
	
	DWORD dwIPAddress;
	bool ok = ConvertToDW(szIPAddr, &dwIPAddress);
	if (!ok)
		return false;
	
	hostEnt = gethostbyaddr((char *) &dwIPAddress, 4, AF_INET);
	
	if (hostEnt == NULL)
		return false;
	if (hostEnt->h_name == NULL)
		return false;
	
	memcpy((char FAR *) &sockAddr.sin_addr, hostEnt->h_addr, hostEnt->h_length);
	sockAddr.sin_family = AF_INET;
	sockAddr.sin_port = htons(SOCKET_PORT);
    
	// connect client and server
	failed = connect(m_Socket, (struct sockaddr FAR *) &sockAddr, sizeof(sockAddr));
	if (failed == SOCKET_ERROR) 
	{
		m_iLastError = WSAGetLastError();
		closesocket(m_Socket);
		m_Socket = INVALID_SOCKET;
		m_bConnected = false;
		return false;
	}
	
	m_bConnected = true;
	
	WSAAsyncSelect(m_Socket, m_hWnd, WSA_EVENT, FD_READ | FD_CONNECT | FD_CLOSE);
	
	return true;
}


////////////////////////////////////////////////////////////////////////////////////////////
////   ACCEPT  - server accepts new client /////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
bool CConnection::Accept()
{
	if (m_ConnType == CLIENT)
		return false;
	
	SOCKADDR_IN addr;
	int len = sizeof(SOCKADDR_IN);
	
	m_iLastError = 0;
	
	// add new client
	m_pClients =(SOCKET **) realloc(m_pClients, (m_iClientCount + 1) * sizeof(void *));
	m_pClients[m_iClientCount] = new SOCKET;
	*m_pClients[m_iClientCount] = accept(m_ListenSocket, (struct sockaddr FAR *) &addr, &len);
	
	if (*m_pClients[m_iClientCount] == INVALID_SOCKET) 
	{
		m_iLastError = WSAGetLastError();
		return false;
	}
	
	// za vsakega novega klienta kreiramo BUFF_COUNT novih bufferjev, ki jih izmenicno filamo
	m_pBuffers =(CBuffer **) realloc(m_pBuffers, (m_iClientCount + 1) * sizeof(void *));
	m_pBuffers[m_iClientCount] = new CBuffer;
	
	m_iClientCount++;						
	
	m_bConnected = true;
	return true;
}

////////////////////////////////////////////////////////////////////////////////////////////
////   FINISH    - close all sockets and clean up //////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
void CConnection::Finish()
{
	if (!m_bInitialized)
		return;
	
	if (m_Socket != INVALID_SOCKET)
		shutdown(m_Socket, SD_BOTH);
	closesocket(m_Socket);
	m_Socket = INVALID_SOCKET;
	
	if (m_ConnType == SERVER)
	{
		for (int i = m_iClientCount; i >0; i--)
		{
			// sockets
			shutdown(*m_pClients[i - 1], SD_BOTH);
			closesocket(*m_pClients[i - 1]);
			delete m_pClients[i - 1];
			
			// buffers
			delete m_pBuffers[i - 1];
		}
	}
	else if (m_ConnType == CLIENT)
	{
		shutdown(m_Socket, SD_BOTH);
		closesocket(m_Socket);
		if (m_pBuffers)
			delete m_pBuffers[0];
	}
	
	if (m_pBuffers)
		delete m_pBuffers; // delete tabelo pointerjev
	if (m_pClients)
		delete m_pClients;
	
	m_pClients = NULL;
	m_pBuffers = NULL;
	
	m_iClientCount = 0;
	m_bSocketCreated = false;
	m_bConnected = false;
}


////////////////////////////////////////////////////////////////////////////////////////////
////   SEND   - send buffer buff of size size //////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
int CConnection::Send(char *buff, int size)
{
	if (!m_bConnected)
		return 0;
	if ((m_ConnType == SERVER) &&(!m_pClients))
		return 0;
	if ((m_ConnType == CLIENT) &&(m_Socket == INVALID_SOCKET))
		return 0;
	

	CHeader head;
	head.cType = DATA;
	head.iBuffLen = size;
	
	char * temp = new char[size + sizeof(head)];
	memcpy(&temp[0], &head, sizeof(head));
	memcpy(&temp[sizeof(head)], buff, size);
	
	if (m_ConnType == SERVER)
	{
		for (int i = 0; i < m_iClientCount; i++)
			send(*m_pClients[i], temp, size + sizeof(head), 0);
	}
	else send(m_Socket, temp, size + sizeof(head), 0);
	
	delete temp;
	
	return size;
}

// added by Damir to make separate sending for 
int CConnection::Send(int i, char *buff, int size)
{
	if (!m_bConnected)
		return 0;
	if ((m_ConnType == SERVER) &&(!m_pClients))
		return 0;
	if ((m_ConnType == CLIENT) &&(m_Socket == INVALID_SOCKET))
		return 0;
	
	CHeader head;
	head.cType = DATA;
	head.iBuffLen = size;
	
	char * temp = new char[size + sizeof(head)];
	memcpy(&temp[0], &head, sizeof(head));
	memcpy(&temp[sizeof(head)], buff, size);
	
	if (m_ConnType == SERVER)
	{
		if (i < m_iClientCount)
			send(*m_pClients[i], temp, size + sizeof(head), 0);
	}
	else send(m_Socket, temp, size + sizeof(head), 0);
	
	delete temp;
	
	return size;
}



////////////////////////////////////////////////////////////////////////////////////////////
////   RECEIVE   - receive from client clientID pointer to a buffer of size size ///////////
////////////////////////////////////////////////////////////////////////////////////////////
int CConnection::Receive(int clientID, void **buff, int *size)
{
	// ASSERT (buff);
	// ASSERT (size);
	
	int id;
	if (!IsNewDataAvailable(&id))
		return 0;
	
	CHeader * head =(CHeader *) m_pBuffers[id]->m_pData;
	*size = head->iBuffLen;
	*buff = m_pBuffers[id]->m_pData + sizeof(CHeader);

	
	return head->cType;
}

////////////////////////////////////////////////////////////////////////////////////////////
////   IS NEW DATA AVAILABLE  //////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
bool CConnection::IsNewDataAvailable(int *clientID)
{
	// ASSERT(clientID);
	
	if (!m_bConnected)
		return false;
	
	CHeader *head;
	
	// poiscimo, ce kateri od klientov polni buffer
	for (int j = 0; j < m_iClientCount; j++, m_iCurrentClient =(m_iCurrentClient + 1) % m_iClientCount)
	{
		if (m_pBuffers[m_iCurrentClient]->m_pData)
		{
			head =(CHeader *) m_pBuffers[m_iCurrentClient]->m_pData;
			int len = head->iBuffLen; // MITJA: CRASH
			
			if (abs(len + sizeof(CHeader)) <= abs(m_pBuffers[m_iCurrentClient]->m_iTotalLen)) 
			{
				*clientID = m_iCurrentClient; 
				m_iCurrentClient =(m_iCurrentClient + 1) % m_iClientCount; 
				return true; 
			}
		}
	}
	return false;
}

////////////////////////////////////////////////////////////////////////////////////////////
////   DISPOSE BUFFER  -  destroy the content of a used buffer that came from client id ////
////////////////////////////////////////////////////////////////////////////////////////////
void CConnection::DisposeBuffer(int id)
{
	// ASSERT (id < m_iClientCount);
	
	m_pBuffers[id]->DisposeBuffer();
}

////////////////////////////////////////////////////////////////////////////////////////////
////   APPEND DATA   - append new packet to the queue  /////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
void CConnection::AppendData(int clientID, char *buff, int size)
{
	// ASSERT (clientID >=0);
	
	m_pBuffers[clientID]->AppendData(buff, size);
}


//////////////////////////////////////////////////////////////////////////////////////////////
// GET CLIENT ID  - get clientID of a client that has this socket  ///////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////
int	CConnection::GetClientID(SOCKET socket)
{
	if (m_ConnType == CLIENT)
	{
		if (m_Socket == socket)
			return 0;
		else return -1;
	}
	
	for (int i = 0; i < m_iClientCount; i++)
	{
		if (*m_pClients[i] == socket)
			return i;
	}
	
	return -1;
}

//////////////////////////////////////////////////////////////////////////////////////////////
// CLOSE SOCKET  - close connection with a client clientID  //////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////
void CConnection::CloseSocket(int clientID)
{
	if (m_ConnType == CLIENT)
	{
		shutdown(m_Socket, SD_BOTH);
		closesocket(m_Socket);
		
		m_bConnected = false;
		delete m_pBuffers[clientID];
		m_pBuffers = NULL;
		m_iClientCount--;
	}
	else if (m_ConnType == SERVER)
	{
		shutdown(*m_pClients[clientID], SD_BOTH);
		closesocket(*m_pClients[clientID]);
		delete m_pClients[clientID];
		memmove(&m_pClients[clientID], &m_pClients[clientID + 1], sizeof(void *) *(m_iClientCount - clientID - 1));
		
		delete m_pBuffers[clientID];
		memmove(&m_pBuffers[clientID], &m_pBuffers[clientID + 1], sizeof(void *) *(m_iClientCount - clientID - 1));	
		
		m_iClientCount--;
		if (m_iClientCount == 0) 
		{
			m_bConnected = false;
			delete m_pClients;
			m_pClients = NULL;
		}	
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////
// GET CLIENT COUNT  - return number of connected clients   //////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////
int CConnection::GetClientCount()
{
	if (m_ConnType == CLIENT)
		return 0;
	
	return m_iClientCount;
}

//////////////////////////////////////////////////////////////////////////////////////////////
// SEND MAP  - send a map from server to every client   //////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////
bool CConnection::SendMap(char *buff, int size)
{
	if (m_ConnType == CLIENT)
		return false;
	if (!m_bConnected)
		return false;
	
	char * temp =  new char[size + sizeof(CHeader)];
	CHeader head;
	head.cType = MAP;
	head.iBuffLen = size;
	
	memcpy(temp, &head, sizeof(CHeader));
	memcpy(&temp[sizeof(CHeader)], buff, size);
	
	for (int i = 0; i < m_iClientCount; i++)
	{
		send(*m_pClients[i], temp, size + sizeof(CHeader), 0);
	}
	
	delete temp;
	
	return true;
}


///////////////////////////////////////////////////////////////////////////////////////////
////   START GAME  ///////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
void CConnection::StartGame()
{
	if (m_ConnType == SERVER)
		return;
	if (!m_bInitialized)
		return;
	if (!m_bConnected)
		return;
	
	char packet[100];
	CHeader *head =(CHeader*) packet;
	head->cType = START_GAME;
	head->iBuffLen = 0;
	
	send(m_Socket, packet, sizeof(CHeader), 0);
}



////////////////////////////////////////////////////////////////////////////////////////////
////   SEND BEGIN GAME NOTIFY ///////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
void CConnection::SendBeginGameNotify()
{
	CHeader head;
	head.cType = START;
	head.iBuffLen = 0;

	if (!m_bInitialized) return;
	if (!m_bConnected)   return;

	for (int i = 0; i < m_iClientCount; i++)
	{
		send(*m_pClients[i], (char *) &head, sizeof(CHeader), 0);
	}
}


////////////////////////////////////////////////////////////////////////////////////////////
////   SEND CLIENT ID  - server sends to client that send this request its ID number ///////
////////////////////////////////////////////////////////////////////////////////////////////
void CConnection::SendClientID(int id)
{
	if (m_ConnType != SERVER) return;
	if (!m_bInitialized)      return;
	if (!m_bConnected)        return;

	
	CHeader head;
	head.cType = CLIENT_ID;
	head.iBuffLen = sizeof(int);
	
	int iID = id;
	char * temp = new char[sizeof(int) + sizeof(head)];
	memcpy(&temp[0], &head, sizeof(head));
	memcpy(&temp[sizeof(head)], &iID, sizeof(int));
	
	if (id >= m_iClientCount) return;

	send(*m_pClients[iID], temp, sizeof(int) + sizeof(head), 0);
	
	delete temp;
}


////////////////////////////////////////////////////////////////////////////////////////////
////  GET CLIENT ID  - client wants to know its ID /////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
void CConnection::GetClientID()
{
	CHeader head;
	head.cType = CLIENT_ID;
	head.iBuffLen = 0;
	
	if (m_ConnType != CLIENT) return;
	if (!m_bInitialized)      return;
	if (!m_bConnected)        return;

	send(m_Socket, (char *) &head, sizeof(CHeader), 0);
}


////////////////////////////////////////////////////////////////////////////////////////////
////   GET LAST ERROR  ///////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
int CConnection::GetLastError()
{
	return m_iLastError;
}


////////////////////////////////////////////////////////////////////////////////////////////
////   CONVERT TO DOUBLE  //////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
bool CConnection::ConvertToDW(char *address, DWORD *IP)
{
	char szIP[100];
	strcpy(szIP, address);
	
	m_iLastError = 0;
	int i = 0, ex_i = 0;
	int count = 0;
	for (i = 0; i < abs(strlen(szIP)); i++)
	{
		if (szIP[i] == '.')
			count++; 
	}
	if (count != 3)
	{
		m_iLastError = ERR_INVALID_IP_ADDRESS;
		return false; 
	}
	
	i = 0;
	char temp[20];
	while (szIP[i] != '.') i++;
	strncpy(temp, &szIP[ex_i], i - ex_i);
	temp[i - ex_i] = 0;
	unsigned char value = atoi(temp);
	memcpy(IP, &value, 1);
	
	i++;
	ex_i = i; 
	while (szIP[i] != '.') i++;
	strncpy(temp, &szIP[ex_i], i - ex_i);
	temp[i - ex_i] = 0;
	value = atoi(temp);
	memcpy(((char *)IP) + 1, &value, 1);
	
	i++;
	ex_i = i;
	while (szIP[i] != '.') i++;
	strncpy(temp, &szIP[ex_i], i - ex_i);
	temp[i - ex_i] = 0;
	value = atoi(temp);
	memcpy(((char *)IP) + 2, &value, 1);
	
	ex_i = i + 1;
	i = strlen(szIP);	
	strncpy(temp, &szIP[ex_i], i - ex_i);
	temp[i - ex_i] = 0;
	value = atoi(temp);
	memcpy(((char *)IP) + 3, &value, 1);
	
	return true;
}

////////////////////////////////////////////////////////////////////////////////////////////
////   CREATE MY WINDOW  ///////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
bool CConnection::CreateMyWindow()
{
	WNDCLASS    WC;
	
	HINSTANCE hInst = ::GetModuleHandle(NULL);
	
	WC.style = 0;
	WC.lpfnWndProc = WinProc;
	WC.cbClsExtra = 0;
	WC.cbWndExtra = 0;
	WC.hInstance = hInst;
	WC.hIcon = NULL;
	WC.hCursor = NULL;
	WC.hbrBackground =(HBRUSH) GetStockObject(LTGRAY_BRUSH);
	WC.lpszMenuName = NULL;
	WC.lpszClassName = CLASSNAME;
	ATOM ret = RegisterClass(&WC);
	m_hWnd = CreateWindowEx(0, CLASSNAME , "", 0, 0, 0, 0, 0 , NULL, NULL, hInst, NULL);
	if (!m_hWnd)
	{
		m_iLastError = GetLastError();
		return false; 
	}

	SetTimer(m_hWnd, 0, FLOOD_CHECK, NULL);

	return true;
}


void CConnection::LogBufferInfo()
{
	if (!m_bInitialized)      return;
	if (!m_bConnected)        return;

	for (int i=0; i < m_iClientCount; i++)
	{
		if (m_pBuffers[i]->m_iTotalLen > FLOOD_SIZE)
		{
			char buff[100];
			sprintf(buff, "A large amount of data from client %d is unread : %d bytes", i, m_pBuffers[i]->m_iTotalLen);
			if (m_pLog) m_pLog->print(buff);
		}
	}


}



////////////////////////////////////////////////////////////////////////////////////////////
////   GOT MESSAGE   /////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
LRESULT CConnection::GotMessage(HWND hWnd, UINT message, UINT wParam, LONG lParam)
{
	if (message == WSA_EVENT)
	{
		switch (LOWORD(lParam)) 
		{
			case FD_READ:
				TryToReceive(wParam);
				break;
				
			case FD_ACCEPT: 
				if (m_ConnType == CLIENT)
					return FALSE;
				Accept();
				break;
				
			case FD_CLOSE: 
				int id = pConnection->GetClientID(wParam);
				// ASSERT (id != -1);
				pConnection->CloseSocket(id);
				break;
		}
	}
	else if (message == WM_TIMER)	LogBufferInfo();		
	
	return 1;
}

////////////////////////////////////////////////////////////////////////////////////////////
////  TRY TO RECEIVE  ////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
void CConnection::TryToReceive(SOCKET socket)
{
	if (!m_bConnected)
		return;
	
	// poiscimo kateri je index socketa, ki naj bi sprejel paket
	int id = GetClientID(socket);
	// ASSERT (id != -1);
	
	int ret = recv(socket, recv_buff, MAX_RECV, 0);
	if (ret == SOCKET_ERROR)
	{
		int err = WSAGetLastError();
		
		if (err == WSAEWOULDBLOCK)
			return;	// verjetno se ni podatkov
		if ((err == WSAENOTCONN) &&(err == WSAESHUTDOWN))
		{
			m_bConnected = false;
			return; 
		}
		
		// ce je bil error samo v tem, da je buffer premajhen, potem veckrat poskusamo sprejeti podatke
		if (err == WSAEMSGSIZE)
		{
			while (err != WSAEMSGSIZE)
			{
				AppendData(id, recv_buff, ret);	
				
				ret = recv(socket, recv_buff, MAX_RECV, 0);
				if (ret == SOCKET_ERROR)
					err = WSAGetLastError();
				else err = 0;
			}
		}
	}
	else AppendData(id, recv_buff, ret);	
}


////////////////////////////////////////////////////////////////////////////////////////////
////                     ///////////////////////////////////////////////////////////////////
//// CLASS  :   CBUFFER  ///////////////////////////////////////////////////////////////////
////                     ///////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////

CBuffer::CBuffer()
{
	m_iTotalLen = 0;
	m_iAllocated= 0;
	m_pData = NULL;
}

CBuffer::~CBuffer()
{
	DestroyData();
}


////////////////////////////////////////////////////////////////////////////////////////////
////   APPEND DATA  ////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
void CBuffer::AppendData(char *buff, int size)
{
	if (m_iTotalLen + size > m_iAllocated)
	{	
		if (m_iAllocated == 0)
		{
			CHeader head;
			memcpy(&head, buff, sizeof(CHeader));
			
			memcpy(recv_buff, &head, sizeof(CHeader));
		}
		int alloc = max(size , BLOCK_SIZE);
		m_pData =(char *) realloc(m_pData, m_iAllocated + alloc);
		m_iAllocated += alloc;
	}
	
	memcpy(&m_pData[m_iTotalLen], buff, size); // dodamo novi paket
	m_iTotalLen  += size;					
}

////////////////////////////////////////////////////////////////////////////////////////////
////  DISPOSE BUFFER  //////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
void CBuffer::DisposeBuffer()
{
	if (!m_pData)
		return;
	if (m_iTotalLen < sizeof(CHeader))
		return;
	
	CHeader *head =(CHeader *) m_pData;
	int size = head->iBuffLen;
	
	char *off = m_pData + head->iBuffLen + sizeof(CHeader);
	
	m_iTotalLen = m_iTotalLen - head->iBuffLen - sizeof(CHeader);
	if (m_iTotalLen == 0) 
	{
		delete m_pData;
		m_pData = NULL;
		m_iAllocated = 0;
		return;
	}
	
	char * data = new char[m_iTotalLen +(BLOCK_SIZE -(m_iTotalLen % BLOCK_SIZE))];
	memcpy(data, off, m_iTotalLen);
	m_iAllocated = m_iTotalLen + BLOCK_SIZE;
	
	delete m_pData;
	m_pData	 = data;
}

////////////////////////////////////////////////////////////////////////////////////////////
////   DESTROY DATA  ///////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
void CBuffer::DestroyData()
{
	if (m_pData)
		delete m_pData;
	
	m_pData = NULL;
	m_iTotalLen = 0;
	m_iAllocated = 0;
}


////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
//// THREAD PROCEDURES + TIMER HANDLING ////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////


LRESULT CALLBACK WinProc(HWND hWnd, UINT message, UINT wParam, LONG lParam) 
{
	if ((message == WSA_EVENT) || (message == WM_TIMER))
		return pConnection->GotMessage(hWnd, message, wParam, lParam);
	
	return DefWindowProc(hWnd, message, wParam, lParam);
}



