
// $Id: Connection.h,v 1.5 2001/04/08 15:40:52 mitja Exp $

///////////////////////////////////////////////////////////////////////
//
// Connection.h: interface for the CConnection class.
//	
//	VERSION 2.2
//
//	this class is used to create a client server connection between
//	a server and possible multiple clients.
//	Don't forget to link Ws2_32.lib library 
//
//	(C) Leban Gregor, Dec 2000
//
//////////////////////////////////////////////////////////////////////

#ifndef __CONNECTION__
#define __CONNECTION__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <Winsock2.h>
#include "Log.hpp"

// when we Initialize the CConnection class we must tell, wether we'll be a client or a server
enum conn_type			
{
	SERVER,	
	CLIENT
};


enum recv_type
{
	START_GAME,	// en od clientov poslje serverju ukaz za zacetek igre
	MAP,        // server poslje vsem klientom paket, ki vsebuje mapo
	START,      // server poslje vsem klentom notify, da je bila mapa poslana vsem in da se bo igra zacela
	DATA,        // obicajni paketi, ki se posiljajo med igro
	CLIENT_ID	// klient je poslal prosnjo po tem, da izve svoj ID
};

//
// Every packet we receive has this header
//
struct CHeader                // packet header
{
	int iBuffLen;             // packet length
	char cType;               // content type ( not used yet )
};

//
// class to store packets received from different clients
//
class CBuffer
{
public:
	CBuffer();
	~CBuffer();
	
	void AppendData(char *buff, int size);  // append new packet to the queue
	void DisposeBuffer();                   // destroy the content of the oldest packet
	void DestroyData();                     // destroy all the data
	
	int	m_iTotalLen;                        // used buffer size
	int m_iAllocated;                       // allocated buffer size
	
	char * m_pData;                         // pointer to data
};


////////////////////////////////////////
// CConnection - main class ////////////
////////////////////////////////////////

class CConnection  
{
public:
	void GetClientID();
	void SendClientID (int id);
	CConnection();          // constructor
	virtual ~CConnection(); // destructor
	
	
	// server and client functions : 
	bool Initialize(conn_type type);                    // call before using any other functions
	void Finish();                                      // cleanup, close all sockets
	
	int  Send(char *buff, int size);                    // send buffer buff of size size
	int  Send(int i, char *buff, int size);
	int  Receive(int clientID, void **buff, int *size); // when IsNewDataAvailable returns true, you can receive a pointer to new data
	bool IsNewDataAvailable(int *clientID);             // true if we received new data - if true, use this clientID return value as a parameter when calling Receive ()
	void DisposeBuffer(int id);                         // MUST CALL when we finish using buff from Receive !!
	int  GetLastError();                                // return the last error that happened (0 if no error)
	
	// server functions :
	bool Listen();                                      // if we are a server we have to listen for new connection requests
	bool SendMap(char * buff, int size);                // send a map to every client
	int  GetClientCount();                              // return number of connected clients
	void SendBeginGameNotify();
	
	// client functions :
	bool Call(char *address);
	void StartGame();
	
	
	/// NOT FOR USE !! needed to be public because of WinProc...
	LRESULT GotMessage(HWND hWnd, UINT message, UINT wParam, LONG lParam);
	
private:

	void	LogBufferInfo();
	int     GetClientID(SOCKET socket);
	void    AppendData(int clientID, char *buff, int size);
	bool    Accept();							
	void    CloseSocket(int clientID);
	
	bool    CreateMyWindow();                   // create an invisible window to receive WSA_EVENT messages
	bool    ConvertToDW(char *szIP, DWORD *IP); // convert IP address in string into a double word
	void    TryToReceive(SOCKET socket);        // try to receive new data from socket

	Log		*m_pLog;

	bool    m_bInitialized;                     // have we initialized ?
	bool    m_bSocketCreated;                   // did the initialization create a socket
	
	char    m_szMyCompName[256];                // name of my computer (maybe for future use)
	int     m_iLastError;                       // if a function fails, it tells us whats the reason
	
	bool    m_bConnected;
	int     m_ConnType;
	
	SOCKET  m_Socket;                           // if we are a client, then this is a socket used to communicate
	SOCKET  m_ListenSocket;
	
	SOCKET  **m_pClients;                       // array of client sockets
	CBuffer **m_pBuffers;                       // array of buffers for each client
	
	int     m_iClientCount;								
	int     m_iCurrentClient;                   // not really important
	
	HWND    m_hWnd;                             // handle to invisible window that receives the messages
};

#endif // __CONNECTION__


/* HISTORY 

17.11.2000	CConnection, member funkcije

19.11.2000	CBuffer
	
3.12.2000	CConnection - popravilo nekaterih funkcij

5.12.2000	spremenil CBuffer in njegova uporaba, da deluje lepo pri vec klientih	

17.12.2000 	dodal StartGame, SendMap, GetClientCount. Receive vrne tip paketa : MAP, START_GAME, DATA, START

19.12.2000 - popravil stvari tako da dela lepo tako na Win32 application kot tudi na console application

3.3. 2001 - dodal posiljanje ClientIDja

7.3. 2001 - dodal logiranje stanja dolzine sprejetega bufferja

*/
