
// $Id: Console.hpp,v 1.9 2001/04/08 15:40:52 mitja Exp $

/* Console class
 *
 * Main console which handles all the mouse and key events, listenes for communication updates,
 * updates the whole screen by dirty rectangles, scrolls the selected vieports, issues commands...
 * Created by: Matija Jekovec
 * Created on: 22.11.2000
 */

#ifndef __CONSOLE__
#define __CONSOLE__

class Console;

#include "TUIGraphics.hpp"
#include "Viewport.hpp"
#include "CommandPanel.hpp"
#include "SDL.h"
#include "List.hpp"

const int    CONSOLE_SCREEN_WIDTH  = 640;
const int    CONSOLE_SCREEN_HEIGHT = 480;
const int    CONSOLE_BIT_DEPTH     = 16;
const int    CONSOLE_VIDEO_FLAGS   = SDL_SWSURFACE/*|SDL_FULLSCREEN*/;
const Uint32 SCROLL_SPEED          = 60; // speed of scrolling
const int    SCROLL_STEP           = 35; // default scroll step
const int    SCROLL_BORDER         = 20; // border of the screen where scroll is initiated
const float  SCROLL_DIAGONAL_FACTOR= (float)0.2;	//percent of the screen border to be used for diagonal scrolling
const int    NO_ACTIVE_VIEWPORT    = -1; // no viewport is currently active
const float  CONSOLE_ZOOM_STEP     = 1.5;


int	InterceptEvents(const SDL_Event *event); // intercepts events
Uint32 OnScrollTimer(Uint32 interval);       // initializes scrolling


class Console
{
private:	
	SDL_Surface       *screen;      // this is it, representation of the screen
	EnvironmentClient *environment; // pointer to the environment
		
	int  activeViewport;
	int  numViewports;
	bool drag;
	TUI_pointStruct startDragPoint;
	
	void ArrangeViewports(int numViewports);
	
public:
	int			shiftState; // stores the current state of the Shift modifiers
	bool		displayWaypoints;
	bool		isBeingScrolled;		//auto scroll is in action
	int	 scrollStepsLeft;			//steps left for auto scroll on soldier
	bool autoScrollEnabled;
	Direction currentDirection;

	int			groupID;

	List *selected;
	List **preselected;
	List *waypoints;
	List *areasOfDanger;

	Direction	GetScrollingDirection(int x, int y, int sx, int sy, int w, int h);
	Viewport  **viewports;      // array of viewports created at construction	
	CommandPanel *commandPanel; // command panel which features buttons, logo ...
	void   (*OnWorldMouseDown)(float x, float y, Uint8 button); // is called when WorldMouseButton is called
	Uint32 scrollSpeed;         // interval of the scroll timer
	int    scrollStep;          // step of one scroll	
	
	Console(int numViewports, EnvironmentClient *env); // creates and arranges all viewports, initializes video, timer, cp...
	~Console(); // destroys timer, screen, command panel

	void RefreshOverlays(); // draws overlays for all viewports
	void WorldMouseDown(Viewport *sender, float x, float y, Uint8 button); // called by intercept events	
	void WorldMouseUp(Viewport *sender, float x, float y, Uint8 button);   // called by intercept events	

	void DefineGroup(int n); // defines a new group of soldiers
	void SelectGroup(int n); // selects a defined group of soldiers

	SDL_Surface *GetScreen();                   // returns screen
	void SetActiveViewport(int index);          // get and set active viewports
	void SetActiveViewport(Viewport *viewport); // get and set active viewports	
	int  GetActiveViewport();	
	int  GetNumViewports();
	EnvironmentClient *GetEnvironment();			
};

#endif // __CONSOLE__
