
// $Id: Environment.cpp,v 1.26 2001-04-16 17:59:18+02 damir Exp damir $

/* Environment class implementation
 *
 * Created by: Damir Arh
 * Created on: 19.11.2000
 */

#include "stdafx.h"

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "Environment.hpp"
#include "Bullet.hpp"
#include "Soldier.hpp"
#include "List.hpp"
#include "Auxiliary.hpp"
#include "TextDisplay.hpp"
#include "vectordir.h"
#include "PlayerData.hpp"
#include "HeuristicsMDM.hpp"
#include "MapAttributesMDM.hpp"

#include "Group.hpp"

#include "AutomatonDemo.hpp"
#include "AutomatonMDM.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


extern TextDisplay *mainDisplay;
extern DrawStruct draw;
// calculates the index in the matrix array
#define INDEX(X, Y)(X + 1) + (Y + 1)*(maxSx + 2)

// definitions for loading the map file
// (this part should probably be in a separate header)

#define ENV_NAME_LENGTH 32
#define ENV_TEXTURE_FILE_LENGTH ENV_NAME_LENGTH

struct MapHeader 
{
	float version;                              // map format version
	int   width, height;                         // map dimensions
	char  name[ENV_NAME_LENGTH];                // map name (?)
	char  textureFile[ENV_TEXTURE_FILE_LENGTH]; // name of the texture file
	int   dataOffset;                           // data starting position, currently sizeof(MapHeader)
};

struct MapSquare
{
	unsigned char passability;        // terrain passability
	char          textureId;          // textureID for the square
	char          startingPosition;   // index of the side which can start at this position
};


/* Environment()
 * 
 * This constructor generates a random map with the size of
 * 50 x 50 squares.
 */
Environment::Environment() 
{
	Environment(50, 50);
}


/* Environment(int width, int height)
 *
 * Parameters:
 *    width  : width of the map
 *    height : height of the map
 *
 * This constructor generates a random map with given size. Two
 * soldiers are put on the map at random positions.
 */
Environment::Environment(int width, int height) 
{
	int i;
	Vector start;
	
	// internal values initialization
	time = 0.0f;
	turnInterval = 0.05f;
	squareBorder = 1.0f;
	
	// map size
	maxSx = width;
	maxSy = height;
	
	// random map
	matrix = new Square*[(maxSx + 2)*(maxSy + 2)];
	for (int j = 0; j < height; j++)
		for (int i = 0; i < width; i++)
			matrix[INDEX(i, j)] = new Square((int)floor(RANDOM(1) + 0.5));
		
		// create wall below and above map
		for (i=-1; i < width + 1; i++)
		{
			matrix[INDEX(i, - 1)] = new Square(0);
			matrix[INDEX(i, height)] = new Square(0);
		}
		// create wall to the left and to the right of the map
		for (j = 0; j < height; j++)
		{
			matrix[INDEX(-1, j)] = new Square(0);
			matrix[INDEX(width, j)] = new Square(0);
		}
		
		placeSoldiers(4);
}


/* Environment(char *filename)
 *
 * Parameters:
 *    filename : name of the mapfile
 *
 * This constructor generates a map with data from the mapfile
 * with the given filename
 */
Environment::Environment(char *filename) 
{
	FILE *file;
	MapHeader head;
	MapSquare square;


	// internal values initialization
	time = 0.0f;
	turnInterval = 0.05f;
	squareBorder = 1.0f;
	
	file = fopen(filename, "r");
	
	if (file == NULL)
		exit(1);
	//		throw EErrorLoadingEnvironment();
	
	// read header
	fread(&head, sizeof(head), 1, file);
	
	// TODO: check version in the head
	
	// get map size and generate matrix
	maxSx = head.width;
	maxSy = head.height;
	matrix = new Square*[(maxSx + 2)*(maxSy + 2)];
	
	for (int j = 0; j < head.height; j++)
	{
		for (int i = 0; i < head.width; i++)
		{			
			fread(&square, sizeof(square), 1, file);
			matrix[INDEX(i, j)] = new Square(square.passability);
			// TODO: get info on soldier positioning and use it properly
		}
	}

	// create wall below and above map
	for (int i=-1; i < head.width + 1; i++)
	{
		matrix[INDEX(i, - 1)] = new Square(0);
		matrix[INDEX(i, head.height)] = new Square(0);
	}
	// create wall to the left and to the right of the map
	for (j = 0; j < head.height; j++)
	{
		matrix[INDEX(-1, j)] = new Square(0);
		matrix[INDEX(head.width, j)] = new Square(0);
	}
	
	fclose(file);
//	placeSoldiers(8);

	Vector start, delta;
	Soldier *sol;
	int ai = 0;
	start.x = 4.5; start.y = 1.5;
	sol = new Soldier(10, ai, start * squareBorder, 0);
	soldiers.add(sol);
	plData[ai].friendlySoldiers.add(sol);
	start.x = 5.5; start.y = 1.5;
	sol = new Soldier(11, ai, start * squareBorder, 0);
	soldiers.add(sol);
	plData[ai].friendlySoldiers.add(sol);
	start.x = 6.5; start.y = 1.5;
	sol = new Soldier(12, ai, start * squareBorder, 0);
	soldiers.add(sol);
	plData[ai].friendlySoldiers.add(sol);
	start.x = 7.5; start.y = 1.5;
	sol = new Soldier(13, ai, start * squareBorder, 0);
	soldiers.add(sol);
	plData[ai].friendlySoldiers.add(sol);

	ai = 1;
	start.x = 20.5; start.y = 2.5;
	sol = new Soldier(20, ai, start * squareBorder, 0);
	soldiers.add(sol);
	plData[ai].friendlySoldiers.add(sol);
	start.x = 21.5; start.y = 2.5;
	sol = new Soldier(21, ai, start * squareBorder, 0);
	soldiers.add(sol);
	plData[ai].friendlySoldiers.add(sol);
	start.x = 20.5; start.y = 3.5;
	sol = new Soldier(22, ai, start * squareBorder, 0);
	soldiers.add(sol);
	plData[ai].friendlySoldiers.add(sol);
	start.x = 21.5; start.y = 3.5;
	sol = new Soldier(23, ai, start * squareBorder, 0);
	soldiers.add(sol);
	plData[ai].friendlySoldiers.add(sol);
	
	plData[0].visGraph = new VisGraph(this, new HeuristicsMDM (&plData[0].areaod));
	plData[1].visGraph = new VisGraph(this, NULL);

	plData[0].attr = new MapAttributesMDM(this);
	plData[1].attr = NULL;
	
	m_CollAvoid.CreateVectMap(matrix, getMaxSx(), getMaxSy());

	Vector zero (0, 0); // MITJA
	ghostSoldier=new Soldier (-1, -1, zero, 0); // MITJA
}


/* ~Environment()
 *
 * Destroys complete environment and release all related memory.
 */
Environment::~Environment() 
{
	// free map
	for (int i = 0; i <(maxSx + 2) *(maxSy + 2); i++)
		delete matrix[i];
	delete matrix;
	// free objects in the lists
//	soldiers.deleteAll(List::SOLDIER); // MITJA: to sem prestavil v PlayerData, ker se sicer najprej brisejo vojaki in nato skupine, kar povzroca tezave.
	bullets.deleteAll(List::BULLET);
//	delete visGraph; // MITJA
	delete ghostSoldier; // MITJA
}


/* placeSoldiers(int n)
 *
 * Parameters:
 *    n : number of soldiers to generate
 *
 * Generates a requested number of soldiers at random positions on the
 * map so that they don't overlap or get stuck in the wall.
 *
 * Current implementation could loop forever if too many soldiers are requested
 * for the available free space on the map but this shouldn't be a problem as
 * this routine will have to change anyhow to take into account the predefined
 * positioning in the map.
 */
void Environment::placeSoldiers(int n) 
{
	Soldier *sol;
	Vector *used = new Vector[n];
	Vector start, delta;
	int ai = 0, found;
	
	for (int i = 0; i < n; i++)
	{
		// generate requested number of soldiers
		found = false;
		while (!found) 
		{ // repeat until generated coordinates are suitable
			// find a passable square
			start.x =(float)floor(RANDOM(maxSx) + 0.5);
			start.y =(float)floor(RANDOM(maxSy) + 0.5);
			
			while (!matrix[INDEX((int)start.x, (int)start.y)]->getPassability()) 
			{
				start.x =(float)floor(RANDOM(maxSx) + 0.5);
				start.y =(float)floor(RANDOM(maxSy) + 0.5);
			}
			found = true;
			// check if there is already a soldier there
			for (int j = 0; j < i; j++)
				if (start == used[j])
					found = false;
		}
		// generate a new soldier
		delta.x = 0.5f;
		delta.y = 0.5f;
		start = start + delta;
		sol = new Soldier(i, ai, start * squareBorder, 0);
		soldiers.add(sol);
		plData[ai].friendlySoldiers.add(sol);
		used[i] = start; // store his position
		ai = 1 - ai;     // change the ai
	}
	delete used;
}


/* update()
 *
 * Does all the calculation for the current turn by updating the
 * time and calling update() method of all objects in the
 * environment.
 *
 * The soldier collision check is a big cludge. It actually checks if
 * soldier ending positions collide with another soldier or wall and
 * resets his position if this happens. It will have to be completely
 * rewritten if correct collision check is to be implemented.
 *
 * Due to difference between bullet and soldier speed the soldier
 * movement again isn't interpolated in the bullet collision check.
 * The routine only checks if the bullet hits the soldier ending
 * position.
 */
void Environment::update() 
{
	Soldier *soldier, *hitSoldier;
	Bullet *bullet;
	Vector oldCoords, newCoords, *hitCoords;

	Group *temp; //\\// -->
	for (int id = 0; id<2; id++)
	{
		temp=(Group *)(plData[id].groups.getFirst ());
		while (temp!=NULL)
		{
			temp->update ();
			temp=(Group *)(plData[id].groups.getNext ());
		} //\\// <--
	}

	Refresh();	// tu so ze narisani novi vektorji

//	SleepEx(200, TRUE);

	// time increases
	time += turnInterval;

	// get new bullets
	soldier =(Soldier *) soldiers.getFirst();
	while (soldier) 
	{
		if (soldier->getHealth()) // only living soldiers have shot
			if (bullet = soldier->getBullet())
				bullets.add(bullet);
			soldier =(Soldier *) soldiers.getNext();
	}
	
	// move the bullets
	bullet =(Bullet *) bullets.getFirst();
	while (bullet) 
	{
		int collision = false;

		// get starting and ending coordinates
		oldCoords = bullet->getCoords();
		newCoords = bullet->getNewCoords(time);
		
		// check colisions with walls
		if (hitCoords = bulletHitWall(bullet, oldCoords, newCoords))
		{
			collision = true;
			newCoords = *hitCoords;
			delete hitCoords;
		}
		
		// check collisions with soldiers
		if (hitSoldier = bulletHitSoldier(bullet, oldCoords, newCoords))
		{
			collision = true;
		}
		
		// destroy bullet if it hit something
		if (collision)
		{
			bullets.remove();
			delete bullet;
		}
		
		// get the next bullet
		bullet =(Bullet *) bullets.getNext();
	}
	
	// create temporary arrays for storing soldier movement details
	int numSoldiers = soldiers.getNumItems();
	Vector *oldC = new Vector[numSoldiers];
	Vector *newC = new Vector[numSoldiers];
	Soldier **pSoldier = new Soldier*[numSoldiers + 1];
	
	// move the soldiers
	int i = 0;
	soldier =(Soldier *) soldiers.getFirst();
	while (soldier) 
	{
		if (soldier->getHealth())
		{
			// perform soldier update only if he's still alive
			// fill in the arrays
			oldC[i] = soldier->getCoords();
			newC[i] = soldier->getNewCoords(time);
			pSoldier[i++] = soldier;
		}
		
		// get next soldier
		soldier =(Soldier *) soldiers.getNext();
	}
	for (; i <= numSoldiers; i++) // array guard
		pSoldier[i] = NULL;
	
	// perform the actual collision check
	for (i = 0; pSoldier[i]; i++)
	{
		// check soldier to soldier collision
		for (int j = i + 1; pSoldier[j]; j++)
		{
/*
			if (circleIntersectsCircle(newC[i], pSoldier[i]->getRadius(), newC[j], pSoldier[j]->getRadius()))
			{
				pSoldier[i]->stop(oldC[i]);
				pSoldier[j]->stop(oldC[j]);
			}
*/
			if (circleIntersectsCircle(newC[i], pSoldier[i]->getRadius(), newC[j], pSoldier[j]->getRadius()))
			{
				if (!circleIntersectsCircle(newC[i], pSoldier[i]->getRadius(), oldC[j], pSoldier[j]->getRadius()))
				{	
					pSoldier[j]->stop(oldC[j]);		
				}
				else if (!circleIntersectsCircle(oldC[i], pSoldier[i]->getRadius(), newC[j], pSoldier[j]->getRadius()))
				{
					pSoldier[i]->stop(oldC[i]);
				}
				else
				{
					pSoldier[i]->stop(oldC[i]);
					pSoldier[j]->stop(oldC[j]);
				}
			}

		}
		if (hitCoords = soldierHitWall(pSoldier[i], oldC[i], newC[i]))
		{
			delete hitCoords;
			pSoldier[i]->stop(oldC[i]);
		}
	}

	// check for visible soldiers
	plData[0].enemySoldiers.empty();
	plData[1].enemySoldiers.empty();
	for (i=0; pSoldier[i]; i++)
	{
		int id = pSoldier[i]->getAiID();
		if (pSoldier[i]->getHealth())
		{
			List *enemies = getVisibleEnemies(pSoldier[i]);
			plData[id].enemySoldiers.merge(enemies);
			delete enemies;
		}
	}
	
	// delete temporary arrays
	delete oldC;
	delete newC;
	delete pSoldier;
}


/* getSquareIndex(float coord)
 *
 * Parameters:
 *    coord : coordinate to get sqare index for
 *
 * Result:
 *    int : index of the square containing the coordinate
 *
 * Returns the index of the square corresponding to the given coordinate.
 * Works also for coordinates outside the matrix.
 */
int Environment::getSquareIndex(float coord) 
{
	return (int) floor(coord / squareBorder);
}


/* bulletHitWall(Bullet *bullet, Vector oldC, Vector newC)
 *
 * Parameters:
 *    bullet : bullet to check collision for
 *    oldC   : starting position of the bullet
 *    newC   : ending position for the bullet
 *
 * Result:
 *    Vector * : pointer to a vector containing hit coordinates
 *
 * Calculates bullet - wall collision. Returns coordinates of the collision
 * or NULL if no collision takes place. You should free the vector once you
 * don't need it anymore.
 *
 * The routine could maybe be improved if the collision check would be
 * performed along the flight path and not separately for horizontal and
 * vertical walls.
 */
Vector *Environment::bulletHitWall(Bullet *bullet, Vector oldC, Vector newC) 
{
	int i, hit = false;
	float k, n, y, x;
	Vector *finalCx = NULL, *finalCy = NULL;
	
	// calculate hits with vertical walls
	if (oldC.x != newC.x)
	{
		// get y = f(x) function
		k =(newC.y - oldC.y) /(newC.x - oldC.x);
		n = oldC.y - k * oldC.x;
		// if x coordinate increases
		if (oldC.x < newC.x)
			// check all possible vertical walls in increasing direction
			for (i = getSquareIndex(oldC.x) + 1; i <= getSquareIndex(newC.x); i++)
			{
				y = i * squareBorder * k + n;
				if ((getSquareIndex(y)>maxSy) ||(getSquareIndex(y) < -1))
					break;
				// actually hits a wall
				if (!matrix[INDEX(i, getSquareIndex(y))]->getPassability())
				{
					finalCx = new Vector(i*squareBorder, y);
					hit = true;
					break;
				}
			}
			else // x coordinate decreases
				// check all possible vertical walls in decreasing direction
				for (i = getSquareIndex(oldC.x) - 1; i >= getSquareIndex(newC.x); i--)
				{
					y =(i + 1) * squareBorder * k + n;
					if ((getSquareIndex(y)>maxSy) ||(getSquareIndex(y) < -1))
						break;
					// actually hits a wall
					if (!matrix[INDEX(i, getSquareIndex(y))]->getPassability())
					{
						finalCx = new Vector(i*squareBorder, y);
						hit = true;
						break;
					}
				}
	}
	
	// calculate hits with horizontal walls
	if (oldC.y != newC.y)
	{
		// get x = f(y) function
		k =(newC.x - oldC.x) /(newC.y - oldC.y);
		n = oldC.x - k * oldC.y;
		// if y coordinate increases
		if (oldC.y < newC.y)
			// check all possible horizontal walls in increasing direction
			for (i = getSquareIndex(oldC.y) + 1; i <= getSquareIndex(newC.y); i++)
			{
				x = i * squareBorder * k + n;
				if ((getSquareIndex(x)>maxSx) ||(getSquareIndex(x) < -1))
					break;
				// actually hits a wall
				if (!matrix[INDEX(getSquareIndex(x), i)]->getPassability())
				{
					finalCy = new Vector(x, i*squareBorder);
					hit = true;
					break;
				}
			}
			else // y coordinate decreases
				// check all possible horizontal walls in decreasing direction
				for (i = getSquareIndex(oldC.y) - 1; i >= getSquareIndex(newC.y); i--)
				{
					x =(i + 1) * squareBorder * k + n;
					if ((getSquareIndex(x)>maxSx) ||(getSquareIndex(x) < -1))
						break;
					// actually hits a wall
					if (!matrix[INDEX(getSquareIndex(x), i)]->getPassability())
					{
						finalCy = new Vector(x, i*squareBorder);
						hit = true;
						break;
					}
				}
	}
	
	// only hits horizontal or vertical wall
	if (!finalCx)
		return finalCy;
	if (!finalCy)
		return finalCx;
	// hits both walls; which one is closer
	Vector diffX = *finalCx - oldC;
	Vector diffY = *finalCy - oldC;
	if (diffX.lengthSquared() < diffY.lengthSquared())
	{
		delete finalCy;
		return finalCx;
	}
	else 
	{
		delete finalCx;
		return finalCy;
	}
}


/* bulletHitSoldier(Bullet *bullet, Vector oldC, Vector newC)
 *
 * Parameters:
 *    bullet : bullet to check collision for
 *    oldC   : starting position of the bullet
 *    newC   : ending position for the bullet
 *
 * Result:
 *    Soldier * : pointer to the soldier which was hit
 *
 * Calculates bullet - soldier collision. Returns the soldier which was hit. The
 * hit has not yet been recorded in the soldier. The caller should take care
 * of that if needed. Do not free the obtained pointer!
 * 
 * The routine uses approximates to determine which soldier is closer on the
 * bullet flight path and so which one was actually hit. It calculates the
 * distance between the bullet starting point and the soldier center instead
 * of the hitpoint. Must be changed if this could cause wrong calculations.
 */
Soldier *Environment::bulletHitSoldier(Bullet *bullet, Vector oldC, Vector newC) 
{
	float hit = 0, actHit = 0;
	Soldier *soldier, *hitSoldier = NULL;
	Vector diff, diffHit;
	
	soldier =(Soldier *) soldiers.getFirst();
	while (soldier) 
	{
		if (soldier->getHealth())
			if (hit = circleHitByLineSegment(soldier->getCoords(), soldier->getHitRadius(), oldC, newC))
				if (hitSoldier)
				{
					// did we already hit another soldier
					diff = soldier->getCoords() - oldC;
					diffHit = hitSoldier->getCoords() - oldC;
					// approximation of the distance (read above comment)
					if (diff.lengthSquared() < diffHit.lengthSquared())
					{
						hitSoldier = soldier;
						actHit = hit;
					}
				}
				else
				{
					hitSoldier = soldier;
					actHit = hit;
				}
				soldier =(Soldier *) soldiers.getNext();
	}
	if (hitSoldier)
		hitSoldier->setIsShot(actHit); // actually hurt the soldier
	return hitSoldier;
}


/* soldierHitWall(Soldier *soldier, Vector oldC, Vector newC)
 *
 * Parameters:
 *    soldier : soldier to check collision for
 *    oldC    : starting position of the soldier
 *    newC    : ending position for the soldier
 *
 * Result:
 *    Vector * : pointer to a vector containing hit coordinates
 *
 * Calculates soldier - wall collision. Returns coordinates of the collision
 * or NULL if no collision takes place. You should free the vector once you
 * don't need it anymore.
 *
 * The routine is really primitive at the moment. It just checks 9 points of
 * interest for collision with a wall(center and k*PI/4 positions on the
 * bounding circle. Returned coordinates are just starting coordinates, so no
 * moving closer to the wall at the moment. There's certainly room for massive
 * improvement and even optimization here.
 */
Vector *Environment::soldierHitWall(Soldier *soldier, Vector oldC, Vector newC) 
{
	float r = soldier->getRadius();
	float d = r *(float) sqrt(2) / 2; // coordinate difference for 45 angle
	if (!matrix[INDEX(getSquareIndex(newC.x), getSquareIndex(newC.y))]->getPassability())
		return new Vector(oldC.x, oldC.y);
	if (!matrix[INDEX(getSquareIndex(newC.x), getSquareIndex(newC.y + r))]->getPassability())
		return new Vector(oldC.x, oldC.y);
	if (!matrix[INDEX(getSquareIndex(newC.x), getSquareIndex(newC.y - r))]->getPassability())
		return new Vector(oldC.x, oldC.y);
	if (!matrix[INDEX(getSquareIndex(newC.x + r), getSquareIndex(newC.y))]->getPassability())
		return new Vector(oldC.x, oldC.y);
	if (!matrix[INDEX(getSquareIndex(newC.x - r), getSquareIndex(newC.y))]->getPassability())
		return new Vector(oldC.x, oldC.y);
	if (!matrix[INDEX(getSquareIndex(newC.x + d), getSquareIndex(newC.y + d))]->getPassability())
		return new Vector(oldC.x, oldC.y);
	if (!matrix[INDEX(getSquareIndex(newC.x - d), getSquareIndex(newC.y + d))]->getPassability())
		return new Vector(oldC.x, oldC.y);
	if (!matrix[INDEX(getSquareIndex(newC.x + d), getSquareIndex(newC.y - d))]->getPassability())
		return new Vector(oldC.x, oldC.y);
	if (!matrix[INDEX(getSquareIndex(newC.x - d), getSquareIndex(newC.y - d))]->getPassability())
		return new Vector(oldC.x, oldC.y);
	return NULL;
}


/* getSquareBorder()
 *
 * Result:
 *    float : size of the border of map squares
 *
 * Returns the size of the border square. This can be used to calculate the 
 * square ccordinates for any position in the map.
 */
float Environment::getSquareBorder() 
{
	return squareBorder;
}


/* getSquare(int x, int y)
 *
 * Parameters:
 *    x : the x coordinate of the requested square
 *    y : the y coordinate of the requested square
 *
 * Result:
 *    Square * : pointer to the requested square
 *
 * The pointer to the class gives you two much control over the returned 
 * value. Never delete the obtained pointer as you would cause many
 * problems.
 */
Square *Environment::getSquare(int x, int y) 
{
	return matrix[INDEX(x, y)];
}


/* getSoldierList()
 *
 * Result:
 *    List * : pointer to the list of all soldiers in the environment
 *
 * The pointer to the list gives you two much control over the returned 
 * value. Never delete the list nor change its contents. Only Environment
 * class is allowed to do that. Dead soldiers are also in the list, check
 * their health before using their data.
 */
List *Environment::getSoldierList() 
{
	return &soldiers;
}


/* getBulletList()
 *
 * Result:
 *    List * : pointer to the list of all bullets in the environment
 *
 * The pointer to the list gives you two much control over the returned 
 * value. Never delete the list nor change its contents. Only Environment
 * class is allowed to do that. 
 */
List *Environment::getBulletList() 
{
	return &bullets;
}


/* getVisibleEnemies(Soldier *soldier)
 *
 * Parameters:
 *    soldier : soldier to get visible enemies for
 *
 * Result:
 *    List * :  list of enemies, the soldier sees
 *
 * Calculates which enemies the soldier sees. Returns a list of pointers to
 * those enemies. You should free the list once you don't need it any more,
 * but keep the pointers to soldiers intact.
 *
 * The routine assumes the soldier sees at an infinite length. It uses
 * the bulletHitWall() routine which will have to be changed if this routine
 * will try to do anything with the bullet pointer in the future. It also
 * assumes that the soldier has to see the center of the enemy to see it at all.
 * It ignores dead soldiers completely.
 */
List *Environment::getVisibleEnemies(Soldier *soldier)
{
	Soldier *tSold;
	List *enemies;
	Vector *temp;
	int obstacle = 0, i = 0;

	enemies = new List(); // initialize list

	// store soldier data in array for later use
	int numSold = soldiers.getNumItems();
	Vector *coords = new Vector[numSold];
	float *r = new float[numSold];
	int *id = new int[numSold];
	
	tSold =(Soldier *) soldiers.getFirst();
	while (tSold) 
	{
		if (tSold->getHealth())
		{
			coords[i] = tSold->getCoords();
			r[i] = tSold->getRadius();
			id[i++] = tSold->getID();
		}
		
		// get next soldier
		tSold =(Soldier *) soldiers.getNext();
	}
	numSold = i;
	
	tSold = (Soldier *) soldiers.getFirst();
	while (tSold)
	{
		// don't look for own soldiers and dead soldiers
		if ((tSold->getHealth()) && (tSold->getAiID() != soldier->getAiID()))
		{
			// calculate relative soldier position and check if he is in the field of view
			float soldAngle = angleOfVector(soldier->getCoords(), tSold->getCoords());
			float diff = (float)fabs(soldAngle - soldier->getViewAngle());
			if (diff > PI) diff = 2 * PI - diff;
			if (diff < soldier->getFieldOfView() / 2.0f)
			{
				// is there a wall in the way
				if (temp = bulletHitWall(NULL, soldier->getCoords(), tSold->getCoords()))
					delete temp; // delete coordinates of the offending wall
				else
				{
					// check if there are any soldiers in the way
					obstacle = 0;
					for (i = 0; i<numSold; i++)
						if (((soldier->getID ()!=-1) && (id[i] != soldier->getID()) && (id[i] != tSold->getID())) || ((soldier->getID ()==-1) && (id[i] != tSold->getID()))) // MITJA: za getVisible
							if (circleHitByLineSegment(coords[i], r[i],
								soldier->getCoords(), tSold->getCoords()))
							{
								obstacle = 1;
								break;
							}
					if (!obstacle) // he can see the enemy
						enemies->add(tSold);
				}
			}
		}
		tSold = (Soldier *) soldiers.getNext();
	}
	delete[] coords;
	delete[] r;
	delete[] id;
	return enemies;
}

List *Environment::getClientSoldiers(int id)
{
	List *list = new List();

	// add living friendly soldiers
	Soldier *temp = (Soldier *) plData[id].friendlySoldiers.getFirst();
	while (temp)
	{
		if (temp->getHealth())
			list->add(temp);
		temp = (Soldier *) plData[id].friendlySoldiers.getNext();
	}
	
	// add visible enemy soldiers
	list->merge(&(plData[id].enemySoldiers));
	return list;
}

int Environment::getMaxSx ()
{
	return maxSx;
}

int Environment::getMaxSy ()
{
	return maxSy;
}

void Environment::readWeights()
{
	Group *temp;
	for (int i=0; i<2; i++)
	{
		temp = (Group *) plData[i].groups.getFirst();
		while (temp)
		{
			temp->readWeights();
			temp = (Group *) plData[i].groups.getNext();
		}
	}
}

// MITJA
Automaton *Environment::makeAutomaton (int aiID, Group *grp)
{
	Automaton *aut;

	if (aiID==0)
		aut=new AutomatonMDM (grp, this, "LaserMDM.ini");
	else
		if (aiID==1)
			aut=new AutomatonMDM (grp, this, "LaserDemo.ini");

	return aut;
}

// MITJA
bool Environment::getVisible (Soldier *sees, Soldier *isSeen, Vector from, float angle)
{
	List *enemies;

	ghostSoldier->coords=from;
	ghostSoldier->viewAngle=angle;
	if (sees!=NULL)
		ghostSoldier->soldierID=sees->getID ();
	else
		ghostSoldier->soldierID=-1;
	if (sees!=NULL)
	  ghostSoldier->aiID=sees->getAiID ();
	else
	  ghostSoldier->aiID=-1;
	enemies=getVisibleEnemies (ghostSoldier);
	if (enemies->contains (isSeen))
	{
		delete enemies;
		return true;
	}
	else
	{
		delete enemies;
		return false;
	}
}

void Environment::getMatrix(Square ***Matrix, int *maxX, int *maxY)
{
	*Matrix = matrix;
	*maxX = maxSx;
	*maxY = maxSy;
}
