
// $Id: Environment.hpp,v 1.16 2001/04/17 07:18:49 damir Exp $

/* Environment class
 *
 * This is a class for the description of the environment. It contains all the environment
 * data (map matrix, lists of present soldiers and bullets), an update method which takes
 * care of updating the soldiers and bullets and some helper methods for calculating
 * collisions in the environment.
 *
 * There will have to be additional informational methods which will supplay AI classes with
 * necessary info. They will be decided on at a later time. The constructor will have to be
 * modified according to the way of supplying it with map matrix. This info should be provided
 * by Matija.
 *
 * Created by: Damir Arh
 * Created on: 17.11.2000
 */

#ifndef __ENVIRONMENT__
#define __ENVIRONMENT__

#include "List.hpp"
#include "Square.hpp"
#include "Bullet.hpp"
#include "Vector.hpp"
#include "Soldier.hpp"
#include "PlayerData.hpp"
#include "CollAvoidance.h"

#include "Automaton.hpp" // MITJA

#include "VisGraph.hpp" //\\//


class Environment
{
friend class AiMethods;
friend class MapAttributes;
friend class MapAttributesMDM;
private:
	// environment description variables
	Square **matrix;      // map matrix of maxSx x maxSy Squares
	List bullets;         // a list of all bullets in the environment
	List soldiers;        // a list of all soldiers in the environment
	int maxSx, maxSy;     // dimensions of the map matrix
	float time;           // time counter [s]
	float turnInterval;   // duration of a turn [s]
	float squareBorder;   // size of square border [m]

	// private helper methods
	int getSquareIndex(float coord); // returns the corresponding square index for a coordinate value

	// private initialization methods
	void placeSoldiers(int n); // place soldiers at random positions on the map

	// private methods for calculating collisions
	Vector *bulletHitWall(Bullet *bullet, Vector oldCoords, Vector newCoords);     // where does the bullet hit a wall
	Soldier *bulletHitSoldier(Bullet *bullet, Vector oldCoords, Vector newCoords); // which soldier does the bullet hit
	Vector *soldierHitWall(Soldier *soldier, Vector oldCoords, Vector newCoords);  // where does the soldier hit a wall

	// MITJA: used to determine if a soldier is visible by another soldier
	Soldier *ghostSoldier; // soldier moved around to determine whether his enemies are visible from there

public:

	PlayerData plData[2]; // private data for each player

	CCollAvoidance m_CollAvoid;

	// constructors and destructors
	Environment();
	Environment(int width, int height);
	Environment(char *filename);
	~Environment();

	// called every update to calculate all the changes
	void update();

	void readWeights();

	// public methods for obtaining info on the environment
	float getSquareBorder();         // returns the size of the square border
	Square *getSquare(int x, int y); // returns pointer to the square at the specified coordinates
	void getMatrix(Square ***Matrix, int *maxX, int *maxY);	// GREGOR - returns matrix
	List *getSoldierList();          // returns pointer to the list of soldiers
	List *getBulletList();           // returns pointer to the list of bullets
	List *getVisibleEnemies(Soldier *soldier); // returns a pointer to the list of enemies the soldier sees
	List *getClientSoldiers(int id); // returns pointer to a list to send to a particular client
	int getMaxSx ();
	int getMaxSy ();

	// MITJA: miscellaneous
	Automaton *makeAutomaton (int aiID, Group *grp);                            // return automaton appropriate for AI with given ID for given group
	bool getVisible (Soldier *sees, Soldier *isSeen, Vector from, float angle); // determine whether 'sees' (or any soldier if it is NULL) sees enemy (or any if 'sees' is NULL) soldier 'isSeen'if he's immobile at given coordinates looking at given angle
};

#endif // __ENVIRONMENT__
