
// $Id: EnvironmentClient.cpp,v 1.7 2001/04/08 20:59:44 mitja Exp $

#include "stdafx.h"
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <crtdbg.h>
#include "EnvironmentClient.hpp"
#include "PicCollection.hpp"
#include "TuiGraphics.hpp"
#include "Errors.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/*
* EnvironmentClient::EnvironmentClient(char *mapFile, char *tileFile, char *overlayFile)
* Desc: Main constructor, creates matrix, loads it from file, and loads textures for tiling
*       and overlay objects
* In:   mapFile: File containing data regarding the environment 
*/
EnvironmentClient::EnvironmentClient(char *mapFile)
{
	FILE       *file;
	SMapHeader head;
	SMapSquare square;
	int        i = 0, j = 0;
	
	squareBorder = 50;
	file = fopen(mapFile, "r");
	
	if (file == NULL)
		throw EErrorLoadingEnvironment();
	
	fread(&head, sizeof(head), 1, file);
	
	// check version in the head
	
	strncpy(name, head.name, ENV_NAME_LENGTH);
	tileMaps = new PicCollection(head.textureFile, EC_TILE_SIZE_X, EC_TILE_SIZE_Y);
	
	CreateMatrix(head.width, head.height);
	
	for (j = 0; j < head.height; j++)
	{
		for (i = 0; i < head.width; i++)
		{			
			fread(&square, sizeof(square), 1, file);
			
			matrix[i][j].passability = square.passability;
			matrix[i][j].textureId = square.textureId;
			
			// add to a starting position structure
		} // for
	} // for
	
	maxSx = head.width;
	maxSy = head.height;	
	fclose(file);	
	
	soldiers = new List();
	bullets = new List();
}// function


EnvironmentClient::~EnvironmentClient()
{
	int i = 0;
	
	for (i = 0; i < maxSx; i++)
	{
		free(matrix[i]);
	}		
	free(matrix);
	
	delete tileMaps;
	soldiers->deleteAll(List::CSOLDIER);
	delete soldiers;
	bullets->deleteAll(List::CBULLET);
	delete bullets;
}


/*
* void Environment::c_CreateMatrix(int sizeX, int sizeY)
* Desc: Initiates the matrix to the given size
* In:   sizeX, sizeY is the size of the matrix
*/
void EnvironmentClient::CreateMatrix(int sizeX, int sizeY)
{
	int i = 0, j = 0;
	
	matrix =(SquareClient**)(malloc(sizeX * sizeof(SquareClient*)));
	
	for (i = 0; i < sizeX; i++)
	{
		matrix[i] =(SquareClient*)(malloc(sizeY * sizeof(SquareClient)));
		for (j = 0; j < sizeY; j++)
		{
			SquareClient S(0, 0);
			matrix[i][j] = S;
		}			
	}
}


/*
* SoldierClient* EnvironmentClient::GetSoldierAt(float x, float y)
* Desc: Returns the soldier occupying x, y or NULL if there are none 
* In:   x, y are the coordinates to be checked
*/
SoldierClient* EnvironmentClient::GetSoldierAt(float x, float y)
{
	SoldierClient *soldier;
	
	soldier =(SoldierClient*)(soldiers->getFirst());
	
	while (soldier != NULL) 
	{
		if (pow(soldier->GetX() - x, 2) + pow(soldier->GetY() - y, 2) <
			pow(soldier->GetRadius(), 2))
			return soldier;
		
		soldier =(SoldierClient*)(soldiers->getNext());
	}
	return NULL;
}


/*
* void EnvironmentClient::SetAllSoldiersSelected(bool value)
* Desc: Set the selected member of all soldiers in the environment
* In:   value is the value for all soldiers
*/
void EnvironmentClient::SetAllSoldiersSelected(bool value)
{
	SoldierClient *soldier;
	
	soldier =(SoldierClient*)(soldiers->getFirst());
	
	while (soldier != NULL) 
	{
		soldier->selected = value;
		soldier =(SoldierClient*)(soldiers->getNext());
	}	
}


/* These are some Get/Set properties for protected members. */
int EnvironmentClient::GetMaxSX()
{
	return maxSx;
}


int EnvironmentClient::GetMaxSY()
{
	return maxSy;
}


int EnvironmentClient::GetSquareBorder()
{
	return squareBorder;
}


SquareClient *EnvironmentClient::GetSquare(int x, int y)
{
	if (x >= 0 && x < GetMaxSX() && y >= 0 && y < GetMaxSY())
		return &matrix[x][y];
	else
		return NULL;
}


char *EnvironmentClient::GetName()
{
	return name;
}
