
// $Id: EnvironmentClient.hpp,v 1.7 2001/04/08 15:40:52 mitja Exp $

/* Environment class on the client
 *
 * Extended environment class, which allows to load bitmaps for the tiles and soldiers/bullets
 * draws the squares at the requested coords.
 * Created by: Matija Jekovec
 * Created on: 22.11.2000
 */

#ifndef __ENVIRONMENT_CLIENT__
#define __ENVIRONMENT_CLIENT__

class EnvironmentClient;

#include "enums.hpp"
#include "SoldierClient.hpp"
#include "SquareClient.hpp"
#include "Viewport.hpp"
#include "List.hpp"
#include "PicCollection.hpp"

#define EC_TILE_SIZE_X 16 // size for one picture in loaded texture files
#define EC_TILE_SIZE_Y 16
#define EC_OVERLAY_SIZE_X 16
#define EC_OVERLAY_SIZE_Y 16
#define ENV_NAME_LENGTH 32
#define ENV_TEXTURE_FILE_LENGTH ENV_NAME_LENGTH

struct SMapHeader
{
	float version;
	int   width,height; // dimenzije matrike
	char  name[ENV_NAME_LENGTH];
	char  textureFile[ENV_TEXTURE_FILE_LENGTH];	// ime datoteke s tekturami
	int   dataOffset;   // kje se podatki zacnejo - sedaj sizeOf(MapHeader)		
};

struct SMapSquare
{
	unsigned char passability;      // prehodnost
	unsigned char textureId;
	unsigned char startingPosition; // stran, ki ima ta kvadratek za zacetni
};

class EnvironmentClient
{
private:
	/* a bitmap with somer additional functions for cropping. Bitmap is made 
	 * out of pictures of the environment */
	PicCollection *tileMaps;
	PicCollection *overlayMaps; // same as tileMaps for the soldiers and bullets
	int squareBorder;           // size of square border [cm]
	int maxSx, maxSy;           // dimensions of the map matrix [squares]
	SquareClient **matrix;      // map matrix of maxSx * maxSy Squares
	char name[ENV_NAME_LENGTH]; // name of the map
	
	void CreateMatrix(int sizeX, int sizeY); // initiates the matrix

public:
	// constructor creates both the tileMaps and overlayMaps, loading the bitmaps from disk
	EnvironmentClient(char *mapFile);
	~EnvironmentClient(); // destructor, also destroys the tileMap and overlayMap

	/* draws the square at x,y (indexes in the matrix) to the surface, using and
	 * converting the coordinates in the view, taking account of zoom. The 
	 * function is defined in this class and not in the Square, becasue the
	 * function is called only at scrolling and not every update */
	//	void DrawSquare(int x, int y, Viewport *viewport, SDL_Surface *surface);
	List *soldiers;
	List *bullets;
	SoldierClient* GetSoldierAt(float x, float y); // returns soldier at x,y
	void SetAllSoldiersSelected(bool value);       // sets value to selected for all soldiers

	 // access to protected members
	int GetMaxSX();
	int GetMaxSY();
	int GetSquareBorder();
	SquareClient* GetSquare(int x, int y);
	char* GetName();

};

#endif // __ENVIRONMENT_CLIENT__
