// Flood.cpp: implementation of the CFlood class.
//
//	Gregor Leban, 20.4.2001
//
//
//
//////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "Flood.h"
#include <math.h>
#include <stdio.h>


// calculates the index in the matrix array
#define INDEX(X, Y)(X + Y * m_iXsize)
#define MATRIX(X, Y)(X + 1) + (Y + 1)*(m_iMaxX + 2)

#pragma warning ( disable : 4244)		//conversion double to float ...

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFlood::CFlood()
{
	m_pEnv = NULL;
	m_pbArea = NULL;
	m_pMatrix = NULL;
}

CFlood::~CFlood()
{
	Free();
}

void CFlood::Free()
{
	if (m_pbArea) delete m_pbArea;
	m_pbArea = NULL;
	m_pEnv = NULL;
	m_iXoff = 0;
	m_iYoff = 0;
	m_iXsize = 0;
	m_iYsize = 0;
}


void CFlood::FloodArea(Vector begin, Vector end, Environment *env, int Xoff, int Yoff)
{
	Vector start, finish;

	Free();

	m_pEnv = env;
	env->getMatrix(&m_pMatrix, &m_iMaxX, &m_iMaxY);

	//pravilno obrnemo in omejimo
	start.x = floor(min(begin.x, end.x));
	start.y = floor(min(begin.y, end.y));
	finish.x = floor(max (begin.x, end.x))+1;
	finish.y = floor(max (begin.y, end.y))+1;
	start.x = max(start.x - Xoff, 0);
	start.y = max(start.y - Yoff, 0);
	finish.x = min (finish.x + Xoff, env->getMaxSx());
	finish.y = min (finish.y + Yoff, env->getMaxSy());


	m_iXoff = start.x;
	m_iYoff = start.y;
	m_iXsize = finish.x - start.x;
	m_iYsize = finish.y - start.y;

	m_pbArea = new bool [m_iXsize * m_iYsize];

	int x,y;
	x = floor(begin.x - m_iXoff);
	y = floor(begin.y - m_iYoff);

	SetFalse();

	// zapolnimo prvo vrstico
	for (int i=x; (i < m_iXsize) && (Passability(i,y)); i++) SetTrue(i,y);
	for (    i=x; (i >= 0 ) && (Passability(i,y)); i--) SetTrue(i,y);

	for (    i=y; (i < m_iYsize) && (Passability(x,i)); i++) SetTrue(x,i);
	for (    i=y; (i >= 0 ) && (Passability(x,i)); i--) SetTrue(x,i);

	for ( i = y; i < m_iYsize; i++) FillLines(i);
	for ( i = y-1; i >= 0    ; i--) FillLines(i);


	for ( i = x; i < m_iXsize; i++) FillCols(i);
	for ( i = x-1; i >= 0    ; i--) FillCols(i);
}


int CFlood::Passability(int x, int y)
{
	if (x < 0) return 0;
	if (y < 0) return 0;
	if (x >=m_iMaxX) return 0;
	if (y >=m_iMaxY) return 0;

	int xpos = x + m_iXoff;
	int ypos = y + m_iYoff;

	return m_pMatrix[MATRIX(xpos, ypos)]->getPassability();
}

int CFlood::Accessibility(int x, int y)
{
	if (x < 0) return 0;
	if (y < 0) return 0;
	if (x >=m_iXsize) return 0;
	if (y >=m_iYsize) return 0;

	return m_pbArea[INDEX(x, y)];
}


void CFlood::SetFalse(int x, int y)
{
	if (x == -1)
	{
		memset (m_pbArea, 0, sizeof(bool) * m_iXsize * m_iYsize);
	}
	else m_pbArea[INDEX(x, y)] = false;
}

void CFlood::SetTrue(int x, int y)
{
	m_pbArea[INDEX(x,y)] = true;
}

void CFlood::FillLines(int y)
{
	// potrebno je iti 2* cez vse kocke, sicer lahko ne zapolnimo vsega...
	for (int i=0; i < m_iXsize; i++)
	{
		if (!Passability(i, y)) continue;
		if (Accessibility(i, y-1) || Accessibility(i, y+1) || Accessibility(i-1, y) || Accessibility(i+1, y)) SetTrue(i, y);
	}

	for (i = m_iXsize-1; i>=0; i--)
	{
		if (!Passability(i, y)) continue;
		if (Accessibility(i, y-1) || Accessibility(i, y+1) || Accessibility(i-1, y) || Accessibility(i+1, y)) SetTrue(i, y);
	}
}

void CFlood::FillCols(int x)
{
	for (int i=0; i < m_iYsize; i++)
	{
		if (!Passability(x, i)) continue;
		if (Accessibility(x-1, i) || Accessibility(x+1, i) || Accessibility(x, i-1) || Accessibility(x, i+1)) SetTrue(x, i);
	}

	for (i = m_iYsize-1; i>=0; i--)
	{
		if (!Passability(x, i)) continue;
		if (Accessibility(x-1, i) || Accessibility(x+1, i) || Accessibility(x, i-1) || Accessibility(x, i+1)) SetTrue(x, i);
	}
}



int CFlood::GetAccessibility(int x, int y)
{
	if (!m_pbArea) return 0;
	int xpos = x - m_iXoff;
	int ypos = y - m_iYoff;

	return Accessibility(xpos, ypos);
}

void CFlood::Printout()
{
	if (!m_pbArea) return;

	FILE * f = fopen ("flood.txt", "wt");

	char one[3];
	char nul[3];
	strcpy (one, "1");
	strcpy (nul, "0");

	for (int j = 0; j < m_iYsize; j++)
	{
		for (int i=0; i < m_iXsize; i++)
		{
			int out;
			out = Accessibility(i, j);
			if (out > 0) fwrite (&one, 1, strlen(one), f);
			else fwrite (&nul, 1, strlen(nul), f);
		}
		fwrite ("\n", 1, 1, f);
	}

	fclose (f);
}

void CFlood::BuildDangerMap(Environment *env, char *filename, int offset)
{
	FILE * f = fopen(filename, "wb");
	
	Square **map;
	env->getMatrix(&map, &m_iMaxX, &m_iMaxY);


	for (int j=0; j < env->getMaxSy(); j++)
	{
		for (int i=0; i < env->getMaxSx(); i++)
		{
			Free();
			Vector curr (i, j);
			
			if (map[MATRIX(i,j)]->passability)
			{
				FloodArea(curr, curr, env, offset, offset);
				int no = m_iXsize * m_iYsize - CountTrue();
				int total = m_iXsize * m_iYsize;
	
				char val = float(no) * 10.0f / float(total);
				val++;
				fwrite(&val, 1, 1, f);
			}
			else
			{
				char val[3];
				val[0] = 0;
				fwrite(&val, 1, 1, f);
			}
		}
	}

	Free ();

	fclose (f);
}

int CFlood::CountTrue()
{
	int count = 0;
	for (int i=0; i< m_iXsize * m_iYsize; i++) 
	{
		if (m_pbArea[i] == true) count++;
	}
	return count;
}

bool CFlood::LoadDangerMap(char *filename, char **pMap, int size)
{
	FILE *f = fopen (filename, "rb");
	if (!f) return false;

	char * array = new char [size];

	int read = fread (array, 1, size, f);
	if (read != size)
	{
		delete array;
		return false;
	}
	
	*pMap = array;
	return true;
}

void CFlood::PrintDanger(char *array, Environment *env)
{
	FILE *f = fopen ("danger.txt", "wt");

	Square **map;
	int x, y;
	env->getMatrix(&map, &x, &y);

	for (int i=0; i < y; i++)
	{
		for (int j=0; j < x; j++)
		{
			char buff[3];
			char val = array[i*x + j];
			if (val == 0)
			{
				strcpy (buff, "X");
			}
			else
			{
				buff[0] = val + 48;
				buff[1] = 0;
			}
			fwrite (buff, 1, strlen(buff), f);
		}
		char nl[3];
		strcpy (nl, "\n");
		fwrite (&nl, 1, strlen(nl), f);
	}

	fclose (f);
}
