// Group.cpp: implementation of the Group class.
//
// Gregor Leban  
//		History:
//			- 5.4 - 15.4 - osnovni razred z osnovnimi metodami Getfreesoldier in Getsafeposition
//			- 15.4  - Lustrek popravil nekaj memory leakov in dodal neke funkcije
//
//
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include <stdio.h>
#include <math.h>
#include "Group.hpp"
#include "List.hpp"

#include "Group.hpp"
#include "Environment.hpp"
#include "Auxiliary.hpp"
#include "VisGraph.hpp"
#include "Soldier.hpp"
#include "AiMethods.hpp"
#include "Dangermap.hpp"
#include "TextDisplay.hpp"
#include "VectorDir.h"
#include "Flood.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define TOLERANCE		1
#define MAX_DIST		2		// ce sta vojaka narazen za vec kot toliko, potem se smatra, da se ne ovirata
#define GROUP_TOLERANCE	4		// koliko je lahko oddaljenost centra grupe od waypointa, da gremo gledat ze naslednji WP
#define OFF				0.2
#define MIN_SQUARE		8		// minimalno stevilo kvadratkov za sirino ali visino kvadrata v katerem bomo iskali varna podrocja
#define BEEN_THERE		2		// minimalno stevilo kvadratkov za koliko se mora razlikovati nova koordinata, da recemo, da vojak se ni bil tam


// calculates the index in the matrix array
#define INDEX(X, Y)(X + 1) + (Y + 1)*(iX + 2)

extern TextDisplay *mainDisplay;
extern DrawStruct draw;

#pragma warning ( disable : 4244)		//conversion double to float ...
//////////////////////////////////////////////////////////////////////
// CLASS: SoldierItem - vsi parametri, ki se ticejo enega soldierja
//////////////////////////////////////////////////////////////////////

CSoldierItem::CSoldierItem()
{
	m_iWaypointCount = 0;
	m_aWaypoints = NULL;

	m_pSoldier = NULL;

	m_MotionState = MOTION_NO_WAYPOINT;
	m_iCurrWaypoint = 0;
	for (int i=0; i < sizeof(m_aExWaypoints) / sizeof(Vector); i++)
	{
		m_aExWaypoints[i].x = -10;
		m_aExWaypoints[i].y = -10;
	}
	m_iTimeMoved = GetTickCount();
}

CSoldierItem::~CSoldierItem()
{	
	m_iWaypointCount = 0;
	if (m_aWaypoints) delete m_aWaypoints;
	m_aWaypoints = NULL;
	
	m_pSoldier = NULL;

	m_MotionState = MOTION_NO_WAYPOINT;
	m_iCurrWaypoint = 0;
}

//////////////////////////////////////////////////////////////////////////////////////
// doda seznam waypointov, ki jih mora soldier obiskati
//////////////////////////////////////////////////////////////////////////////////////
void CSoldierItem::AddPath(Vector coords, Group *grp)
{
	List *path;

	// zapomnimo si, da ne bomo se kdaj sli na ta wp
	memmove (&m_aExWaypoints[0], &m_aExWaypoints[1], sizeof(Vector) * 9);
	m_aExWaypoints[0] = coords;

	if (m_aWaypoints) delete m_aWaypoints; // MITJA: CRASH vrste 'user breakpoint ...', nato nekaj 'debug assertationov' (m_iWaypointCount=1; m_aWaypoints.x, .y=nekaj cudnega), dokler nisem dodal
	
	path = grp->m_pVisGraph->findPath (m_pSoldier->getCoords (), coords);
	
	if (path!=NULL) // MITJA
	{
		int count = path->getNumItems()+1;
		m_aWaypoints = new Vector [count];
	
		Vector *pItem = (Vector *) path->getFirst();
		for (int i=0; i < count-1 ; i++)
		{
			ASSERT(pItem);
			m_aWaypoints[i] = *pItem;
			pItem = (Vector *) path->getNext();
		}
		m_aWaypoints[count-1] = coords;

		path->deleteAll(List::VECTOR);
		delete path;

		// nastavimo ostale variable
		m_iCurrWaypoint = 0;
		m_iWaypointCount = count;
		m_MotionState=MOTION_MOVING;
	}
}

//////////////////////////////////////////////////////////////////////////////////////
// MITJA: remove soldier's waypoints and stop him
//////////////////////////////////////////////////////////////////////////////////////

void CSoldierItem::removePath ()
{
	if (m_aWaypoints)
		delete m_aWaypoints;
	m_aWaypoints=NULL;
	m_iWaypointCount=0;
	m_MotionState=MOTION_NO_WAYPOINT;
	m_iCurrWaypoint=0;

	m_pSoldier->setSpeed (0);
}

//////////////////////////////////////////////////////////////////////
// CLASS : Group
//////////////////////////////////////////////////////////////////////

/*
kaj mora delati grupa
	- ima seznam vojakov
	- shrani si pot (seznam waypointov) po kateri vozi svoje vojake
	- skrbi za to, da posilja vojake po poti, pri cemer izbere njihovo
	  naslednjo lokacijo glede na to, v katerem stanju grupa je. pravilno
	  uposteva to, da so dovolj skupaj, da pokrivajo dovolj veliko podrocje,
	  da se krijejo med sabo ...
	- skrbi za to, da ne poslje vseh vojakov naenkrat na novo lokacijo (izogibanje
	  nepotrebni koleziji)
*/


Group::Group (int groupID1, VisGraph *visGraph, Environment* env)
{
	m_aWaypoints = NULL;
	m_iWaypointCount = 0;
	
	m_aSoldierItems = NULL;
	m_iSoldierCount = 0;

	m_iStepsBeforeSendingNext = 5;
	m_iCurrSteps = 0;

	m_pAI = new AiMethods (this, env);
	m_pAutomat = NULL;
	m_pVisGraph = visGraph;
	m_pEnv = env;
	m_pSoldierList = new List();;
	m_aSoldierItems = NULL;

	m_vGroupCenter.x = 0;
	m_vGroupCenter.y = 0;

	m_iGroupID = groupID1;
	m_MotionType = MOTION_NO_WAYPOINT;

	m_aExChoosenSoldiers[0] = -1;
	m_aExChoosenSoldiers[1] = -1;
	m_aExChoosenSoldiers[2] = -1;

	Square ** matrix;
	int x,y;
	env->getMatrix(&matrix, &x, &y);
	m_DangerMap.CreateDangerMap(env, matrix, x, y);
}


Group::~Group()
{
/*	if (m_aWaypoints) delete m_aWaypoints;
	m_aWaypoints = NULL;
	m_iWaypointCount = 0;*/
	removeWayPoints (); // MITJA
	
	if (m_aSoldierItems)
	{
		for (int i=0; i< m_iSoldierCount; i++) m_aSoldierItems[i].~CSoldierItem();
	}
	delete [] m_aSoldierItems;

	m_aSoldierItems = NULL;
	m_iSoldierCount = 0;
	m_iCurrSteps = 0;

	if (m_pSoldierList) 
	{
//		m_pSoldierList->deleteAll(List::SOLDIER);
		delete m_pSoldierList;
		m_pSoldierList = NULL;
	}
	
	if (m_pAI) delete m_pAI;

	draw.DeleteAll();
	m_MotionType = MOTION_NO_WAYPOINT;

	if (m_pAutomat) // MITJA
		delete m_pAutomat;
}

//////////////////////////////////////////////////////////////////////////////////////
// dodaj nov seznam vojakov
//////////////////////////////////////////////////////////////////////////////////////
void Group::addSoldiers(List *list)
{
	m_iSoldierCount = list->getNumItems();

	if (m_pSoldierList) 
	{
//		m_pSoldierList->deleteAll(List::SOLDIER);
		delete m_pSoldierList;
		m_pSoldierList = NULL;
	}

	if (m_aSoldierItems)
	{
		for (int i=0; i< m_iSoldierCount; i++) m_aSoldierItems[i].~CSoldierItem();
	}

//	if (m_aSoldierItems) delete m_aSoldierItems;
	m_aSoldierItems = new CSoldierItem[m_iSoldierCount];

	// ne vem, ali moram klicati konstruktor posebej za vsak element m_aSoldierItems ali se je klical avtomatsko
	
	Soldier * pItem = (Soldier *) list->getFirst();
	for (int i=0; i < m_iSoldierCount; i++)
	{
		ASSERT(pItem);
		m_aSoldierItems[i].m_pSoldier = pItem;
		pItem = (Soldier *) list->getNext();
	}

	m_pSoldierList = list;
	m_iCurrSteps = 0;

	draw.DeleteAll();
	draw.SetSoldCount(m_iSoldierCount);
}


//////////////////////////////////////////////////////////////////////////////////////
//	odstrani vojaka iz grupe - ce je bil ubit, ali pa je sel k drugi grupi
//////////////////////////////////////////////////////////////////////////////////////
void Group::RemoveSoldier(int index)
{
	Soldier *temp; // MITJA

	ASSERT (index < m_iSoldierCount);
	ASSERT (index >=0);

	// MITJA
	temp=(Soldier *)(m_pSoldierList->getFirst ());
	while (temp!=NULL)
	{
		if (temp==m_aSoldierItems [index].m_pSoldier)
		{
			m_pSoldierList->remove ();
			break;
		}
		temp=(Soldier *)(m_pSoldierList->getNext ());
	}
	
	// zbrisemo njegovo pot
	m_aSoldierItems[index].removePath();
	
	// premik preostalih
	CSoldierItem item = m_aSoldierItems[index];
	for (int i= index; i < m_iSoldierCount-1; i++)
	{
		m_aSoldierItems[i] = m_aSoldierItems[i+1];
	}
	m_aSoldierItems[m_iSoldierCount-1] = item;

	m_iSoldierCount--;
}


//////////////////////////////////////////////////////////////////////////////////////
//	doda v grupo seznam waypointov katerim bodo vojaki sledili
//////////////////////////////////////////////////////////////////////////////////////
void Group::addWayPoints (List *way)
{
	if (m_aWaypoints) delete m_aWaypoints;
	m_aWaypoints = new Vector [way->getNumItems() + 1];
	m_aWaypoints[0] = m_vGroupCenter;
		
	m_iWaypointCount = 0;
	Vector from = m_vGroupCenter;
	Vector to = *(Vector *)(way->getFirst ());
	Vector *temp;
	for (int i=0; i < way->getNumItems (); i++)
	{
		List * wayPointsBetween = m_pVisGraph->findPath (from, to);
		if (wayPointsBetween!=NULL)
		{
			if (m_aWaypoints==NULL) m_aWaypoints = new Vector [wayPointsBetween->getNumItems()+1];
			else					 m_aWaypoints = (Vector *) realloc (m_aWaypoints, sizeof (Vector)*(m_iWaypointCount+wayPointsBetween->getNumItems ()+1));
			
			temp = (Vector *)(wayPointsBetween->getFirst ());
			while (temp!=NULL)
			{
				m_aWaypoints [m_iWaypointCount] = *temp;
				m_iWaypointCount++;
				temp = (Vector *)(wayPointsBetween->getNext ());
			}
			m_aWaypoints [m_iWaypointCount] = to;
			m_iWaypointCount++;
			wayPointsBetween->deleteAll(List::VECTOR);
		}
		else break;
		from = to;
		Vector *next =(Vector *)(way->getNext ());
		if (next) to = *next;

		delete wayPointsBetween; // MITJA
	}
	m_MotionType = MOTION_MOVING;
	m_iCurrentWaypoint = 0;

	way->deleteAll(List::VECTOR);
	delete way;

	startingCoords=m_vGroupCenter; // MITJA
}

//////////////////////////////////////////////////////////////////////////////////////
//	MITJA: remove group's waypoints
//////////////////////////////////////////////////////////////////////////////////////

void Group::removeWayPoints ()
{
	int i;

	if (m_aWaypoints)
		delete m_aWaypoints;
	m_aWaypoints = NULL;
	m_iWaypointCount = 0;
	m_MotionType=MOTION_NO_WAYPOINT;
	m_iCurrSteps=0;

	if (m_aSoldierItems)
		for (i=0; i<m_iSoldierCount; i++)
			m_aSoldierItems [i].removePath ();
}

//////////////////////////////////////////////////////////////////////////////////////
//	glavna procedura, ki skrbi za takticno premikanje vojakov
//////////////////////////////////////////////////////////////////////////////////////
void Group::update()
{
/*	
	- odstrani mrtve vojake
	- sprehodi se najprej po seznamu vseh vojakov, ki so trenutno v gibanju in jim 
		izracunaj nove koordinate
	- ce je ze minilo dovolj korakov odkar smo poslali v gibanje zadnjega vojaka
	  izvedi : 
			- GetFreeSoldier - izmed vseh mirujocih vojakov izberi tistega, ki je 
				najbolj oddaljen od naslednjega waypointa in "ni v sredini grupe"
			- GetSafePosition - izmed vseh varnih pozicij glede na stanje grupe
				vrne vektor, kamor naj gre soldier
*/
//	SleepEx(80, TRUE);
	CalculateCenter();
	
	// dead-soldier removal
	int i=0;
	while (i < m_iSoldierCount)
	{
		if (!m_aSoldierItems[i].m_pSoldier->getHealth()) RemoveSoldier(i);
		else i++;
	}

	if (m_pAutomat!=NULL)
		m_pAutomat->update ();

	bool someAction = false;
	for (i=0; i < m_iSoldierCount; i++)
	{
		if (m_aSoldierItems[i].m_MotionState == MOTION_MOVING)
		{
			someAction = true;
			GetNextCoords(i);
			if ((m_aSoldierItems[i].m_pSoldier->getCoords() - m_aSoldierItems[i].m_aWaypoints[m_aSoldierItems[i].m_iCurrWaypoint]).length() <= TOLERANCE)
			{
				if (m_aSoldierItems[i].m_iCurrWaypoint + 1 == m_aSoldierItems[i].m_iWaypointCount) 
				{
//					draw.ClearSoldierWaypoints(i);
//					mainDisplay->display ("Soldier %d has reached final waypoint (%.2lf, %.2lf)", i, m_aSoldierItems[i].m_aWaypoints[m_aSoldierItems[i].m_iCurrWaypoint].x, m_aSoldierItems[i].m_aWaypoints[m_aSoldierItems[i].m_iCurrWaypoint].y);
					m_aSoldierItems[i].m_MotionState = MOTION_NO_WAYPOINT;
					m_aSoldierItems[i].m_pSoldier->setSpeed (0);
					draw.ClearSoldierWaypoints(i);
				}
				else 
				{
//					mainDisplay->display ("Soldier %d has reached waypoint (%.2lf, %.2lf)", i, m_aSoldierItems[i].m_aWaypoints[m_aSoldierItems[i].m_iCurrWaypoint].x, m_aSoldierItems[i].m_aWaypoints[m_aSoldierItems[i].m_iCurrWaypoint].y);
					m_aSoldierItems[i].m_iCurrWaypoint++;
				}
			}
		}
	}

	if ((m_MotionType == MOTION_NO_WAYPOINT) && (!someAction)) return;

	//poglejmo, ce smo s centrom grupe ze dovolj blizu waypointu
	Vector diff; // MITJA
	if (m_aWaypoints)
		diff = m_vGroupCenter - m_aWaypoints[m_iCurrentWaypoint];
	else
	{
		diff.x=GROUP_TOLERANCE;
		diff.y=GROUP_TOLERANCE;
		m_MotionType = MOTION_NO_WAYPOINT;
	}
//	diff.x = diff.x / m_iSoldierCount;		// to ni kul, bi pa morali deliti z neko funkcijo stevila vojakov
//	diff.y = diff.y / m_iSoldierCount;
	if (diff.length() < GROUP_TOLERANCE)
	{
		if (m_iCurrentWaypoint+1 < m_iWaypointCount) m_iCurrentWaypoint++;
		else	m_MotionType = MOTION_NO_WAYPOINT;		// poskrbimo, da se ustavi premikanje
	}

	// ta del kode bi bil lahko kasneje precej spremenjen
	// m_iStepsBeforeSendingNext bi lahko bila funckija odvisna od stanja, ...
	if ((m_iCurrSteps >= m_iStepsBeforeSendingNext) && (m_MotionType == MOTION_MOVING))
	{
		m_iCurrSteps = 0;
		int index;
		index = GetFreeSoldier ();
		if (index < 0) return;
		Vector dest = GetSafePosition(index);
		if ((dest.x < 0) || (dest.y < 0))
		{
//			mainDisplay->display ("GetSafePosition returned with invalid vector (%.2lf, %.2lf)", dest.x, dest.y);
			return;	// to se ne sme zgoditi
		}
		else
		{
//			mainDisplay->display ("Soldier %d has new waypoint (%.2lf, %.2lf)", index, dest.x, dest.y);
		}

		m_aSoldierItems[index].AddPath(dest, this);
			// narisimo to na vektor oknu...
		draw.SetSoldierWaypoints(index, m_aSoldierItems[index].m_pSoldier->getCoords(), m_aSoldierItems[index].m_aWaypoints, m_aSoldierItems[index].m_iWaypointCount);
//		SendSoldier (index, dest);
	}
	else
	{
		m_iCurrSteps++;
	}
}


//////////////////////////////////////////////////////////////////////////////////////
//	glede na waypointe od vojaka in s pomocjo boidov in colldet premakne vojaka naprej
//////////////////////////////////////////////////////////////////////////////////////
void Group::GetNextCoords(int index)
{
	m_aSoldierItems[index].m_iTimeMoved = GetTickCount();
	m_aSoldierItems[index].m_pSoldier->setMoveVector (m_pAI->calculateVector (index));
}

//////////////////////////////////////////////////////////////////////////////////////
///	--------------------------------- ////////////////////////////////////////////////
/// G E T   F R E E   S O L D I E R   ////////////////////////////////////////////////
///	--------------------------------- ////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////
//	vrne index vojaka, ki 
//		- je najbolj odd. od naslednjega waypointa (da vojaki ne bodo zaostajali)
//		- ni v sredini grupe (da se ne bodo prerivali)
//		- miruje
//////////////////////////////////////////////////////////////////////////////////////
int Group::GetFreeSoldier()
{
	if (m_iSoldierCount == 1) 
	{
		if (m_aSoldierItems[0].m_MotionState == MOTION_MOVING) return -1;
		else return 0;
	}

	List *solds = new List();
	// najprej dodamo vojake, ki imajo prost vsaj 1 kvadrant
	for (int i=0; i < m_iSoldierCount; i++)
	{
		bool out = false;
		Vector soldA = m_aSoldierItems[i].m_pSoldier->getCoords();
		if (m_aSoldierItems[i].m_MotionState == MOTION_NO_WAYPOINT)
		{
			if (FreeArea(soldA, 1) || FreeArea(soldA, 2) || FreeArea(soldA, 3) || FreeArea(soldA, 4)) solds->add((void *) m_aSoldierItems[i].m_pSoldier);
		}
	}

	// izmed vseh, izberemo tistega, ki ni bil premaknjen ze najvec casa 
	int best = -1;
	Soldier * item = (Soldier *) solds->getFirst();
	for (i=0; i < solds->getNumItems(); i++)
	{
		// tu je problem .....
		int index  = FindSoldierIndex (item);
		ASSERT (index != -1);
		
		if (best == -1) best = index;
		else if (m_aSoldierItems[index].m_iTimeMoved < m_aSoldierItems[best].m_iTimeMoved) best = index;

		item = (Soldier *) solds->getNext();
	}

	delete solds; // MITJA

	return best;
}

//////////////////////////////////////////////////////////////////////////////////////
///	--------------------------------- ////////////////////////////////////////////////
/// G E T   S A F E   P O S I T I O N ////////////////////////////////////////////////
///	--------------------------------- ////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////
//	vrne vektor s pozicijo, ki je
//		- "varna"
//		- s katere je dober pregled
//		- ni prevec oddaljena od centra grupe
//		- ...  - tu se uposteva lahko vse zeljene atribute in na podlagi njih in stanja izberemo najboljso pozicijo
//////////////////////////////////////////////////////////////////////////////////////
Vector Group::GetSafePosition(int index)
{
	Vector v1 = m_aSoldierItems[index].m_pSoldier->getCoords();
	Vector v2 = m_aWaypoints[m_iCurrentWaypoint];
	Vector begin, end;
	
	begin.x = min (v1.x, v2.x);
	begin.y = min (v1.y, v2.y);
	end.x = max (v1.x, v2.x);
	end.y = max (v1.y, v2.y);

	if (abs(end.x - begin.x) < MIN_SQUARE )
	{
		begin.x -= MIN_SQUARE / 2;
		end.x += MIN_SQUARE / 2;
		if (begin.x < 0) begin.x = 0;
		if (end.x > m_pEnv->getMaxSx()) end.x = m_pEnv->getMaxSx();
	}
	if (abs(end.y - begin.y) < MIN_SQUARE )
	{
		begin.y -= MIN_SQUARE / 2;
		end.y += MIN_SQUARE / 2;
		if (begin.y < 0) begin.y = 0;
		if (end.y > m_pEnv->getMaxSy()) end.y = m_pEnv->getMaxSy();
	}

	Vector vBest(-1,-1);
//	int iBestVal;

	Vector soldierPos = m_aSoldierItems[index].m_pSoldier->getCoords();

	OrderedList list (10);

	CFlood flood;
	flood.FloodArea(v1, v2, m_pEnv, 2, 2);
	flood.Printout();


	for (int i=floor(begin.x); i < floor(end.x); i++)
	{
		for (int j=floor (begin.y); j < floor(end.y); j++)
		{
			Vector pos (i,j);
			// izracunamo vrednosti moznih parametrov, ki vplivajo na kvaliteto podrocja
			int safety = m_DangerMap.GetSafety(i, j);
			int available = GetAvailability(i, j);
//			int accessible = GetAccessibility(i, j);
			int accessible = flood.GetAccessibility(i, j);
			int distance = -(soldierPos - pos).length();
			int progression = GetProgression(soldierPos, pos, m_aWaypoints[m_iCurrentWaypoint]);	// koliko bi v tej tocki napredovali

			if (!available || !accessible) continue;

			int val = Evaluate (safety, available, distance, progression) + 500;
			list.Insert(val, i, j, index);
		}
	}
	
	flood.Free();

	i = 0;
	while ((i < list.m_iCount) && (BeenThere(list.m_aItems1[i], list.m_aItems2[i], index))) i++;
	if (i < list.m_iCount)
	{
		vBest.x = list.m_aItems1[i];
		vBest.y = list.m_aItems2[i];		
	}
	else
	{
		vBest.x = -1;		// ce nimamo nobene pametne lokacije raje mirujemo
		vBest.y = -1;
	}

	if ((vBest.x > 100) || (vBest.y > 100))
	{
		int i=0;
	}
	//najboljsega povecamo za pol kvadratka, tako da bo sel vojak v sredino
	vBest.x += 0.5;
	vBest.y += 0.5;

	return vBest;
}

//////////////////////////////////////////////////////////////////////////////////////
//	zgradi waypointe za vojaka in ga poslje po tej poti
//////////////////////////////////////////////////////////////////////////////////////
/*
void Group::SendSoldier(int index, Vector dest)
{	
	memmove (&m_aSoldierItems[index].m_aExWaypoints[0], &m_aSoldierItems[index].m_aExWaypoints[1], sizeof(Vector) * 9);
	m_aSoldierItems[index].m_aExWaypoints[0] = dest;

	Vector from = m_aSoldierItems[index].m_pSoldier->getCoords();

	// izracunaj pot po kateri ga bomo poslali
	List * way = m_pVisGraph->findPath (from, dest);

	m_aSoldierItems[index].m_iCurrWaypoint = 0;
	m_aSoldierItems[index].m_iWaypointCount = 0;

	if (m_aSoldierItems[index].m_aWaypoints) delete m_aSoldierItems[index].m_aWaypoints;
	m_aSoldierItems[index].m_aWaypoints = NULL;
	if (way == NULL) return;	// ce ni poti koncamo

	m_aSoldierItems[index].m_aWaypoints = new Vector [way->getNumItems() + 1];

	// dodamo v seznam 
	m_aSoldierItems[index].m_aWaypoints[way->getNumItems()] = dest;
	Vector *item = (Vector *)(way->getFirst ());
	for (int i=0; i < way->getNumItems (); i++)
	{
		m_aSoldierItems[index].m_aWaypoints[i] = *item;
		item = (Vector *) way->getNext();
	}
	m_aSoldierItems[index].m_iWaypointCount = way->getNumItems() + 1;

	// narisimo to na vektor oknu...
	draw.SetSoldierWaypoints(index, from, way, dest);

	way->deleteAll(List::VECTOR);
	delete way; // MITJA

	// soldierju moramo nastaviti stanje na gibajoce
	m_aSoldierItems[index].m_MotionState = MOTION_MOVING;
}
*/

////////////////////////////////////////////////////////////////////////////////////
// pogleda, ce je kaksen od kvadrantov okoli vojaka prost- ali ima vojak kaksno prosto pot
////////////////////////////////////////////////////////////////////////////////////
bool Group::FreeArea(Vector pos, int area)
{
	int x,y;
	if		(area == 1) { x =  1; y =  1;	}
	else if (area == 2) { x = -1; y =  1;	}
	else if (area == 3) { x = -1; y = -1;	}
	else if (area == 4) { x =  1; y = -1;	}

	bool free = true;
	for (int i=0; i < m_iSoldierCount; i++)
	{
		Vector item = m_aSoldierItems[i].m_pSoldier->getCoords();
		if ((x*(item.x+OFF) > x*pos.x) && (y*(item.y-OFF) < y*pos.y))
		{
			if (abs(item.x - pos.x) + abs(item.y - pos.y) < MAX_DIST) free = false;		
		}
	}

	if (!free) return free;

	// pregledamo, ce imamo okoli sebe vsaj kak prost kvadratek
	Square ** map;
	int iX, iY;
	m_pEnv->getMatrix(&map, &iX, &iY);
	if ((map[INDEX((int)floor(pos.x + x*1), (int)floor(pos.y))]->getPassability())
	 || (map[INDEX((int)floor(pos.x), (int)floor(pos.y - y*1))]->getPassability()))
	{
		return true;
	}
	else	return false;
}


////////////////////////////////////////////////////////////////////////////////////
// vrne 0/1 odvisno od tega, ali smo na ta polozaj ze poslali kakega vojaka
////////////////////////////////////////////////////////////////////////////////////
int Group::GetAvailability(int i, int j)
{
	for (int k=0; k < m_iSoldierCount; k++)
	{
		if (!m_aSoldierItems[k].m_aWaypoints) continue;
		Vector final = m_aSoldierItems[k].m_aWaypoints[m_aSoldierItems[k].m_iWaypointCount-1];
		if ((floor(final.x) == i) && (floor(final.y) == j)) return 0;
	}
	return 1;
}


////////////////////////////////////////////////////////////////////////////////////////
// koliko napredujemo v smeri Waypointa, ce se pomaknemo iz pos proti dest
////////////////////////////////////////////////////////////////////////////////////////
int Group::GetProgression(Vector pos, Vector dest, Vector wp)
{
	float diff = (wp - dest).length();

	float now = (pos - dest).length();

	if (now < diff) diff *= 10;

	int ret = -floor(diff);	// bolj kot je negativna vrednost slabse je
	return ret;
}



////////////////////////////////////////////////////////////////////////////////////////
///  K R I T E R I J S K A    F U N K C I J A  /////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
// tu pride v postev uporaba utezi iz avtomata in ostalo....
////////////////////////////////////////////////////////////////////////////////////////
int Group::Evaluate(int safety, int available, int distance, int progression)
{
	if (available == 0) return 0;

	// safety in [0, 10]
	// available in [0,1]
	// distance in [-diagonalaMape - 0]
	// progression in [-diagonalaMape - 0]
	int ret = safety*m_pAutomat->getWeight (Automaton::WEIGHT_SAFETY)+distance*m_pAutomat->getWeight (Automaton::WEIGHT_DISTANCE)+progression*m_pAutomat->getWeight (Automaton::WEIGHT_PROGRESSION); // MITJA

	return ret;
}


////////////////////////////////////////////////////////////////////////////////////////
// pogledamo, ce je bil vojak soldier ze v okolici tocke (x,y)
////////////////////////////////////////////////////////////////////////////////////////
bool Group::BeenThere(int x, int y, int soldier)
{
	for (int i=0; i < 10; i++)
	{
		if ((abs(m_aSoldierItems[soldier].m_aExWaypoints[i].x - x) < BEEN_THERE) &&
			(abs(m_aSoldierItems[soldier].m_aExWaypoints[i].y - y) < BEEN_THERE))
		{
			return true;
		}
	}
	return false;
}



void Group::CalculateCenter()
{
	Vector res(0,0);
	
	for (int i=0; i< m_iSoldierCount; i++)
	{
		res = res + m_aSoldierItems[i].m_pSoldier->getCoords();
	}

	m_vGroupCenter.x = res.x / m_iSoldierCount;
	m_vGroupCenter.y = res.y / m_iSoldierCount;
}


////////////////////////////////////////////////////////////////////////////////////
// poisce index soldierja sold
////////////////////////////////////////////////////////////////////////////////////
int Group::FindSoldierIndex(Soldier *sold)
{
	int i = 0;
	
	while ((i < m_iSoldierCount) && (m_aSoldierItems[i].m_pSoldier != sold)) i++;
	if (i < m_iSoldierCount) return i;
	else return -1;
}


////////////////////////////////////////////////////////////////////////////////////////////////////////
// vrne ID od grupe
////////////////////////////////////////////////////////////////////////////////////////////////////////
int Group::getID ()
{
	return m_iGroupID;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void Group::readWeights ()
{
	if (m_pAutomat!=NULL)
		m_pAutomat->readWeightsFromFile ();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

Vector Group::GetCenter ()
{
	return m_vGroupCenter;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void Group::attachAutomaton (Automaton *aut1)
{
	m_pAutomat = aut1;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

List * Group::getSoldiers()
{
	return m_pSoldierList;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

Vector Group::getCurrentWayPoint () // MITJA: return waypoint group is moving to
{
	Vector minusOne (-1, -1);

	if ((m_aWaypoints) && (m_MotionType==MOTION_MOVING))
		return m_aWaypoints [m_iCurrentWaypoint];
	else
		return minusOne;
}


/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/// CLASS : ORDERED LIST - urejen seznam integerjev
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////

OrderedList::OrderedList(int count)
{
	m_aKey = new int[count];
	m_aItems1 = new int[count];
	m_aItems2 = new int[count];
	m_aSoldiers = new int[count];
	m_iAllocated = count;
	m_iCount = 0;
	for (int i=0; i< count; i++)
	{
		m_aItems1[i] = -1;
		m_aItems2[i] = -1;
		m_aKey[i] = -1;
		m_aSoldiers[i] = -1;
	}
}

OrderedList::~OrderedList()
{
	if (m_aKey) delete m_aKey;
	if (m_aItems1) delete m_aItems1;
	if (m_aItems2) delete m_aItems2;
	if (m_aSoldiers) delete m_aSoldiers;
	m_aKey = NULL;
	m_aItems1 = NULL;
	m_aItems2 = NULL;
	m_aSoldiers = NULL;
	m_iCount = 0;
	m_iAllocated = 0;
}

int OrderedList::Insert (int key, int val1, int val2, int soldier)
{
	int i=0;
	while ((i < m_iCount) && (m_aKey[i] > key)) i++;
	if (i < m_iCount)
	{
/*		for (int j=m_iCount; j >= i; j--)
		{
			m_aKey[j] = m_aKey[j-1];
			m_aItems1[j] = m_aItems1[j-1];
			m_aItems2[j] = m_aItems2[j-1];
			m_aSoldiers[j] = m_aSoldiers[j-1];
		}
*/		memmove (&m_aKey[i+1], &m_aKey[i], sizeof(int) * (m_iAllocated- i - 1));
		memmove (&m_aItems1[i+1], &m_aItems1[i], sizeof(int) * (m_iAllocated - i - 1));
		memmove (&m_aItems2[i+1], &m_aItems2[i], sizeof(int) * (m_iAllocated - i - 1));
		memmove (&m_aSoldiers[i+1], &m_aSoldiers[i], sizeof(int) * (m_iAllocated - i - 1));
		m_aKey[i] = key;
		m_aItems1[i] = val1;
		m_aItems2[i] = val2;
		m_aSoldiers[i] = soldier;
	}
	else if (m_iCount < m_iAllocated)
	{
		m_aKey[m_iCount] = key;
		m_aItems1[m_iCount] = val1;
		m_aItems2[m_iCount] = val2;
		m_aSoldiers[m_iCount] = soldier;
	}

	m_iCount = min (m_iCount+1, m_iAllocated);
	return i;
}

