// Group.h: interface for the CGroup class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_GROUP_H__25976D80_2C1C_11D5_AB6D_0040F654ADFD__INCLUDED_)
#define AFX_GROUP_H__25976D80_2C1C_11D5_AB6D_0040F654ADFD__INCLUDED_

#include "Vector.hpp"	// Added by ClassView
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "List.hpp"
//#include "Auxiliary.hpp"
#include "Vector.hpp"
#include "VisGraph.hpp"
#include "AiMethods.hpp"
#include "Automaton.hpp"
#include "DangerMap.hpp"

class AiMethods;
class List;

enum tMotion 
{
	
	MOTION_NO_WAYPOINT, 
	MOTION_MOVING
};


class OrderedList
{
public:
	OrderedList(int count);
	~OrderedList();

	int Insert (int key, int val1, int val2, int soldier);

	int * m_aKey;
	int * m_aItems1;
	int * m_aItems2;
	int * m_aSoldiers;
	int m_iCount;
	int m_iAllocated;
};


class CSoldierItem
{
public:
	
	CSoldierItem();
	~CSoldierItem();
	
	void	AddPath (Vector coords, Group *grp);	// MITJA: add path to given coordinates for soldier of given group
	void removePath (); // MITJA: remove soldier's waypoints and stop him

	Soldier * m_pSoldier;		// kazalec na soldierja
	
	int		m_iCurrWaypoint;
	int		m_iWaypointCount;
	int		m_iTimeMoved;		// pove, kdaj se je vojak nazadnje premaknil
	Vector	* m_aWaypoints;
	tMotion	m_MotionState;	// vsak vojak ima doloceno stanje, v katerem je
	Vector  m_aExWaypoints[10];

};


class Group  
{
friend class CSoldierItem;

public:
	Group (int groupID1, VisGraph *visGraph, Environment* env);
	virtual ~Group();
	
	void update();								// updejta koordinate
	
	void RemoveSoldier (int index);				// odstrani vojaka
	void addSoldiers (List * list);				// doda seznam vojakov
	List * getSoldiers ();

	void addWayPoints (List *way);				
	void removeWayPoints (); // MITJA: remove group's waypoints

	Vector getCurrentWayPoint (); // MITJA: return waypoint group is moving to
	int		getID ();
	void	readWeights ();
	Vector	GetCenter ();
	void	attachAutomaton (Automaton *aut1);

	Vector startingCoords; // MITJA: coordinates group way at when addWayPints () was last executed

	Environment * m_pEnv;		// group's environment
	CSoldierItem * m_aSoldierItems;

private:
	bool BeenThere (int x, int y, int soldier);
	int FindSoldierIndex (Soldier *sold);
	int GetProgression (Vector pos, Vector dest, Vector wp);
	int Evaluate (int safety, int available, int distance, int progression);
	int GetAvailability(int i, int j);
	bool FreeArea (Vector pos, int area);

	void CalculateCenter();						// izracuna trenutni center grupe
//	void SendSoldier (int index, Vector dest);	// poslje vojaka na dest
	Vector GetSafePosition (int index);			// najde varno lokacijo za vojaka
	int GetFreeSoldier();						// najde prostega vojaka
	void GetNextCoords(int index);				// uporaba boidov in coll. avoid.

	tMotion	m_MotionType;
	Vector	* m_aWaypoints;		// seznam waypointov, ki jih mora grupa obiskati
	int m_iWaypointCount;		// stevilo waypointov
	
	List		 * m_pSoldierList;
	int m_iSoldierCount;		// stevilo soldierjev v grupi
	int m_aExChoosenSoldiers[3];

	int m_iCurrentWaypoint;		// index, ki pove, kateri waypoint poskusamo trenutno obiskati
	
	int m_iCurrSteps;				// stevilo korakov, ki jih je naredil zadnji vojak
	int m_iStepsBeforeSendingNext;	// stevilo korakov, ki jih naredi vojak, preden posljemo
									// v akcijo naslednjega - ta spr. je lahko odvisna od stanja!!!
	Vector m_vGroupCenter;
		
	int m_iGroupID;
	
	VisGraph	* m_pVisGraph;	// visibility graph
	AiMethods	* m_pAI;		// group's AI methods
	Automaton	* m_pAutomat;	// group's automaton
	DangerMap	  m_DangerMap;

};

#endif // !defined(AFX_GROUP_H__25976D80_2C1C_11D5_AB6D_0040F654ADFD__INCLUDED_)
