
// $Id: List.hpp,v 1.11 2001/04/15 15:05:41 mitja Exp $

/* List class
 *
 * This is a standard list class.
 *
 * Basic methods are available. If anyone needs additional methods, he should contact
 * Damir and they'll be added/implemented.
 *
 * Created by: Damir Arh
 * Created on: 17.11.2000
 */

#ifndef __LIST__
#define __LIST__

#include "ListNode.hpp"

#define MAXBOOKMARKS 10

class List
{
private:

	ListNode * first;                    // first node in the list
	ListNode * current;                  // last returned node (needed for implementation of (remove())
	ListNode * next;                     // the node which will get returned with the next getNext() call
	int count;                           // number of items in the list
	ListNode * bookmarks [MAXBOOKMARKS]; // list of pointers to nodes used as bookmarks

public:
	void addLast (void *item);

	enum tType { SOLDIER, GROUP, BULLET, VECTOR, INT, VERTEX, CSOLDIER, CBULLET, POINT, AOD };

	// constructors and destructors
	List();
	~List(); // does not delete the list contents

	// list methods
	void *getFirst();           // returns first item in the list
	void *getNext();            // returns next item in the list
	void add(void *item);       // adds item at the beginning of the list 
	void remove();              // removes current node from the list
	void empty();               // empties list
	void deleteAll(tType);      // empties the list and frees all stored items
	int getNumItems();          // returns the number of items in the list
	void bookmark(int index);   // store current node into bookmark array
	void *goTo(int index);      // set bookmarked node as current and return its item
	void addList(List *list);   // add items of given list to the beginning and then delete said list
	void merge(List *list);     // adds items of given list to the current list, doesn't allow duplicates
	bool contains (void *item); // MITJA: returns true if the list contains given item, false otherwise
};

#endif // __LIST__
