
// $Id: Log.cpp,v 1.4 2001/04/08 15:42:52 mitja Exp $

/* Log class implementation
 *
 * Created by: Damir Arh
 * Created on: 7.1.2001
 */

#include "stdafx.h"
#include <stdio.h>
#include <stdarg.h>
#include "Log.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/* Log(char *filename, bool append)
 *
 * Parameters:
 *    filename : name of the log file to log to
 *    append   : whether entries should be appended to an existing log or a new log
 *               should be created
 *
 * Tries to open the log with specified attributes. You should check if it succeeded
 * afterwards with isReady() method.
 */
Log::Log(char *filename, bool append)
{
	if (append)
		handle = fopen(filename, "a");
	else
		handle = fopen(filename, "w");
	enabled = true;
}


/* ~Log()
 *
 * Closes the log properly.
 */
Log::~Log()
{
	if (handle)
		fclose(handle);
}


/* isReady()
 *
 * Result:
 *   bool : is the log ready for logging?
 *
 * Returns TRUE if the log is ready to write to.
 */
bool Log::isReady()
{
	return (handle != NULL);
}


/* print(char *string)
 *
 * Parameters:
 *   string : the string to write to the log
 *
 * Writes the string with an appended newline to the log if the log is
 * ready, does nothing if it's not.
 */
void Log::print(char *string,...)
{
	if ((enabled) && (handle))
	{
		va_list args;

		if (string == NULL)
			return;

		va_start(args, string);
			vfprintf(handle, string, args);
		va_end(args);

		fprintf(handle, "\n");
	}
}


/* enable(bool choice)
 *
 * Parameters:
 *   choice : enable or disable logging
 *
 * Enables or disables logging for this logfile.
 */
void Log::enable(bool choice)
{
	enabled = choice;
}


/* isEnabled()
 *
 * Result:
 *   bool : is logging enabled?
 *
 * Returns TRUE if logging for this file is enabled.
 */
bool Log::isEnabled()
{
	return enabled;
}
