
// $Id: Order.cpp,v 1.6 2001/04/11 06:49:24 damir Exp $

#include "stdafx.h"
#include <stdlib.h>
#include <string.h>
#include "Order.hpp"
#include "soldierclient.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

Order::Order()
{
	ready = 0;
	items = NULL;
	groups=NULL;
}


/*The function pack() packs the contents of the class into a buffer
 *and returns a pointer to that buffer. It also returns the length of the
 *package in bytes.
 *
 *The format of the package is: first a structure which contains the
 *information about the order(order type and number of soldiers or number of
 *waypoints for the order, and the groupID). If the order is GROUP there follow
 *the IDs of the soldiers in the new group, if the order is MOVE we have the
 *waypoints for the group
 *GROUPUPATE is an order which send the new groups to the server
 */
void *Order::pack(int *length)
{
	char *buf;
	int i;
	OrderPack ord;
	PathPack p;
	SoldierClient *temp;
	TUI_pointStruct* point;
	int len=0;


	// packing the order

	ord.orderType = orderType;
	
	ord.groupID=groupId;


	switch(orderType)
	{	
		case AGRESS:
			len = items->getNumItems();
			*length = sizeof(OrderPack) + len * sizeof(int);
			ord.numItems = len;
			buf = (char *) malloc(sizeof(OrderPack) + len * sizeof(int));
			memcpy(&buf[0], &ord, sizeof(OrderPack));


			i = 0;
			temp = (SoldierClient *) items->getFirst();
			while (temp != NULL)
			{
				int id = temp->GetSoldierId();
				memcpy(&buf[sizeof(OrderPack) + i * sizeof(int)], &id , sizeof(int));
				temp = (SoldierClient*)items->getNext();
				i++;
			}
			break;
		case ADDAOD:
			//we are sending just the last AOD that was created
			AreaOfDanger *area;
			AODpack ap;
			len = 1;
			ord.numItems = len;
			*length = sizeof(OrderPack) + len * sizeof(AODpack);
			buf = (char *) malloc(sizeof(OrderPack) + len * sizeof(AODpack));
			memcpy(&buf[0], &ord, sizeof(OrderPack));

			// packing the AODs
			area = (AreaOfDanger *) items->getFirst();
			if(area != NULL )
			{
				ap.x = area->x/50.0f;
				ap.y = area->y/50.0f;
				ap.r = area->r/50.0f;
				memcpy(&buf[sizeof(OrderPack)], &ap , sizeof(AODpack));
			}
			break;
		case REAOD:
			ord.numItems = 0;
			*length = sizeof(OrderPack);
			buf = (char *) malloc(sizeof(OrderPack));
			memcpy(&buf[0], &ord, sizeof(OrderPack));
			break;


		case NEWGROUP:
			len = items->getNumItems();
			ord.numItems = len;
			*length = sizeof(OrderPack) + len * sizeof(int);
			buf = (char *) malloc(sizeof(OrderPack) + len * sizeof(int));
			memcpy(&buf[0], &ord, sizeof(OrderPack));

			// packing the soldiers IDs
			i = 0;
			temp = (SoldierClient *) items->getFirst();
			while (temp != NULL)
			{
				int id = temp->GetSoldierId();
				memcpy(&buf[sizeof(OrderPack) + i * sizeof(int)], &id , sizeof(int));
				temp = (SoldierClient*)items->getNext();
				i++;
			}
			break;
		case MOVE:
			len = items->getNumItems();
			*length = sizeof(OrderPack) + len * sizeof(PathPack);
			ord.numItems = len;
			buf = (char *) malloc(sizeof(OrderPack) + len * sizeof(PathPack));
			memcpy(&buf[0], &ord, sizeof(OrderPack));

			//packing the waypoints
			i = 0;
			point = (TUI_pointStruct *) items->getFirst();
			while (point != NULL)
			{
				p.x = point->x / 50;
				p.y = point->y / 50;
  				memcpy(&buf[sizeof(OrderPack) + i * sizeof(PathPack)], &p , sizeof(PathPack));
				point = (TUI_pointStruct*)items->getNext();
				i++;
			}
			break;
		case SHOOT:
			len = items->getNumItems();
			ord.numItems = len;
			*length = sizeof(OrderPack) + sizeof(PathPack);
			buf = (char *) malloc(sizeof(OrderPack) + sizeof(PathPack));
			memcpy(&buf[0], &ord, sizeof(OrderPack));

			//packing the cordinates for the shot
			point = (TUI_pointStruct *) items->getFirst();
			p.x = point->x;
			p.y = point->y;
			memcpy(&buf[ sizeof(OrderPack) ], &p , sizeof(PathPack));
			delete items;
			break;
		case GROUPUPDATE:
			//first we must get the total size of the package
			int size = 0;
			int nonempty = 0;
			for(i=0;i<10;i++)
			{
				size+=groups[i]->getNumItems();
				if(groups[i]->getNumItems()!=0)nonempty++;
			}
			//then we pack the groups (groupID|numberOfSoldiersInGroup|ID|ID|ID....|groupID...)
			*length = sizeof(OrderPack)+size*sizeof(int)+2*nonempty*sizeof(int);
			ord.numItems = nonempty;
			buf = (char*) malloc(sizeof(OrderPack)+size*sizeof(int)+2*nonempty*sizeof(int));
			memcpy(&buf[0], &ord, sizeof(OrderPack));
			int setof = sizeof(OrderPack);
			for(i=0;i<10;i++)
			{
				temp=(SoldierClient*)groups[i]->getFirst();
				if(temp!=NULL)
				{
					memcpy(&buf[setof],&i,sizeof(int));
					setof+=sizeof(int);
					int numSol = groups[i]->getNumItems(); 
					memcpy(&buf[setof],&numSol,sizeof(int));
					setof+=sizeof(int);
					while(temp!=NULL)
					{
						int id=temp->GetSoldierId();
						memcpy(&buf[setof],&id,sizeof(int));
						setof+=sizeof(int);
						temp = (SoldierClient*)groups[i]->getNext();
					}
				}
			}
			break;
	
	}
	return buf;
}
