
// $Id: PicCollection.cpp,v 1.4 2001/04/08 15:42:52 mitja Exp $


#include "stdafx.h"
#include "Errors.hpp"
#include "PicCollection.hpp"
#include "SDL.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/*
*PicCollection::PicCollection(const char* fileName, int aSizeX, int aSizeY)
*Desc: Main constructor, also creates the picture surface by loading the pic
*      and intializes sizeX, sizeY to given values
*In:   fileName is the bitmap file to be loaded
*      aSizeX, aSizeY are the default values for cropping the image
*/
PicCollection::PicCollection(const char* fileName, int aSizeX, int aSizeY)
{
	sizeX = aSizeX;
	sizeY = aSizeY;
	
	picture = SDL_LoadBMP(fileName);
	
	if (picture == NULL)
		throw EErrorLoadingBMP();
}

/*
*void PicCollection::GetPic(int index, int *x, int *y, const SDL_Surface *surface)
*Desc: returns the coordinates and surface for a picture with a given index. This 
*      way, the calling func can directly blit to dest surface
*In:   index is the index of a picture
*Out:  x, y are the top - left coordinates of the pic, pic rect is(x, y, sizeX, sizeY)
*      surface is picture surface
*/
void PicCollection::GetPic(int index, int *x, int *y, SDL_Surface **surface)
{
	int numPicsX = (int)((picture->w) / sizeX);
	
	if (index / numPicsX > (int)(picture->h / sizeY))
		throw EGetPictureIndexOOR();
	
	*y = sizeY * (int)(index / numPicsX);
	*x = sizeX * (index % numPicsX);
	
	*surface = picture;
}

/*
*int PicCollection::PicCount()
*Desc: returns number of all images or  how many tiles of sizeX*sizeY can i put in
*      the picture surface
*Out:  number of pics
*/
int	PicCollection::PicCount()
{
	return (int)(picture->w / sizeX) *(int)(picture->h / sizeY);
}

/*
*PicCollection::~PicCollection()
*Desc: main destructor, destroys picture surface
*/
PicCollection::~PicCollection()
{
	SDL_FreeSurface(picture);
}
