// $Id: Soldier.cpp,v 1.13 2001-04-07 18:13:38+02 damir Exp damir $

/* Soldier.cpp, 2.8, 2001-05-12, Mitja Lustrek
 *
 * This code implements the very thing that is so aptly described in its header file ;-)
 */

#include "stdafx.h"
#include <stdlib.h>
#include <math.h>
#include "Soldier.hpp"
#include "Environment.hpp"
#include "Auxiliary.hpp"

#include "TextDisplay.hpp" //!!!

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define MAXTMOVEMENT 3
#define MAXTSHOOT 4
#define PI 3.141592654f

extern TextDisplay *mainDisplay; //!!!

////////////////////////////////////////////////////////////////////////////////////////////////////////

Soldier::Soldier (int soldierID1, int aiID1, Vector coords1, float angle)
{
	soldierID=soldierID1;
	aiID=aiID1;

	radius=0.3f;
	maxSpeed=6.0f;
	backSpeedFactor=0.6f;
	accuracyTangens=0.003f;
	rotationMoveSpeed=2.0f*PI;
	rotationViewSpeed=4.0f*PI;
	acceleration=3.0f;
	decceleration=5.0f;
	leftShootLimit=0.125f*PI;
	rightShootLimit=0.5f*PI;
	timeLastTurn=-1.0f;
	tick=0.0f;
	speedFactor=new float [MAXTMOVEMENT];
	speedFactor [0]=1.0f;
	speedFactor [1]=0.6f;
	speedFactor [2]=0.15f;
	movementChangeDelay=new float [MAXTMOVEMENT*MAXTMOVEMENT];
	movementChangeDelay [0]=0.0f;
	movementChangeDelay [1]=0.2f;
	movementChangeDelay [2]=0.8f;
	movementChangeDelay [3]=0.2f;
	movementChangeDelay [4]=0.0f;
	movementChangeDelay [5]=0.7f;
	movementChangeDelay [6]=1.5f;
	movementChangeDelay [7]=1.2f;
	movementChangeDelay [8]=0.0f;
	hitRadiusFactor=new float [MAXTMOVEMENT];
	hitRadiusFactor [0]=1.0f;
	hitRadiusFactor [1]=0.7f;
	hitRadiusFactor [2]=0.3f;
	shootDelay=new float [MAXTSHOOT];
	shootDelay [0]=3.0f;
	shootDelay [1]=1.0f;
	shootDelay [2]=0.1f;
	shootDelay [3]=0.0f;
	accuracyFactor=new float [MAXTSHOOT];
	accuracyFactor [0]=0.5f;
	accuracyFactor [1]=1.0f;
	accuracyFactor [2]=3.0f;
	accuracyFactor [3]=0.0f;
	fieldOfViewLimits=new float [2];
	fieldOfViewLimits [0]=PI/3.0f;
	fieldOfViewLimits [1]=0.9f*PI;

	coords=coords1;
	oldCoords.x = -1;
	oldCoords.y = -1;
	moveAngle=angle;
	viewAngle=moveAngle;
	speed=0.0f;
	moveType=MOVEMENT_WALK;

	desiredViewAngle=viewAngle;
	desiredViewAngleRel=-10;
	desiredViewCoords.x=-1;
	desiredMoveAngle=moveAngle;
	desiredSpeed=speed;
	desiredMoveType=moveType;
	desiredMaxLeftAngleDifference=0;
	desiredMaxRightAngleDifference=0;
	desiredCanReverse=false;
	timeMoveChanged=-1.0f;
	desiredShootType=SHOOT_NONE;
	timeLastShoot=-1.0f;

	health=255;
	agression=5;
	shotAtAngle=-1.0f;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

Soldier::~Soldier ()
{
	if (speedFactor) delete speedFactor;
	if (movementChangeDelay) delete movementChangeDelay;
	if (hitRadiusFactor) delete hitRadiusFactor;
	if (shootDelay) delete shootDelay;
	if (accuracyFactor) delete accuracyFactor;
	if (fieldOfViewLimits) delete fieldOfViewLimits;

	speedFactor = NULL;
	movementChangeDelay = NULL;
	hitRadiusFactor = NULL;
	shootDelay = NULL;
	accuracyFactor = NULL;
	fieldOfViewLimits = NULL; 
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void Soldier::setViewAngle (float angle)
{
	desiredViewAngle=angleFormat (angle);
	desiredViewAngleRel=-10;
	desiredViewCoords.x=-1;
}


////////////////////////////////////////////////////////////////////////////////////////////////////////

void Soldier::setViewAngleRel (float angle)
{
	if (angle>0)
		desiredViewAngleRel=MIN (PI, angle);
	else
		desiredViewAngleRel=MAX (-PI, angle);
	desiredViewCoords.x=-1;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void Soldier::setViewAnglePoint (Vector coords1)
{
	desiredViewCoords=coords1;
	desiredViewAngleRel=-10;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void Soldier::setMoveAngle (float angle) {
	desiredMoveAngle=angle;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void Soldier::setSpeed (float speed1)
{
	desiredSpeed=speed1;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////

void Soldier::setMoveType (tMovement type)
{
	desiredMoveType=type;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void Soldier::setMaxAngleDifference (float difference, bool reverse)
{
	desiredMaxLeftAngleDifference=difference;
	desiredMaxRightAngleDifference=difference;
	desiredCanReverse=reverse;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void Soldier::setMaxAngleDifference (tAngleDifference difference, bool reverse)
{
	switch (difference)
	{
		case DIFFERENCE_SHOOT_UNIMPEDED:
			desiredMaxLeftAngleDifference=leftShootLimit;
			desiredMaxRightAngleDifference=rightShootLimit;
			break;
		case DIFFERENCE_SHOOT_IMPEDED:
			desiredMaxLeftAngleDifference=leftShootLimit+0.25f*PI;
			desiredMaxRightAngleDifference=rightShootLimit+0.25f*PI;
			break;
	}
	desiredCanReverse=reverse;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void Soldier::setMoveVector (Vector moveVector)
{
	Vector zero;
	
	setSpeed (moveVector.length ());
	zero.x=0;
	zero.y=0;
	setMoveAngle (angleOfVector (zero, moveVector));
}


////////////////////////////////////////////////////////////////////////////////////////////////////////

void Soldier::setAgression (int agr)
{
	agression=agr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void Soldier::shoot (tShoot type)
{
	if (desiredShootType==SHOOT_NONE)
		timeLastShoot=timeLastTurn;
  desiredShootType=type;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void Soldier::setShotAtAngle(float angle)
{
	shotAtAngle=angle;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void Soldier::setIsShot (float hitAccuracy)
{
  mainDisplay->display ("%d: %lf", soldierID, hitAccuracy); //!!!
	health=(int)(MAX (0.0f, health-30.0-hitAccuracy*97.5));
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

Vector Soldier::getCoords ()
{
	return coords;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

Vector Soldier::getOldCoords ()
{
	return oldCoords;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

Vector Soldier::getNewCoords (float time)
{
	float delta, reverse, correctedDesiredMoveAngle=desiredMoveAngle, correctedDesiredViewAngle=desiredViewAngle, correctedDesiredSpeed=desiredSpeed;
	bool forceLeftMove=false, forceRightMove=false, forceLeftView=false, forceRightView=false;

	// Store coords as oldCoords
	oldCoords = coords;

	// Calculate interval from last update
	if (timeLastTurn>0)
		tick=time-timeLastTurn;
	timeLastTurn=time;

	// Calculate preliminary viewAngle (for purpose of reversing direction of movement) if it is determined from relative angle of view
	if (desiredViewAngleRel>-9)
		desiredViewAngle=angleFormat (desiredMoveAngle+desiredViewAngleRel);

	// Calculate viewAngle if it is determined from coordinates soldier is to look towards
	if (desiredViewCoords.x>=0)
		desiredViewAngle=angleOfVector (coords, desiredViewCoords);
	
	// Reverse direction of movement if necessary
	if (desiredCanReverse)
		if (angleDifference (desiredMoveAngle, desiredViewAngle)>MIN (0.75f*PI, desiredMaxRightAngleDifference))
		{
			reverse=angleFormat (desiredMoveAngle+PI);
			if (angleDifference (reverse, desiredViewAngle)>-MIN (0.75f*PI, desiredMaxLeftAngleDifference))
			{
				correctedDesiredMoveAngle=reverse;
				correctedDesiredSpeed=-desiredSpeed;
				if (fabs (angleDifference (moveAngle, correctedDesiredMoveAngle))>0.99f*PI)
					forceRightMove=true;
			}
		}
		else
			if (angleDifference (desiredMoveAngle, desiredViewAngle)<-MIN (0.75f*PI, desiredMaxLeftAngleDifference))
			{
				reverse=angleFormat (desiredMoveAngle+PI);
				if (angleDifference (reverse, desiredViewAngle)<MIN (0.75f*PI, desiredMaxRightAngleDifference))
				{
					correctedDesiredMoveAngle=reverse;
					correctedDesiredSpeed=-desiredSpeed;
					if (fabs (angleDifference (moveAngle, correctedDesiredMoveAngle))>0.99f*PI)
						forceLeftMove=true;
				}
			}

	// Set moveAngle
	if (correctedDesiredMoveAngle!=moveAngle)
	{
		delta=angleDifference (moveAngle, correctedDesiredMoveAngle);
		if ((forceLeftMove) && (delta>0))
			delta-=2.0f*PI;
		else
			if ((forceRightMove) && (delta<0))
				delta+=2*PI;
		if (delta>0)
			moveAngle=angleFormat (moveAngle-MIN (rotationMoveSpeed*tick*speedFactor [moveType], delta));
		else
			moveAngle=angleFormat (moveAngle+MIN (rotationMoveSpeed*tick*speedFactor [moveType], -delta));
	}

	// Calculate final viewAngle if it is determined from relative angle of view
	if (desiredViewAngleRel>-9)		
		if (fabs (angleDifference (desiredMoveAngle, correctedDesiredMoveAngle))<3)
			correctedDesiredViewAngle=angleFormat (moveAngle+desiredViewAngleRel);
		else
			correctedDesiredViewAngle=angleFormat (moveAngle+PI+desiredViewAngleRel);

	// Correct viewAngle if necessary
	if (angleDifference (moveAngle, correctedDesiredViewAngle)>MIN (0.75f*PI, desiredMaxRightAngleDifference))
		correctedDesiredViewAngle=angleFormat (moveAngle-MIN (0.75f*PI, desiredMaxRightAngleDifference));
	else
		if (angleDifference (moveAngle, correctedDesiredViewAngle)<-MIN (0.75f*PI, desiredMaxLeftAngleDifference))
			correctedDesiredViewAngle=angleFormat (moveAngle+MIN (0.75f*PI, desiredMaxRightAngleDifference));
	if ((angleDifference (viewAngle, correctedDesiredViewAngle)>0) && (angleDifference (correctedDesiredViewAngle, moveAngle)>0) && (angleDifference (moveAngle, viewAngle)>0))
		forceLeftView=true;
	else
		if ((angleDifference (correctedDesiredViewAngle, viewAngle)>0) && (angleDifference (viewAngle, moveAngle)>0) && (angleDifference (moveAngle, correctedDesiredViewAngle)>0))
			forceRightView=true;
		
	// Set viewAngle 
	if (correctedDesiredViewAngle!=viewAngle)
	{
		delta=angleDifference (viewAngle, correctedDesiredViewAngle);
		if ((forceLeftView) && (delta>0))
			delta-=2.0f*PI;
		else
			if ((forceRightView) && (delta<0))
				delta+=2*PI;
		if (delta>0)
			viewAngle=angleFormat (viewAngle-MIN (rotationViewSpeed*tick, delta));
		else
			viewAngle=angleFormat (viewAngle+MIN (rotationViewSpeed*tick, -delta));
	}
	
	// Set movement type if possible
	if (desiredMoveType!=moveType)
	{
		if ((time-timeMoveChanged>=movementChangeDelay [moveType*MAXTMOVEMENT+desiredMoveType]) && (fabs (speed)<=fabs (calculateMaxSpeed (desiredMoveType))))
			moveType=desiredMoveType;
	}

	// Set speed if possible
	if (correctedDesiredSpeed!=speed)
		if (speed>0)
			if (correctedDesiredSpeed>speed)
				speed=MIN (speed+acceleration*tick, MIN (correctedDesiredSpeed, calculateMaxSpeed (moveType)));
			else
				speed=MAX (speed-decceleration*tick, correctedDesiredSpeed);
		else
			if (correctedDesiredSpeed<speed)
				speed=MAX (speed-acceleration*tick, MIN (correctedDesiredSpeed, calculateMaxSpeed (moveType)));
			else
				speed=MIN (speed+decceleration*tick, correctedDesiredSpeed);

	// Set coords
	if (speed!=0)
	{
		coords.x+=(float)(cos (moveAngle)*speed*tick);
		coords.y+=(float)(sin (moveAngle)*speed*tick);
	}

	// Return coordinates to be moved to
	return getCoords ();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void Soldier::stop (Vector coords1)
{
	if (speed>0.5f*maxSpeed) health-=(int)((speed-0.5f*maxSpeed)*10.0f);
	speed=0.0f;
	desiredSpeed=0.0f;
	coords=coords1;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

float Soldier::getRadius ()
{
	return radius;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

float Soldier::getHitRadius ()
{
	return radius*hitRadiusFactor [moveType];
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

int Soldier::getHealth ()
{
	return health;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

Bullet *Soldier::getBullet ()
{
	Vector bulletCoords;
	float accuracyTangens;

	if (desiredShootType!=SHOOT_NONE)
	{
		accuracyTangens=calculateAccuracyTangens ();
		if ((timeLastTurn-timeLastShoot>=shootDelay [desiredShootType]) && (accuracyTangens>=0))
		{
			desiredShootType=SHOOT_NONE;
			bulletCoords.x=coords.x+(float)((radius+maxSpeed*tick)*1.01*cos (viewAngle));
			bulletCoords.y=coords.y+(float)((radius+maxSpeed*tick)*1.01*sin (viewAngle));
			return new Bullet (bulletCoords, viewAngle+(float)((randomGauss ()-0.5)*2.0*atan (accuracyTangens)), timeLastTurn);
		}
		else
			return NULL;
	}
	else
		return NULL;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

int Soldier::getID ()
{
	return soldierID;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

float Soldier::getMoveAngle ()
{
	return moveAngle;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

float Soldier::getViewAngle ()
{
	return viewAngle;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

float Soldier::getSpeed ()
{
	return speed;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

Soldier::tMovement Soldier::getMoveType ()
{
	return moveType;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

float Soldier::getFieldOfView ()
{
	return fieldOfViewLimits [1]-(fieldOfViewLimits [1]-fieldOfViewLimits [0])*(float)(fabs (speed)/maxSpeed);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

int Soldier::getAiID ()
{
	return aiID;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

Vector Soldier::getMoveVector ()
{
	Vector moveVector;

	moveVector.x=(float)(cos (moveAngle));
	moveVector.y=(float)(sin (moveAngle));
	moveVector=moveVector*(float)(fabs (speed));
	return moveVector;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

int Soldier::getAgression()
{
	return agression;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

float Soldier::calculateMaxSpeed (tMovement type)
{
	float result;

	result=(float)(MIN (maxSpeed, (maxSpeed-(fabs (angleDifference (moveAngle, viewAngle))-PI/6.0f)*0.666f*maxSpeed/(0.583f*PI)))*speedFactor [type]);
	if (health>50.0f) result=MIN (result, result*(health-50.0f)/155.0f);
	if (speed<0) result*=-backSpeedFactor;
	return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

float Soldier::calculateAccuracyTangens ()
{
	float result, delta;

	result=-1;
	delta=angleDifference (moveAngle, viewAngle);
	if ((delta>=-leftShootLimit) && (delta<=rightShootLimit)) result=accuracyTangens;
	if ((delta<-leftShootLimit) && (delta>=-leftShootLimit-0.31f*PI)) result=accuracyTangens-(delta+leftShootLimit)*0.00382f;
	if ((delta>rightShootLimit) && (delta<=rightShootLimit+0.31f*PI)) result=accuracyTangens+(delta-rightShootLimit)*0.00382f;
	result*=(1.0f+0.5f*(float)(fabs (speed)))*accuracyFactor [desiredShootType];
	result=MAX (result, result*(2.0f-health/205.0f));
	return result;
}