
// $Id: SoldierClient.cpp,v 1.8 2001/04/08 20:52:12 mitja Exp $


#include "SDL.h"
#include "SoldierClient.hpp"
#include "TuiGraphics.hpp"
#include "Auxiliary.hpp"
#include "stdafx.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

SoldierClient::SoldierClient(PicCollection *aMaps, int *aPics[], int aSoldierId, MyColor color)
{
	lastX = -1;
	lastY = -1;
	lastMoveAngle = -1;
	lastViewAngle = -1;
	soldierId = aSoldierId;
	radius = 25;
	aiID = -1;
	FOV = 0;
	this->colorName = color;
	selected = false;
	friendly = (color == clBlue);
	updateFlag = true;
	health = 1.0;
	agr = 0.5;
	groupIndex = -1;
	bodyPosition = 0;
}


/* 
* void SoldierClient::DrawSoldier(Viewport* viewport, SDL_Surface *surface) 
* Desc: Draws soldier onto surface using some viewport properties
* In:   viewport is the base viewport to draw the soldier
*       surface is the destination screen
*/
void SoldierClient::DrawSoldier(Viewport* viewport, SDL_Surface *surface){
	float r =((float)radius * viewport->GetZoom());	
	const int BORDER_MOVE = 0;
	const int BORDER_LOOK =(int)(r * 0.5 + 1);
	const int ARROW_X =(int)(r * 0.2 + 1);
	const int ARROW_Y =(int)(r * 0.3+ 1);
	const int NUM_POINTS_MOVE = 4;
	const int NUM_POINTS_VIEW = 3;
	const int NUM_POINTS_HEALTH = 3;
	const int NUM_POINTS_AGR = 3;
	const int NUM_POINTS_GROUP = 1;	
	const int NUM_POINTS_BODYPOS = 1;	
	const int VIEW_RANGE = 400;
	int i = 0;
	float drawX =(GetX() - viewport->posX) * viewport->GetZoom() + viewport->left;
	float drawY =(GetY() - viewport->posY) * viewport->GetZoom() + viewport->top;
	float range = VIEW_RANGE * viewport->GetZoom();
	float healthHeight = (r / 3);
	float agrHeight = (r / 3);
	Uint32 colorFOV = SDL_MapRGB(surface->format, 150, 150, 255);
		
	const Uint32 color = MapColor(colorName, surface->format);
	
	TUI_pointStruct	points[NUM_POINTS_MOVE + NUM_POINTS_VIEW + NUM_POINTS_HEALTH + NUM_POINTS_AGR + NUM_POINTS_GROUP + NUM_POINTS_BODYPOS];
	// END INIT
	
 for (i=0; i<(int)(r*0.6f); i+=1)
		TUI_Circle(surface, TUI_Point(drawX, drawY), (float)i, color, fxOverride); // draw a circle
	
	//move
	points[0] = TUI_Point(drawX, drawY - r + BORDER_MOVE);
	points[1] = TUI_Point(drawX, drawY - r);	
	points[2] = TUI_Point(drawX - ARROW_X, points[0].y + ARROW_Y);
	points[3] = TUI_Point(drawX + ARROW_X, points[0].y + ARROW_Y);
	
	//fov
	points[4] = TUI_Point(drawX, drawY);
	points[5] = TUI_Point(drawX + range * (float)sin(FOV/2), drawY - range * (float)cos(FOV/2));
	points[6] = TUI_Point(drawX - range * (float)sin(FOV/2), drawY - range * (float)cos(FOV/2));

	//health
	points[7] = TUI_Point(drawX, drawY - r - healthHeight);	
	points[8] = TUI_Point(drawX + health * (r), drawY - r);	//adjusted length of line
	points[9] = TUI_Point(drawX + r, drawY - r);	//adjusted length of line

	//agr
	points[10] = TUI_Point(drawX - r, drawY + r);	
	points[11] = TUI_Point(drawX + agr * (r) - r, drawY + r + agrHeight);	//adjusted length of line
	points[12] = TUI_Point(drawX , drawY + r + agrHeight);	//adjusted length of line

	//group
	points[13] = TUI_Point(drawX + r/2, drawY + r);	//adjusted length of line

	//body pos
	points[14] = TUI_Point(drawX - r, drawY - r - healthHeight);	//adjusted length of line
	
	for (i = 0; i < NUM_POINTS_MOVE; i++) // rotate the move arrow
		points[i] = TUI_Rotate(points[i], TUI_Point(drawX, drawY), moveAngle);
	
	for (i = NUM_POINTS_MOVE; i < NUM_POINTS_MOVE + NUM_POINTS_VIEW; i++) // rotate the field of view
		points[i] = TUI_Rotate(points[i], TUI_Point(drawX, drawY), viewAngle);
	
	TUI_Line(points[1], points[2], color, surface, fxOverride);	//soldier
	TUI_Line(points[1], points[3], color, surface, fxOverride);
	TUI_Line(points[0], points[2], color, surface, fxOverride);
	TUI_Line(points[0], points[3], color, surface, fxOverride);	

	TUI_Line(points[4], points[5], colorFOV, surface, fxOverride);	//fov
	TUI_Line(points[5], points[6], colorFOV, surface, fxOverride);
	TUI_Line(points[6], points[4], colorFOV, surface, fxOverride);

	//health
	float red_val = MIN(255, health * 255 * 2);	
	float green_val = MIN(255, (1 - health) * 255 * 2);		//adjusted color of the line
	TUI_Rect(
		surface, 
		points[7], 
		points[8], 
		SDL_MapRGB(surface->format,  (int)green_val, (int)(red_val),  0),
		fxOverride, 
		true
	);
	TUI_Rect(
		surface, 
		points[7], 
		points[9], 
		SDL_MapRGB(surface->format, 0, 0,  0),
		fxOverride, 
		false
	);
	
	//agr
	red_val = MIN(255, agr * 255 * 2);	
	green_val = MIN(255, (1 - agr) * 255 * 2);		//adjusted color of the line
	TUI_Rect(
		surface, 
		points[10], 
		points[11], 
		SDL_MapRGB(surface->format, (int)(red_val), (int)green_val,  0),
		fxOverride, 
		true
	);
	TUI_Rect(
		surface, 
		points[10], 
		points[12], 
		SDL_MapRGB(surface->format, 0, 0,  0),
		fxOverride, 
		false
	);

	if (groupIndex != -1)
		TUI_Numeral(groupIndex, surface, points[13], 0, fxOverride);

	TUI_BodyPosition(bodyPosition, surface, points[14], 0, fxOverride);

	if (selected)	
		DrawSelectedBox(viewport, surface);
}


void SoldierClient::DrawSelectedBox(Viewport* viewport, SDL_Surface *surface)
{
	float r =(float)GetRadius() * viewport->GetZoom();
	float drawX =(GetX() - viewport->posX) * viewport->GetZoom() + viewport->left;
	float drawY =(GetY() - viewport->posY) * viewport->GetZoom() + viewport->top;
	
	const int CORNER_LENGTH = (int)((float)0.5 * r + 0.5);
	TUI_pointStruct points[8];
	Uint32 color = SDL_MapRGB(surface->format, 255, 128, 0);
	
	r = r *(float)0.95;
	points[0] = TUI_Point(drawX - r, drawY - r);
	points[1] = TUI_Point(drawX + r, drawY - r);
	points[2] = TUI_Point(drawX - r, drawY + r);
	points[3] = TUI_Point(drawX + r, drawY + r);
	
	// four corner lines
	TUI_Line(points[0], TUI_Point(points[0].x + CORNER_LENGTH, points[0].y), color, surface, fxOverride);
	TUI_Line(points[0], TUI_Point(points[0].x, points[0].y + CORNER_LENGTH), color, surface, fxOverride);
	
	TUI_Line(points[1], TUI_Point(points[1].x - CORNER_LENGTH, points[1].y), color, surface, fxOverride);
	TUI_Line(points[1], TUI_Point(points[1].x, points[1].y + CORNER_LENGTH), color, surface, fxOverride);
	
	TUI_Line(points[2], TUI_Point(points[2].x + CORNER_LENGTH, points[2].y), color, surface, fxOverride);
	TUI_Line(points[2], TUI_Point(points[2].x, points[2].y - CORNER_LENGTH), color, surface, fxOverride);
	
	TUI_Line(points[3], TUI_Point(points[3].x - CORNER_LENGTH, points[3].y), color, surface, fxOverride);
	TUI_Line(points[3], TUI_Point(points[3].x, points[3].y - CORNER_LENGTH), color, surface, fxOverride);
	
	r = r *(float)0.8;
	points[0] = TUI_Point(drawX - r, drawY - r);
	points[1] = TUI_Point(drawX + r, drawY - r);
	points[2] = TUI_Point(drawX - r, drawY + r);
	points[3] = TUI_Point(drawX + r, drawY + r);
	// another four lines
	TUI_Line(points[0], TUI_Point(points[0].x + CORNER_LENGTH, points[0].y), color, surface, fxOverride);
	TUI_Line(points[0], TUI_Point(points[0].x, points[0].y + CORNER_LENGTH), color, surface, fxOverride);
	
	TUI_Line(points[1], TUI_Point(points[1].x - CORNER_LENGTH, points[1].y), color, surface, fxOverride);
	TUI_Line(points[1], TUI_Point(points[1].x, points[1].y + CORNER_LENGTH), color, surface, fxOverride);
	
	TUI_Line(points[2], TUI_Point(points[2].x + CORNER_LENGTH, points[2].y), color, surface, fxOverride);
	TUI_Line(points[2], TUI_Point(points[2].x, points[2].y - CORNER_LENGTH), color, surface, fxOverride);
	
	TUI_Line(points[3], TUI_Point(points[3].x - CORNER_LENGTH, points[3].y), color, surface, fxOverride);
	TUI_Line(points[3], TUI_Point(points[3].x, points[3].y - CORNER_LENGTH), color, surface, fxOverride);
}


/* These are some Get/Set properties for protected members. Set methods save
*  previous values so they can later be used for interpolation
*/
float SoldierClient::GetX()
{
	return x;
}


void SoldierClient::SetX(float value)
{
	if (x != value)
	{
		lastX = x;
		x = value;
	}	// if
}


float SoldierClient::GetY()
{
	return y;
}


void SoldierClient::SetY(float value)
{
	if (y != value)
	{
		lastY = y;
		y = value;
	}	// if
}


float SoldierClient::GetMoveAngle()
{
	return moveAngle;
}


void SoldierClient::SetMoveAngle(float value)
{
	if (moveAngle != value)
	{
		lastMoveAngle = moveAngle;
		moveAngle = value;
	}	// if
}


float SoldierClient::GetViewAngle()
{
	return viewAngle;
}


void SoldierClient::SetViewAngle(float value)
{
	if (viewAngle != value)
	{
		lastViewAngle = viewAngle;
		viewAngle = value;
	}	// if
}

float SoldierClient::GetFOV()
{
	return FOV;
}


void SoldierClient::SetFOV(float value)
{
	if (FOV != value)
	{
		lastFOV = FOV;
		FOV = value;
	}	// if
}


float SoldierClient::GetRadius()
{
	return radius;
}


void SoldierClient::SetRadius(float value)
{
	if (radius != value)
		radius = value;
}


int SoldierClient::GetSoldierId()
{
	return soldierId;
}

bool SoldierClient::isFriendly()
{
	return friendly;
}

float SoldierClient::GetHealth(){
	return health;
}

void  SoldierClient::SetHealth(float value){
	if (value >=0.0 && value <=1.0)
		health = value; 
}

float SoldierClient::GetAgr(){
	return agr;
}

void  SoldierClient::SetAgr(float value){
	if (value >=0.0 && value <=1.0)
		agr = value; 
}

int		SoldierClient::GetBodyPosition(){
	return bodyPosition;
}
void  SoldierClient::SetBodyPosition(int value){
	if (value == 0 || value == 1 || value == 2)
		bodyPosition = value; 
}