
// $Id: SoldierClient.hpp,v 1.10 2001/04/08 20:51:44 mitja Exp $

/* Soldier class on the client
 *
 * Extended Soldier class that now allows drawing of the soldiers at theis coordinates.
 * Created by: Matija Jekovec
 * Created on: 22.11.2000
 */

#ifndef __SOLDIER_CLIENT__
#define __SOLDIER_CLIENT__

class SoldierClient;

#include "enums.hpp"
#include "Viewport.hpp"
#include "PicCollection.hpp"


class SoldierClient
{
private:
	/* array of integers, that represent the images of the soldier's state in
	 * the PicCollection object */
	int   *pics; 
	PicCollection *maps; // pointer to the PicCollection object containing the bitmaps of soldiers

	int   soldierId; // id of the soldier;
	float radius;    // radius of the soldier 
	float x, y;      // soldier position 
	float moveAngle; // angle of movement [rad]
	float viewAngle; // angle of view (and shooting) [rad]
	float FOV;       // field of view[rad]
	float health;
	float agr;
	int	  aiID;
	bool  friendly;
	int   bodyPosition;

	float   lastX; // previous properties used for interpolation
	float   lastY;
	float   lastMoveAngle;	
	float   lastViewAngle;
	float		lastFOV;
	float   lastAgr;
	MyColor colorName;

	void DrawSelectedBox(Viewport* viewport, SDL_Surface *surface);

public:
	bool selected;
	bool updateFlag;
	int groupIndex;

	// constructor, assigns values to maps and pics pointers
	SoldierClient(PicCollection *aMaps, int *aPics[], int aSoldierId, MyColor color);

	/* draws the soldier at his coordinates to the surface, using and 
	 * converting the coordinates in the viewport, taking account of the zoom. */
	void DrawSoldier(Viewport* viewport, SDL_Surface *surface);

	// get methods for the protected members, set methods also set the last... members
	float GetX(); 
	void  SetX(float value);
	float GetY();
	void  SetY(float value);
	float GetMoveAngle();
	void  SetMoveAngle(float value);
	float GetViewAngle();
	void  SetViewAngle(float value);
	float GetFOV();
	void  SetFOV(float value);
	float GetRadius();
	void  SetRadius(float value);
	int   GetSoldierId();
	float GetHealth();
	void  SetHealth(float value);
	float GetAgr();
	int		GetBodyPosition();
	void  SetBodyPosition(int value);
	void  SetAgr(float value);

	bool  isFriendly();
};

#endif // __SOLDIER_CLIENT__
