// $Id: State.hpp,v 1.1 2001/04/08 15:40:52 mitja Exp $

/* State.hpp, 1.1, 2001-04-06, Mitja Lustrek
 *
 * This class represent a state of a finite state automaton that controls a group of soldiers. It can't
 * be instantiated - derived classes are to be used instead. It should contain methods that more than
 * one derived class uses. Virtual methods of derived class may do whatever they will, except for
 * update which must return tne index of the state automaton should switch to (or the index of the
 * current one of it should remain in it).
 */

#ifndef __STATE__
#define __STATE__

#include "Environment.hpp"
#include "Automaton.hpp"

class Group;

class State
{
protected:
	Group *grp;       // group that automaton that this is state of belongs to
	Environment *env; // environment that automaton operates in
	Automaton *aut;   // automaton that this is state of

public:
	State (Group *grp, Environment *env, Automaton *aut); // construct state for given group and automaton
	virtual void enter ()=0;                              // called upon entering this state
	virtual void leave ()=0;                              // called upon leaving this state
	virtual int update ()=0;                              // switch states and is to be called every turn
};

#endif //__STATE__