// $Id: Group.hpp,v 1.3 2001/03/05 08:28:53 damir Exp $

/* StateMDM.cpp, 1.6, 2001-05-15, Mitja Lustrek
 *
 * See header file.
 */

#include <stdlib.h>
#include <math.h>
#include "stdafx.h"
#include "StateMDM.hpp"
#include "State.hpp"
#include "Group.hpp"
#include "Environment.hpp"
#include "Automaton.hpp"
#include "AutomatonMDM.hpp"
#include "List.hpp"
#include "Soldier.hpp"
#include "Auxiliary.hpp"
#include "MapAttributes.hpp"
#include "SquareAttrMDM.hpp"
#include "AreaOfDanger.hpp"

#define IMPASSABLE 50

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

////////////////////////////////////////////////////////////////////////////////////////////////////////

struct AngleAndSoldier
{
	float angle;
	Soldier *soldier;
};

int compareAAS (const void *first, const void *second)
{
	if (((AngleAndSoldier *)first)->angle>((AngleAndSoldier *)second)->angle)
		return 1;
	else
		return -1;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

struct AngleAndQuality
{
	float angle, quality;
};

int compareAAQbyQ (const void *first, const void *second)
{
	if (((AngleAndQuality *)first)->quality>((AngleAndQuality *)second)->quality)
		return -1;
	else
		return 1;
}

int compareAAQbyA (const void *first, const void *second)
{
	if (((AngleAndQuality *)first)->angle>((AngleAndQuality *)second)->angle)
		return 1;
	else
		return -1;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

struct QualityAndCoords
{
	float quality;
	int enemyCount;
	Vector coords;
};

int compareQAC (const void *first, const void *second)
{
	if (((QualityAndCoords *)first)->quality>((QualityAndCoords *)second)->quality)
		return -1;
	else
		return 1;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

StateMDM::StateMDM (Group *grp, Environment *env, Automaton *aut): State (grp, env, aut)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateMDM::updEnemy ()
{
  List *soldierList, *addedList;
	Soldier *temp;

	// Determine visible enemies
	enemyList.empty ();
	soldierList=grp->getSoldiers ();
	temp=(Soldier *)(soldierList->getFirst ());
	while (temp!=NULL)
	{
		addedList=env->getVisibleEnemies (temp);
		enemyList.merge (addedList);
		delete addedList;
		temp=(Soldier *)(soldierList->getNext ());
	}

	// Determine enemies' strength
	strengthEnemy=0;
	temp=(Soldier *)(enemyList.getFirst ());
	while (temp!=NULL)
	{
		strengthEnemy+=temp->getHealth ()+(int)(aut->getWeight (AutomatonMDM::WEIGHT_STRENGTH_PRESENCE));
		temp=(Soldier *)(enemyList.getNext ());
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateMDM::updUs ()
{
	List *soldierList;
	Soldier *temp;

	strengthUs=0;
	soldierList=grp->getSoldiers ();
	temp=(Soldier *)(soldierList->getFirst ());
	while (temp!=NULL)
	{
		strengthUs+=temp->getHealth ()+(int)(aut->getWeight (AutomatonMDM::WEIGHT_STRENGTH_PRESENCE));
		temp=(Soldier *)(soldierList->getNext ());
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateMDM::updEnemyCentre ()
{
	Soldier *temp;
	int n=0;

	enemyCentre.x=0;
	enemyCentre.y=0;
	temp=(Soldier *)(enemyList.getFirst ());
	while (temp!=NULL)
	{
		enemyCentre=enemyCentre+temp->getCoords ();
		n++;
		temp=(Soldier *)(enemyList.getNext ());
	}
	enemyCentre=enemyCentre*(1.0f/(float)n);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateMDM::updAggressiveness ()
{
	List *soldierList;
	Soldier *temp;

	aggressiveness=0;
	soldierList=grp->getSoldiers ();
	temp=(Soldier *)(soldierList->getFirst ());
	while (temp!=NULL)
	{
		aggressiveness+=temp->getAgression ();
		temp=(Soldier *)(soldierList->getNext ());
	}
	aggressiveness=(int)(((float)(aggressiveness))/((float)(soldierList->getNumItems ())));
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

bool StateMDM::attEnemySeen ()
{
	updEnemy ();
	if (strengthEnemy>0)
	{
		updUs ();
		updAggressiveness ();
		if ((aggressiveness>2) && (strengthEnemy<strengthUs-aut->getWeight (AutomatonMDM::WEIGHT_STRENGTH_TOLERANCE_DOWN)))
			attackType=AutomatonMDM::STATE_ATTACKING_ADVANCING;
		else
			if ((aggressiveness<7) && (strengthEnemy>strengthUs+aut->getWeight (AutomatonMDM::WEIGHT_STRENGTH_TOLERANCE_UP)))
				attackType=AutomatonMDM::STATE_RETREATING;
			else
				attackType=AutomatonMDM::STATE_ATTACKING;
		return true;
	}
	else
		return false;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

bool StateMDM::attHaveOrders ()
{
	int i, orderedCount=0;

	for (i=0; i<grp->getSoldiers ()->getNumItems (); i++)
	{
		if (grp->m_aSoldierItems [i].m_MotionState!=MOTION_NO_WAYPOINT)
			orderedCount++;
		if (orderedCount>aut->getWeight (AutomatonMDM::WEIGHT_MAX_ORDERED))
			return true;
	}
	return false;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateMDM::actCoverArea (bool moveAlso)
{
	List *soldierList;
	Soldier *temp, *leader;
	AngleAndSoldier soldierAngles [20];
	int soldierCount=0, i, j, minIndex, leaderIndex, range, lowerLimit, upperLimit, visValueCount, interval;
	Vector currentWayPoint, centre, tempV;
	float angle, minDifference, difference, squareBorder, minDistance, lowerAngle, upperAngle, dangerFactor;
	SquareAttrMDM *tempSA;
	bool lowerLimitFound;
	AngleAndQuality visValues [16];
	List *areaOfDanger;
	AreaOfDanger *tempAOD;

	soldierList=grp->getSoldiers ();
	if (soldierList->getNumItems ()>0)
	{
		// Enter soldiers' angles from the centre of the group and pointers to soldiers into array
		centre=grp->GetCenter ();
		temp=(Soldier *)(soldierList->getFirst ());
		while (temp!=NULL)
		{
			soldierAngles [soldierCount].angle=angleOfVector (centre, temp->getCoords ());
			soldierAngles [soldierCount].soldier=temp;
			soldierCount++;
			temp=(Soldier *)(soldierList->getNext ());
		}

		minDistance=aut->getWeight (AutomatonMDM::WEIGHT_MIN_DISTANCE);
		squareBorder=grp->m_pEnv->getSquareBorder ();
		if (attHaveOrders ())
		{
			// Sort array entries and determine soldier whose angle from the centre of the group best fits group's direction of movement (leader)
			qsort (soldierAngles, soldierCount, sizeof (AngleAndSoldier), compareAAS);
			currentWayPoint=grp->getCurrentWayPoint ();
			if (currentWayPoint.x<0)
				angle=soldierAngles [0].soldier->getMoveAngle ();
			else
				angle=angleOfVector (centre, currentWayPoint);
			minDifference=10.0f;
			for (i=0; i<soldierCount; i++)
			{
				difference=(float)fabs (angleDifference (soldierAngles [i].angle, angle));
				if (difference<minDifference)
				{
					minDifference=difference;
					minIndex=i;
				}
			}
			leader=soldierAngles [minIndex].soldier;

			// Set leader's angle of view
			tempSA=(SquareAttrMDM *)(grp->m_pEnv->plData [leader->getAiID ()].attr->getSquare ((int)(leader->getCoords ().x/squareBorder), (int)(leader->getCoords ().y/squareBorder)));
			leaderIndex=(int)(leader->getMoveAngle ()/(0.125f*PI));
			range=(int)(aut->getWeight (AutomatonMDM::WEIGHT_LEADER_RANGE));
			lowerLimit=(leaderIndex-range)%16;
			lowerLimitFound=false;
			for (i=lowerLimit; i<(leaderIndex+range)%16; i=(i+1)%16)
				if (tempSA->visvalue [i]<minDistance)
				{
					if (!lowerLimitFound) {
						lowerLimit++;
						upperLimit=lowerLimit;
					}
				}
				else
				{
					lowerLimitFound=true;
					upperLimit=i;
				}
			if (!lowerLimitFound)
			{
				lowerLimit=leaderIndex;
				upperLimit=leaderIndex;
			}
			lowerAngle=lowerLimit*0.125f*PI;
			upperAngle=upperLimit*0.125f*PI;
			if (fabs (angleDifference (leader->getViewAngle (), lowerAngle))<0.01f)
				leader->setViewAngle (upperAngle);
			else
				leader->setViewAngle (lowerAngle);
		}
		else
		{
			minIndex=-1;
			leader=NULL;
		}
		
		// Find interesting angles of view for the rest of the group
		tempSA=(SquareAttrMDM *)(grp->m_pEnv->plData [((Soldier *)(soldierList->getFirst ()))->getAiID ()].attr->getSquare ((int)(centre.x/squareBorder), (int)(centre.y/squareBorder)));
		range=(int)(((Soldier *)(soldierList->getFirst ()))->getFieldOfView ()/(0.125f*PI)+0.5f);
		visValueCount=0;
		i=0;
		while (i<16)
			if (tempSA->visvalue [i]>minDistance)
			{
				visValues [visValueCount].quality=0;
				for (interval=0; interval<range+1; interval++)
					if ((tempSA->visvalue [i+interval]>minDistance) && (interval<range))
						visValues [visValueCount].quality+=tempSA->visvalue [i];
					else
						break;
				visValues [visValueCount].quality/=interval;
				visValues [visValueCount].angle=(0.125f*PI*i+0.125f*PI*(i+interval-1))/2;
				visValueCount++;
				i+=interval;
			}
			else
				i++;
		areaOfDanger=&grp->m_pEnv->plData [((Soldier *)(soldierList->getFirst ()))->getAiID ()].areaod;
		for (i=0; i<visValueCount; i++)
		{
			dangerFactor=0;
			tempAOD=(AreaOfDanger *)(areaOfDanger->getFirst ());
			while (tempAOD!=NULL)
			{
				tempV.x=tempAOD->x;
				tempV.y=tempAOD->y;
				dangerFactor+=(float)((PI-fabs (angleDifference (visValues [i].angle, angleOfVector (centre, tempV))))*tempAOD->r);
				tempAOD=(AreaOfDanger *)(areaOfDanger->getNext ());
			}
			visValues [i].quality*=(1.0f+dangerFactor);
		}
		if (visValueCount==0)
		{
			visValues [0].angle=angleOfVector (centre, currentWayPoint);
			visValueCount=1;
		}

		// Assign found angles of view to soldiers (except leader)
		if (visValueCount>=soldierCount)
		{
			qsort (visValues, visValueCount, sizeof (AngleAndQuality), compareAAQbyQ);
			qsort (visValues, soldierCount-1, sizeof (AngleAndQuality), compareAAQbyA);
		}
		else
			qsort (visValues, visValueCount, sizeof (AngleAndQuality), compareAAQbyA);
		i=0;
		j=0;
		for (i=0; i<soldierCount; i++)
			if (i!=minIndex)
			{
				soldierAngles [i].soldier->setViewAngle (visValues [j].angle);
				if (moveAlso)
					soldierAngles [i].soldier->setMoveAngle (visValues [j].angle);
				j=(j+1)%visValueCount;
			}
	}	
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateMDM::actShootingFormation ()
{
	List *soldierList;
	Soldier *tempS;
	float minX, maxX, minY, maxY, expansion, squareBorder;
	Vector centre, tempC;
	QualityAndCoords *square;
	int startX, startY, rangeX, rangeY, i, j, k, squareCount=0, addition;
	bool relocate, relocated;
	
	updEnemy ();
	updEnemyCentre ();

	// Get square to look within
	centre=grp->GetCenter ();
	minX=centre.x;
	maxX=centre.x;
	minY=centre.y;
	maxY=centre.y;
	soldierList=grp->getSoldiers ();
	tempS=(Soldier *)(soldierList->getFirst ());
	while (tempS!=NULL)
	{
		tempC=tempS->getCoords ();
		if (tempC.x>maxX)
			maxX=tempC.x;
		if (tempC.x<minX)
			minX=tempC.x;
		if (tempC.y>maxY)
			maxY=tempC.y;
		if (tempC.y<minY)
			minY=tempC.y;
		tempS=(Soldier *)(soldierList->getNext ());
	}
	expansion=aut->getWeight (AutomatonMDM::WEIGHT_SHOOT_SQUARE);
	minX=MAX (0, minX-expansion);
	maxX=MIN (env->getMaxSx (), maxX+expansion);
	minY=MAX (0, minY-expansion);
	maxY=MIN (env->getMaxSy (), maxY+expansion);

	// Find feasible coordinates
	squareBorder=env->getSquareBorder ();
	startX=(int)(minX/squareBorder);
	startY=(int)(minY/squareBorder);
	rangeX=((int)(maxX/squareBorder-minX/squareBorder))+1;
	rangeY=((int)(maxY/squareBorder-minY/squareBorder))+1;
	square=new QualityAndCoords [rangeX*rangeY];
	for (i=startX; i<startX+rangeX; i++)
		for (j=startY; j<startY+rangeY; j++)
			if (env->getSquare (i, j)->getPassability ()>IMPASSABLE)
			{
				square [squareCount].coords.x=(float)i+0.5f*squareBorder;
				square [squareCount].coords.y=(float)j+0.5f*squareBorder;
				square [squareCount].enemyCount=0;
				tempS=(Soldier *)(enemyList.getFirst ());
				while (tempS!=NULL)
				{
					if (env->getVisible (NULL, tempS, square [squareCount].coords, angleOfVector (square [squareCount].coords, tempS->getCoords ())))
						square [squareCount].enemyCount++;
					tempS=(Soldier *)(enemyList.getNext ());
				}
				if (square [squareCount].enemyCount>0)
					squareCount++;
			}

	// Assign coordinates to soldiers
	for (i=0; i<soldierList->getNumItems (); i++)
	{
	
		// Determine whether relocation is necessary
		relocate=true;
		tempS=(Soldier *)(enemyList.getFirst ());
		while (tempS!=NULL)
		{
			if (env->getVisible (grp->m_aSoldierItems [i].m_pSoldier, tempS, grp->m_aSoldierItems [i].m_pSoldier->getCoords (), angleOfVector (grp->m_aSoldierItems [i].m_pSoldier->getCoords (), tempS->getCoords ())))
			{
				relocate=false;
				break;
			}
			tempS=(Soldier *)(enemyList.getNext ());
		}
		if (!relocate)
			for (k=0; k<i; k++)
			{
				if (grp->m_aSoldierItems [k].m_MotionState==MOTION_MOVING)
					tempC=grp->m_aSoldierItems [k].m_aWaypoints [grp->m_aSoldierItems [k].m_iWaypointCount-1];
				else
					tempC=grp->m_aSoldierItems [k].m_pSoldier->getCoords ();
				if ((circleHitByLineSegment (tempC, aut->getWeight (AutomatonMDM::WEIGHT_SOLDIER_RADIUS), grp->m_aSoldierItems [i].m_pSoldier->getCoords (), enemyCentre)>0) || (circleHitByLineSegment (grp->m_aSoldierItems [i].m_pSoldier->getCoords (), aut->getWeight (AutomatonMDM::WEIGHT_SOLDIER_RADIUS), tempC, enemyCentre)>=0.01f))
				{
					relocate=true;
					break;
				}
			}

		// Relocate if proven necessary
		if (relocate)
		{
			addition=(int)(aut->getWeight (AutomatonMDM::WEIGHT_QUALITY_ADDITION));
			for (j=0; j<squareCount; j++)
				square [j].quality=((float)(square [j].enemyCount+addition))/(square [j].coords-grp->m_aSoldierItems [i].m_pSoldier->getCoords ()).length ();
			qsort (square, squareCount, sizeof (QualityAndCoords), compareQAC);
			relocated=false;
			for (j=0; j<squareCount; j++) {
				if (square [j].coords.x>=0.0f)
				{
					relocated=true;
					for (k=0; k<i; k++)
					{
						if (grp->m_aSoldierItems [k].m_MotionState==MOTION_MOVING)
							tempC=grp->m_aSoldierItems [k].m_aWaypoints [grp->m_aSoldierItems [k].m_iWaypointCount-1];
						else
							tempC=grp->m_aSoldierItems [k].m_pSoldier->getCoords ();
						if ((circleHitByLineSegment (tempC, aut->getWeight (AutomatonMDM::WEIGHT_SOLDIER_RADIUS), square [j].coords, enemyCentre)>0) || (circleHitByLineSegment (square [j].coords, aut->getWeight (AutomatonMDM::WEIGHT_SOLDIER_RADIUS), tempC, enemyCentre)>=0.01f))
						{
							relocated=false;
							break;
						}
					}
				}
				if (relocated) {
					grp->m_aSoldierItems [i].AddPath (square [j].coords, grp);
					if (grp->m_aSoldierItems [i].m_pSoldier->getAgression ()<5)
						grp->m_aSoldierItems [i].m_pSoldier->setMoveType (Soldier::MOVEMENT_CROUCH);
					else
						grp->m_aSoldierItems [i].m_pSoldier->setMoveType (Soldier::MOVEMENT_WALK);
					square [j].coords.x=-1.0f;
					break;
				}
			}
		}
	}

	delete square;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateMDM::actShoot (bool moveAlso)
{
	int i;
	List *enemyByOneList;
	Soldier *temp;
	Vector enemyCoords;

	for (i=0; i<grp->getSoldiers ()->getNumItems (); i++)
	{
		temp=grp->m_aSoldierItems [i].m_pSoldier;
		enemyByOneList=env->getVisibleEnemies (temp);
		if (enemyByOneList->getNumItems ()>0)
			enemyCoords=((Soldier *)enemyByOneList->getFirst ())->getCoords ();
		else
			enemyCoords=enemyCentre;
		temp->setViewAnglePoint (enemyCoords);
		if (grp->m_aSoldierItems [i].m_MotionState==MOTION_NO_WAYPOINT) {
			if (moveAlso)
				temp->setMoveAngle (angleOfVector (temp->getCoords (), enemyCoords));
			if (attackType==AutomatonMDM::STATE_ATTACKING)
				temp->setMoveType (Soldier::MOVEMENT_CRAWL);
			else
				if (attackType==AutomatonMDM::STATE_ATTACKING_ADVANCING)
					temp->setMoveType (Soldier::MOVEMENT_CROUCH);
		}
		if (enemyByOneList->getNumItems ()>0)
			if ((enemyCoords-temp->getCoords ()).length ()>aut->getWeight (AutomatonMDM::WEIGHT_AIM_LIMIT))
				temp->shoot (Soldier::SHOOT_AIM);
			else
				if ((enemyCoords-temp->getCoords ()).length ()>aut->getWeight (AutomatonMDM::WEIGHT_SNAP_LIMIT))
					temp->shoot (Soldier::SHOOT_SNAP);
				else
					temp->shoot (Soldier::SHOOT_AUTO);
		delete enemyByOneList;
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateMDM::actAdvance ()
{
	int i;

	updEnemyCentre ();
	for (i=0; i<grp->getSoldiers ()->getNumItems (); i++)
	{
		if ((grp->m_aSoldierItems [i].m_MotionState==MOTION_NO_WAYPOINT) && ((enemyCentre-grp->m_aSoldierItems [i].m_pSoldier->getCoords ()).length ()>aut->getWeight (AutomatonMDM::WEIGHT_STOP_ADVANCE)) && (enemyList.getNumItems ()>0))
			grp->m_aSoldierItems [i].m_pSoldier->setSpeed (aut->getWeight (AutomatonMDM::WEIGHT_ADVANCE_SPEED));
		else
			grp->m_aSoldierItems [i].m_pSoldier->setSpeed (0.0f);
	}
}