// $Id: Group.hpp,v 1.3 2001/03/05 08:28:53 damir Exp $

/* StateMDM.hpp, 1.6, 2001-05-15, Mitja Lustrek
 */

#ifndef __STATEMDM__
#define __STATEMDM__

#include "State.hpp"
#include "Group.hpp"
#include "Environment.hpp"
#include "Automaton.hpp"
#include "AutomatonMDM.hpp"
#include "List.hpp"

class StateMDM: public State
{
protected:
	int time;                        // value that controls when state is left
	int clock;                       // value that controls operations executed periodically but not every update
	List enemyList;                  // list of enemies group currently sees
	Vector enemyCentre;              // centre of visible Enemies
	int strengthEnemy;               // strength of list of enemies group currently sees
	int strengthUs;                  // strength of group
	int aggressiveness;              // average aggressiveness of the group
	AutomatonMDM::tState attackType; // type of attack (or retreat) that group is currently carrying out 

public:
	StateMDM (Group *grp, Environment *env, Automaton *aut); // construct state for given group and automaton

	void updEnemy ();          // update list of enemies and determine their strength
	void updUs ();             // determine strength of group
	void updEnemyCentre ();    // calculate centre of visible enemies
	void updAggressiveness (); // calculate average aggressiveness of the group

	bool attEnemySeen ();  // return true if the group sees an enemy; appropriate type of attack (or retreat) is also chosen
	bool attHaveOrders (); // return true if sufficient number of soldiers have orders to move

	void actCoverArea (bool moveAlso); // group's soldiers cover all area; adjust their angles of movement if desired (usually when standing still)
	void actShootingFormation ();      // group's soldiers arrange thenmselves into shooting formation
	void actShoot (bool moveAlso);     // group's soldiers shoot at enemies; adjust their angles of movement if desired (usually when movement is not required while shooting)
	void actAdvance ();                // group's soldiers advance towards enemy
};

#endif //__STATEMDM__