// $Id: StatesDemo.cpp,v 1.2 2001/05/01 10:10:27 mitja Exp $

/* StatesDemo.cpp, 1.1, 2001-04-05, Mitja Lustrek
 *
 * See header file.
 */

#include "stdafx.h"
#include <stdlib.h>
#include <math.h>
#include "StatesDemo.hpp"
#include "StateDemo.hpp"
#include "Group.hpp"
#include "Environment.hpp"
#include "Automaton.hpp"
#include "AutomatonDemo.hpp"
#include "Soldier.hpp"
#include "List.hpp"
#include "Auxiliary.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////

StateNormal::StateNormal (Group *grp, Environment *env, Automaton *aut): StateDemo (grp, env, aut)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateNormal::enter ()
{
	Soldier *temp;

	if (grp->getSoldiers ()->getNumItems ()>0)
	{
		empty=false;
		movementTime=0;
		temp=(Soldier *)(grp->getSoldiers ()->getFirst ());
		while (temp!=NULL)
		{
			temp->setMaxAngleDifference (2.0f, true);
			temp->setViewAngleRel (0);
			temp=(Soldier *)(grp->getSoldiers ()->getNext ());
		}
	}
	else
		empty=true;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateNormal::leave ()
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

int StateNormal::update ()
{
	Soldier *temp;

	// Perform actions for this state
	if (empty)
		enter ();
	temp=(Soldier *)(grp->getSoldiers ()->getFirst ());
	while (temp!=NULL)
	{
		if (fabs (temp->getSpeed ())>0.1f)
		{
			increaseTime (movementTime);
			break;
		}
		temp=(Soldier *)(grp->getSoldiers ()->getNext ());
	}
	
	// Switch state if appropriate
	if (movementTime>aut->getWeight (AutomatonDemo::WEIGHT_MOVEMENT_DELAY))
		return AutomatonDemo::STATE_TURN;
	else
		return AutomatonDemo::STATE_NORMAL;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////

StateTurn::StateTurn (Group *grp, Environment *env, Automaton *aut): StateDemo (grp, env, aut)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateTurn::enter ()
{
	Soldier *temp;

	temp=(Soldier *)(grp->getSoldiers ()->getFirst ());
	while (temp!=NULL)
	{
		temp->setViewAngleRel (PI);
		temp=(Soldier *)(grp->getSoldiers ()->getNext ());
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateTurn::leave ()
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

int StateTurn::update ()
{
	Soldier *temp;
	bool reversed=true;

	// Perform actions for this state
	temp=(Soldier *)(grp->getSoldiers ()->getFirst ());
	while (temp!=NULL)
	{
		if (temp->getSpeed ()>0.01)
			reversed=false;
		temp=(Soldier *)(grp->getSoldiers ()->getNext ());
	}

	// Switch state if appropriate
	if (reversed)
		return AutomatonDemo::STATE_SHOOT;
	else
		return AutomatonDemo::STATE_TURN;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////

StateShoot::StateShoot (Group *grp, Environment *env, Automaton *aut): StateDemo (grp, env, aut)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateShoot::enter ()
{
	Soldier *temp;

	shootTime=0;
	temp=(Soldier *)(grp->getSoldiers ()->getFirst ());
	while (temp!=NULL)
	{
		temp->shoot (Soldier::SHOOT_SNAP);
		temp=(Soldier *)(grp->getSoldiers ()->getNext ());
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateShoot::leave ()
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

int StateShoot::update ()
{
	// Perform actions for this state
	increaseTime (shootTime);

	// Switch state if appropriate
	if (shootTime>aut->getWeight (AutomatonDemo::WEIGHT_SHOOT_DELAY))
		return AutomatonDemo::STATE_NORMAL;
	else
		return AutomatonDemo::STATE_SHOOT;
}
