// $Id: Group.hpp,v 1.3 2001/03/05 08:28:53 damir Exp $

/* StatesMDM.cpp, 1.5, 2001-05-15, Mitja Lustrek
 *
 * See header file.
 */

#include "stdafx.h"
#include "StatesMDM.hpp"
#include "StateMDM.hpp"
#include "Group.hpp"
#include "Environment.hpp"
#include "Automaton.hpp"
#include "AutomatonMDM.hpp"
#include "Soldier.hpp"
#include "List.hpp"
#include "Auxiliary.hpp"
#include "SquareAttrMDM.hpp"
#include "MapAttributes.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////

StateInit::StateInit (Group *grp, Environment *env, Automaton *aut): StateMDM (grp, env, aut)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateInit::enter ()
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateInit::leave ()
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

int StateInit::update ()
{
	// Switch state if appropriate
	if (grp->getSoldiers ()->getNumItems ()>0)
		return AutomatonMDM::STATE_IDLE;
	return AutomatonMDM::STATE_INIT;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////

StateIdle::StateIdle (Group *grp, Environment *env, Automaton *aut): StateMDM (grp, env, aut)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateIdle::enter ()
{
	List *soldierList;
	Soldier *temp;

	clock=-1;
	time=0;
	soldierList=grp->getSoldiers ();
	temp=(Soldier *)(soldierList->getFirst ());
	while (temp!=NULL)
	{
		temp->setMaxAngleDifference (3.0f, true);
		temp->setSpeed (0.0f);
		temp=(Soldier *)(soldierList->getNext ());
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateIdle::leave ()
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

int StateIdle::update ()
{
	// Perform actions for this state
	clock++;
	time++;
	if (clock==0)
		actCoverArea (false);
	else
		if (clock==(int)(aut->getWeight (AutomatonMDM::WEIGHT_CLOCK_RATE)))
			clock=-1;

	// Switch state if appropriate
	if (attEnemySeen ())
		return attackType;
	if (attHaveOrders ())
		return AutomatonMDM::STATE_MARCHING;
	if (time>aut->getWeight (AutomatonMDM::WEIGHT_TIME_TO_LEAVE))
		return AutomatonMDM::STATE_TO_SAFETY;
	return AutomatonMDM::STATE_IDLE;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////

StateToSafety::StateToSafety (Group *grp, Environment *env, Automaton *aut): StateMDM (grp, env, aut)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateToSafety::enter ()
{
	List *soldierList;
	Soldier *tempS;
	Vector centre, tempC, *bestCoords;
	float minX, maxX, minY, maxY, expansion, squareBorder, quality, bestQuality, distanceInfluence;
	int i, j, startX, startY, rangeX, rangeY;
	SquareAttrMDM *tempSA;
	List *safetyList;

	clock=-1;
	started=false;
	soldierList=grp->getSoldiers ();
	tempS=(Soldier *)(soldierList->getFirst ());
	if (tempS==NULL)
		return;
	while (tempS!=NULL)
	{
		tempS->setMaxAngleDifference (Soldier::DIFFERENCE_SHOOT_IMPEDED, true);
		tempS->setMoveType (Soldier::MOVEMENT_CROUCH);
		tempS=(Soldier *)(soldierList->getNext ());
	}

	centre=grp->GetCenter ();
	minX=centre.x;
	maxX=centre.x;
	minY=centre.y;
	maxY=centre.y;
	tempS=(Soldier *)(soldierList->getFirst ());
	while (tempS!=NULL)
	{
		tempC=tempS->getCoords ();
		if (tempC.x>maxX)
			maxX=tempC.x;
		if (tempC.x<minX)
			minX=tempC.x;
		if (tempC.y>maxY)
			maxY=tempC.y;
		if (tempC.y<minY)
			minY=tempC.y;
		tempS=(Soldier *)(soldierList->getNext ());
	}
	expansion=aut->getWeight (AutomatonMDM::WEIGHT_SAFETY_SQUARE);
	minX=MAX (0, minX-expansion);
	maxX=MIN (env->getMaxSx (), maxX+expansion);
	minY=MAX (0, minY-expansion);
	maxY=MIN (env->getMaxSy (), maxY+expansion);
	squareBorder=env->getSquareBorder ();
	startX=(int)(minX/squareBorder);
	startY=(int)(minY/squareBorder);
	rangeX=((int)(maxX/squareBorder-minX/squareBorder))+1;
	rangeY=((int)(maxY/squareBorder-minY/squareBorder))+1;
	distanceInfluence=aut->getWeight (AutomatonMDM::WEIGHT_DISTANCE_INFLUENCE);
	bestQuality=0;
	bestCoords=new Vector (-1, -1);
	for (i=startX; i<startX+rangeX; i++)
		for (j=startY; j<startY+rangeY; j++)
		{
			tempSA=(SquareAttrMDM *)(grp->m_pEnv->plData [((Soldier *)(soldierList->getFirst ()))->getAiID ()].attr->getSquare (i, j));
			quality=tempSA->shelter+distanceInfluence*tempSA->maxvisvalue;
			if (quality>bestQuality)
			{
				bestQuality=quality;
				bestCoords->x=i*squareBorder;
				bestCoords->y=j*squareBorder;
			}
		}
	safety=*bestCoords;
	safetyList=new List;
	safetyList->add (bestCoords);
	grp->addWayPoints (safetyList);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateToSafety::leave ()
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

int StateToSafety::update ()
{
	// Perform actions for this state
	clock++;
	if (clock==0)
		actCoverArea (false);
	else
		if (clock==1)
			started=true;
		else
			if (clock==(int)(aut->getWeight (AutomatonMDM::WEIGHT_CLOCK_RATE)))
				clock=-1;

	// Switch state if appropriate
	if (attEnemySeen ())
		return attackType;
	if ((attHaveOrders ()) && (!(grp->getCurrentWayPoint ()==safety)) && (grp->getCurrentWayPoint ().x>=0))
		return AutomatonMDM::STATE_MARCHING;
	if ((!attHaveOrders ()) && (started))
		return AutomatonMDM::STATE_DUG_IN;
	return AutomatonMDM::STATE_TO_SAFETY;
}


////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////

StateDugIn::StateDugIn (Group *grp, Environment *env, Automaton *aut): StateMDM (grp, env, aut)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateDugIn::enter ()
{
	List *soldierList;
	Soldier *temp;

	clock=-1;
	soldierList=grp->getSoldiers ();
	temp=(Soldier *)(soldierList->getFirst ());
	while (temp!=NULL)
	{
		temp->setMaxAngleDifference (Soldier::DIFFERENCE_SHOOT_UNIMPEDED, true);
		temp->setMoveType (Soldier::MOVEMENT_CRAWL);
		temp=(Soldier *)(soldierList->getNext ());
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateDugIn::leave ()
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

int StateDugIn::update ()
{
	// Perform actions for this state
	clock++;
	if (clock==0)
		actCoverArea (true);
	else
		if (clock==(int)(aut->getWeight (AutomatonMDM::WEIGHT_CLOCK_RATE)))
			clock=-1;

		// Switch state if appropriate
	if (attEnemySeen ())
		return attackType;
	if (attHaveOrders ())
		return AutomatonMDM::STATE_MARCHING;
	return AutomatonMDM::STATE_DUG_IN;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////

StateMarching::StateMarching (Group *grp, Environment *env, Automaton *aut): StateMDM (grp, env, aut)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateMarching::enter ()
{
	List *soldierList;
	Soldier *temp;

	clock=-1;
	soldierList=grp->getSoldiers ();
	temp=(Soldier *)(soldierList->getFirst ());
	while (temp!=NULL)
	{
		temp->setMaxAngleDifference (3.0f, true);
		temp->setMoveType (Soldier::MOVEMENT_WALK);
		temp=(Soldier *)(soldierList->getNext ());
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateMarching::leave ()
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

int StateMarching::update ()
{
	// Perform actions for this state
	clock++;
	if (clock==0)
		actCoverArea (false);
	else
		if (clock==(int)(aut->getWeight (AutomatonMDM::WEIGHT_CLOCK_RATE)))
			clock=-1;

	// Switch state if appropriate
	if (attEnemySeen ())
		return attackType;
	if (!attHaveOrders ())
		return AutomatonMDM::STATE_IDLE;
	return AutomatonMDM::STATE_MARCHING;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////

StateAttackingAdvancing::StateAttackingAdvancing (Group *grp, Environment *env, Automaton *aut): StateMDM (grp, env, aut)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateAttackingAdvancing::enter ()
{
	List *soldierList;
	Soldier *temp;

	clock=-1;
	updEnemy ();
	updEnemyCentre ();
	grp->removeWayPoints ();
	soldierList=grp->getSoldiers ();
	temp=(Soldier *)(soldierList->getFirst ());
	while (temp!=NULL)
	{
		temp->setMaxAngleDifference (Soldier::DIFFERENCE_SHOOT_UNIMPEDED, false);
		temp->setMoveType (Soldier::MOVEMENT_CROUCH);
		temp=(Soldier *)(soldierList->getNext ());
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateAttackingAdvancing::leave ()
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

int StateAttackingAdvancing::update ()
{
	int halfClockRate=(int)(aut->getWeight (AutomatonMDM::WEIGHT_CLOCK_RATE)/2.0f);

	// Perform actions for this state and switch state if appropriate
	clock++;
	actShoot (true);
	if (clock==0)
		actShootingFormation ();
	else
		if ((clock==1) || (clock==halfClockRate+1))
			actAdvance ();
		else
			if ((clock==2) || (clock==halfClockRate+2))
				if (attEnemySeen ())
					return attackType;
				else
					return AutomatonMDM::STATE_IDLE;
			else
				if (clock==2*halfClockRate)
					clock=-1;

	// Stay in current state
	return AutomatonMDM::STATE_ATTACKING_ADVANCING;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////

StateAttacking::StateAttacking (Group *grp, Environment *env, Automaton *aut): StateMDM (grp, env, aut)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateAttacking::enter ()
{
	List *soldierList;
	Soldier *temp;

	clock=-1;
	updEnemy ();
	updEnemyCentre ();
	grp->removeWayPoints ();
	soldierList=grp->getSoldiers ();
	temp=(Soldier *)(soldierList->getFirst ());
	while (temp!=NULL)
	{
		temp->setMaxAngleDifference (Soldier::DIFFERENCE_SHOOT_UNIMPEDED, false);
		temp=(Soldier *)(soldierList->getNext ());
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateAttacking::leave ()
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

int StateAttacking::update ()
{
	int halfClockRate=(int)(aut->getWeight (AutomatonMDM::WEIGHT_CLOCK_RATE)/2.0f);

	// Perform actions for this state and switch state if appropriate
	clock++;
	actShoot (true);
	if (clock==0)
		actShootingFormation ();
	else
		if ((clock==1) || (clock==halfClockRate+1))
			if (attEnemySeen ())
				return attackType;
			else
				return AutomatonMDM::STATE_IDLE;
		else
			if (clock==2*halfClockRate)
				clock=-1;

	// Stay in current state
	return AutomatonMDM::STATE_ATTACKING;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////

StateRetreating::StateRetreating (Group *grp, Environment *env, Automaton *aut): StateMDM (grp, env, aut)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateRetreating::enter ()
{
	List *soldierList, *safeCoordsList;
	Vector *safeCoords;
	Soldier *temp;

	// Reset timing values and update list of enemies
	clock=-1;
	time=0;
	updEnemy ();

	// Determine where to retreat
	grp->removeWayPoints ();
	safeCoordsList=new List;
	safeCoords=new Vector (grp->startingCoords.x, grp->startingCoords.y);
  safeCoordsList->add (safeCoords);
	grp->addWayPoints (safeCoordsList);

	// Set soldiers' attributes
	soldierList=grp->getSoldiers ();
	temp=(Soldier *)(soldierList->getFirst ());
	while (temp!=NULL)
	{
		temp->setMaxAngleDifference (Soldier::DIFFERENCE_SHOOT_IMPEDED, true);
		temp->setMoveType (Soldier::MOVEMENT_CROUCH);
		temp=(Soldier *)(soldierList->getNext ());
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateRetreating::leave ()
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

int StateRetreating::update ()
{
	// Perform actions for this state and switch state if appropriate
	clock++;
	time++;
	actShoot (false);
	if (clock==0)
		if (attEnemySeen ())
			if (time<=aut->getWeight (AutomatonMDM::WEIGHT_TIME_TO_LEAVE))
				return attackType;
			else
				return AutomatonMDM::STATE_ATTACKING;
		else
			if (time<=aut->getWeight (AutomatonMDM::WEIGHT_TIME_TO_LEAVE))
				return AutomatonMDM::STATE_MARCHING;
			else
				return AutomatonMDM::STATE_IDLE;
	else
		if (clock==(int)(aut->getWeight (AutomatonMDM::WEIGHT_CLOCK_RATE)))
			clock=-1;

	// Stay in current state
	return AutomatonMDM::STATE_RETREATING;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////

StateX::StateX (Group *grp, Environment *env, Automaton *aut): StateMDM (grp, env, aut)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateX::enter ()
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void StateX::leave ()
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

int StateX::update ()
{
	// Perform actions for this state

	// Switch state if appropriate
	return AutomatonMDM::STATE_IDLE;
}
