
// $Id: TextDisplay.cpp,v 1.3 2001/04/08 15:42:51 mitja Exp $

/* TextDisplay class implementation
 *
 * Created by: Damir Arh
 * Created on: 7.2.2001
 */

#include "stdafx.h"
#include "TextDisplay.hpp"
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <stdarg.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/* TextDisplay(HWND hwnd, int size1, int length1)
 *
 * Parameters:
 *    hwnd    : handle to the window that will display the text
 *    size1   : number of line to display at the same time
 *    length1 : length of a line of text
 *
 * This constructor creates an object with the buffer of the
 * defined size.
 */
TextDisplay::TextDisplay(HWND hwnd, int size1, int length1)
{
	int i;

	size = size1;
	length = length1;
	hWnd = hwnd;
	latest = 0;

	buffer = (char **) malloc(size * sizeof(char *));
	for (i=0; i<size; i++)
	{
		buffer[i] = (char *) malloc((length+1) * sizeof(char));
		buffer[i][length] = '\0';
		buffer[i][0] = '\0';
	}
}


/* ~TextDisplay()
 *
 * Destroys the object and releasea all related memory.
 */
TextDisplay::~TextDisplay()
{
	for (int i=0; i<size; i++)
	{
		free(buffer[i]);
	}
	free(buffer);
}


/* inc(int n)
 *
 * Parameters:
 *    n : index to increment
 *
 * Result:
 *    int : incremented index
 *
 * Increase the line index and takes care of the size of the bounded buffer.
 */
int TextDisplay::inc(int n)
{
	n++;
	if (n >= size)
	{
		n = 0;
	}
	return n;
}


/* display(cahr *line)
 *
 * Parameters:
 *    line : string to display in the last line
 *
 * Adds a new string to display and the same time removes the oldest
 * one from display. Orders an immediate redraw of the window.
 */
void TextDisplay::display(char *line,...)
{
	va_list args;

	if (line == NULL)
		return;

	latest = inc(latest);

	va_start(args, line);
		vsprintf(buffer[latest], line, args);
	va_end(args);

	RedrawWindow(hWnd, NULL, NULL, RDW_INVALIDATE|RDW_ERASE);
	UpdateWindow(hWnd);
}


/* getBuffer()
 *
 * Result:
 *    char * : buffer to display
 *
 * Returns buffer to display in the window. Should usually only be called by
 * the painting routine of the window.
 *
 * You have to free the obtained buffer once you don't need it anymore.
 */
char *TextDisplay::getBuffer()
{
	int i, c;
	char *expbuffer;

	expbuffer = (char *) malloc((size * (length+1) + 1) * sizeof(char));
	expbuffer[0] = '\0';
	for (i=inc(latest), c=0; c<size; i=inc(i), c++)
	{
		strcat(expbuffer, buffer[i]);
		strcat(expbuffer, "\n");
	}
	return expbuffer;
}
