
// $Id: TextDisplay.hpp,v 1.3 2001/04/08 15:40:52 mitja Exp $

/* TextDisplay class
 *
 * This class emulates a console like text display for easy text
 * output into the program window. It collects print out requests
 * and keeps their history in requested length. This makes it
 * possible for the text drawing routine in the window to obtain
 * a buffer to directly print out. A window update is also
 * requested after every submitted text line.
 *
 * Take care not to give longer lines than you specified in the
 * constructor. Do not use newline characters in your lines to
 * assure proper output.
 *
 * Created by: Damir Arh
 * Created on: 7.2.2001
 */

#ifndef __TEXTDISPLAY__
#define __TEXTDISPLAY__

#include <windows.h>


class TextDisplay
{
private:
	char **buffer; // text buffer
	int latest;    // index of the latest changed line
	int size;      // number of lines in the buffer
	int length;    // length of a line in the buffer
	HWND hWnd;     // window handle to request update

	int inc(int n); // increases n as index of the bounded buffer

public:
	TextDisplay(HWND hWnd, int size, int length); // constructs an object with given buffer size
	~TextDisplay();

	char *getBuffer();            // returns complete buffer to display
	void display(char *line,...); // orders a new line of text for displaying
};

#endif // __TEXTDISPLAY__
