
// $Id: TuiGraphics.cpp,v 1.5 2001/04/08 20:59:44 mitja Exp $

#include "stdafx.h"
#include <math.h>
#include "TuiGraphics.hpp"
#include "Auxiliary.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

TUI_rectStruct clipRect;


/*
* pointStruct TUI_Point(float x, float y)
* Desc: Returns the point strucutre for the given coordinates
* In:   x, y are the given coordinates
*/
TUI_pointStruct TUI_Point(float x, float y)
{
	TUI_pointStruct p;
	
	p.x = x;
	p.y = y;
	
	return p;
}


/*
* TUI_rectStruct	TUI_Rect(float x, float y, float w, float h)
* Desc: returns rectangle composed of points
* In:   x, y, w, h are the rectangle coordinates
*/
TUI_rectStruct TUI_Rect(float x, float y, float w, float h)
{
	TUI_rectStruct r;
	
	r.x = x;
	r.y = y;
	r.w = w;
	r.h = h;
	
	return r;
}


/*
* pointStruct rotate(pointStruct p, pointStruct start, int angle)
* Desc: Rotates a point p around start for the given angle in degrees(0 - 360) 
* In:   p is the point to be rotated
*       start is the starting point
*       angle is the angle of rotation in degrees[0, 360]
*/
TUI_pointStruct TUI_Rotate(TUI_pointStruct p, TUI_pointStruct start, float angle)
{
	float r;
	float fi;
	
	p.x = p.x - start.x;
	p.y = p.y - start.y;
	
	r =(float) sqrt(p.x*p.x + p.y*p.y);
	fi =(float) atan2(p.y, p.x); 
	
	fi =(float)(fi + angle);
	
	p.x= -(float)(r * sin(fi)) + start.x;
	p.y =(float)(r * cos(fi)) + start.y;
	
	return p;
}

	
/*	
* void TUI_Circle(SDL_Surface *surface, TUI_pointStruct p, float radius, Uint32 color)
* Desc: Draws a circle using the surface, center point, radius and color
* In:   surface is the destination surface
*       p is the center point 
*       radius is the radius in pixels
*       color is the color of the drawn circle
*       fx is the type of effect used to write a pixel
*/
void TUI_Circle(SDL_Surface *surface, TUI_pointStruct p, float radius, Uint32 color, TUI_SetPixelEffects fx)
{
	float x = 0;
	float y = radius;
	float sw = 3 - 2*radius;
	
	while (x <= y) 
	{
		TUI_SetPixel(TUI_Point(x + p.x, y + p.y) , color, surface, fx);
		TUI_SetPixel(TUI_Point(x+ p.x, - y+ p.y) , color, surface, fx);
		
		TUI_SetPixel(TUI_Point(-x+ p.x, y+ p.y) , color, surface, fx);
		TUI_SetPixel(TUI_Point(-x+ p.x, - y + p.y) , color, surface, fx);
		
		TUI_SetPixel(TUI_Point(y+ p.x, x+ p.y) , color, surface, fx);
		TUI_SetPixel(TUI_Point(y+ p.x, - x+ p.y) , color, surface, fx);
		
		TUI_SetPixel(TUI_Point(-y+ p.x, x+ p.y) , color, surface, fx);
		TUI_SetPixel(TUI_Point(-y+ p.x, - x+ p.y) , color, surface, fx);
		
		if (sw < 0)
			sw = sw + 4 * x + 6;
		else 
		{
			sw = sw + 4 *(x - y) + 10;      
			y = y - 1;
		}
		x = x + 1;
	}
}


/*
* void TUI_SetPixel(TUI_pointStruct p, Uint32 color, SDL_Surface *screen)
* Desc: Writes a pixel on the surface at p with a color. Drawing
*       outside bounds produces an error
* In:   p is the point
*       color is the color of the pixel to be set
*       screen is the destination surface
*       fx is the type of effect used to write a pixel
*/
void TUI_SetPixel(TUI_pointStruct p, Uint32 color, SDL_Surface *screen, TUI_SetPixelEffects fx)
{
		int X = (int) p.x, Y = (int) p.y;
		Uint8		bpp;
		Uint8		*bits;
		Uint32	bits32;

		if (!((p.x > clipRect.x) &&
				(p.x < clipRect.x + clipRect.w) &&
				(p.y > clipRect.y) &&
				(p.y < clipRect.y + clipRect.h)))

				return;

		
		bpp = screen->format->BytesPerPixel;
		bits = ((Uint8 *)screen->pixels) + Y*screen->pitch + X*bpp;
		bits32 = *((Uint32 *)bits);			//use the largest for operations

		switch (fx){		//note: any operations that depend on the bit depth will not be rendered correctly
			case fxInvertSrc:
				bits32 = ~color;
				break;
			case fxInvertDst:
				bits32 = ~bits32;
				break;
			case fxSrcAnd:
				bits32 = bits32 & color;
				break;
			case fxSrcOr:
				bits32 = bits32 | color;
				break;		
			default:
				bits32 = color;
		}		//switch

		switch(bpp) {			//cast back
			case 1:
				*((Uint8 *)(bits)) = (Uint8)bits32;
				break;
			case 2:					
				*((Uint16 *)(bits)) = (Uint16)bits32;
				break;
			case 3: { // Format/endian independent - didn't touch this one
				Uint8 r, g, b;
				r = (color>>screen->format->Rshift)&0xFF;
				g = (color>>screen->format->Gshift)&0xFF;
				b = (color>>screen->format->Bshift)&0xFF;
				*((bits)+screen->format->Rshift/8) = r; 
				*((bits)+screen->format->Gshift/8) = g;
				*((bits)+screen->format->Bshift/8) = b;
				}
				break;
			case 4:
				*((Uint32 *)(bits)) = bits32;
				break;
		}
}


/*
* void TUI_Line(TUI_pointStruct pStart, TUI_pointStruct pEnd, Uint32 color, SDL_Surface *screen)
* Desc: Draws a  line from pStart to pEnd with color on screen
* In:   pStart is the starting point
*       pEnd is the ending point
*       color is the color of the pixel
*       screen is the destination surface
*       fx is the type of effect used to write a pixel
*/
void TUI_Line(TUI_pointStruct pStart, TUI_pointStruct pEnd, Uint32 color, SDL_Surface *screen, TUI_SetPixelEffects fx)
{
	int x0 = (int)pStart.x, y0 = (int)pStart.y;
	int x1 = (int)pEnd.x, y1 = (int)pEnd.y;
	int dy = (int)(y1 - y0);
	int dx = (int)(x1 - x0);
	int stepx, stepy;
	
	if (dy < 0)
	{
		dy = -dy;
		stepy = -1;
	}
	else
	{
		stepy = 1;
	}

	if (dx < 0)
	{
		dx = -dx;
		stepx = -1;
	}
	else
	{
		stepx = 1;
	}
	dy <<= 1;                                                  
	dx <<= 1;                                                  

	TUI_SetPixel(TUI_Point((float)x0, (float)y0), color, screen, fx);
	if (dx > dy)
	{
		int fraction = dy -(dx >> 1);                         
		while (x0 != x1) 
		{
			if (fraction >0)
			{
				y0 += stepy;
				fraction -= dx;                                
			}
			x0 += stepx;
			fraction += dy;                                    
			TUI_SetPixel(TUI_Point((float)x0, (float)y0), color, screen, fx);
		} // while
	}
	else 
	{
		int fraction = dx - (dy >> 1);
		while (y0 != y1) 
		{
			if (fraction > 0)
			{
				x0 += stepx;
				fraction -= dy;
			}
			y0 += stepy;
			fraction += dx;
			TUI_SetPixel(TUI_Point((float)x0, (float)y0), color, screen, fx);
		} // while
	} // if
}


/*
* void TUI_Rect(SDL_Surface *surface, TUI_pointStruct p1, TUI_pointStruct p2, Uint32 color, TUI_SetPixelEffects fx)
* Desc: Draws a  rect from p1 to p2 with color on screen
* In:   p1 is the starting point
*       p2 is the ending point
*       color is the color of the pixel
*       surface is the destination surface
*       fx is the type of effect used to write a pixel
*/
void TUI_Rect(SDL_Surface *surface, TUI_pointStruct p1, TUI_pointStruct p2, Uint32 color, TUI_SetPixelEffects fx, bool filled)
{
	TUI_pointStruct start, end;
	float tmp = 0;
	
	start.x = MIN(p1.x, p2.x);
	start.y = MIN(p1.y, p2.y);
	end.x = MAX(p1.x, p2.x);
	end.y = MAX(p1.y, p2.y);

	if (!filled) {
		TUI_Line(start, TUI_Point(end.x, start.y), color, surface, fx);
		TUI_Line(start, TUI_Point(start.x, end.y), color, surface, fx);
		TUI_Line(end, TUI_Point(start.x, end.y), color, surface, fx);
		TUI_Line(end, TUI_Point(end.x, start.y), color, surface, fx);
	} else {
		SDL_Rect rect;

		rect.x = (short)MAX(start.x, clipRect.x);
		rect.y = (short)MAX(start.y, clipRect.y);

		tmp = (MIN(end.x, clipRect.x + clipRect.w) - rect.x);
		if (tmp <= 0) 
			rect.w = 0;
		else
			rect.w = (short)tmp;

		tmp = (MIN(end.y, clipRect.y + clipRect.h) - rect.y);
		if (tmp <= 0) 
			rect.h = 0;
		else
			rect.h = (short)tmp;

		SDL_FillRect(surface, &rect, color);

	}
}

/*
* void DrawNumberLine(int position)
* Desc: draws a line for a numeral
* In:   position tells it what line to draw
*				pos is the drawing position - coordinates
*/
void DrawNumberLine(int position, SDL_Surface *surface, TUI_pointStruct pos, Uint32 color, TUI_SetPixelEffects fx){
	const int HEIGHT = 8;
	const int WIDTH =5;

	int i = 0;
	float middle = 0.0;

	switch (position){
		case 0:		//top
			for (i=0; i<WIDTH; i++)
				TUI_SetPixel(TUI_Point(pos.x + i, pos.y) , color, surface, fx);		
			break;
		case 1:		//middle
			middle = (HEIGHT / 2);
			for (i=0; i<WIDTH; i++)
				TUI_SetPixel(TUI_Point(pos.x + i, pos.y + middle) , color, surface, fx);		
			break;
		case 2:		//bottom
			for (i=0; i<WIDTH; i++)
				TUI_SetPixel(TUI_Point(pos.x + i, pos.y + HEIGHT) , color, surface, fx);		
			break;
		case 3:		//left upper
			middle = (HEIGHT / 2);
			for (i=0; i<middle; i++)
				TUI_SetPixel(TUI_Point(pos.x, pos.y + i) , color, surface, fx);		
			break;
		case 4:		//right upper
			middle = (HEIGHT / 2);
			for (i=0; i<middle; i++)
				TUI_SetPixel(TUI_Point(pos.x + WIDTH, pos.y + i) , color, surface, fx);		
			break;
		case 5:		//left lower
			middle = (HEIGHT / 2);
			for (i=0; i<middle; i++)
				TUI_SetPixel(TUI_Point(pos.x, pos.y + i + middle) , color, surface, fx);		
			break;
		case 6:		//right lower
			middle = (HEIGHT / 2);
			for (i=0; i<middle; i++)
				TUI_SetPixel(TUI_Point(pos.x + WIDTH, pos.y + i + middle) , color, surface, fx);		
			break;
	}
}

/*
*void TUI_Numeral(int number, SDL_Surface *surface, TUI_pointStruct position, Uint32 color, TUI_SetPixelEffects fx);
* Desc: writes a numeral on surface 
* In:   number is the selected numeral
*       position is the position
*       color is the color of the pixel
*       surface is the destination surface
*       fx is the type of effect used to write a pixel
*/
void TUI_Numeral(int number, SDL_Surface *surface, TUI_pointStruct position, Uint32 color, TUI_SetPixelEffects fx){

	float x = position.x;
	float y = position.y;
	int i = 0;


	switch (number){
		case 0:
			DrawNumberLine(0, surface, position, color, fx);
			DrawNumberLine(2, surface, position, color, fx);
			DrawNumberLine(3, surface, position, color, fx);
			DrawNumberLine(4, surface, position, color, fx);
			DrawNumberLine(5, surface, position, color, fx);
			DrawNumberLine(6, surface, position, color, fx);
			break;
		case 1:
			DrawNumberLine(4, surface, position, color, fx);
			DrawNumberLine(6, surface, position, color, fx);
			break;
		case 2:
			DrawNumberLine(0, surface, position, color, fx);
			DrawNumberLine(1, surface, position, color, fx);
			DrawNumberLine(2, surface, position, color, fx);
			DrawNumberLine(4, surface, position, color, fx);
			DrawNumberLine(5, surface, position, color, fx);
			break;
		case 3:
			DrawNumberLine(0, surface, position, color, fx);
			DrawNumberLine(1, surface, position, color, fx);
			DrawNumberLine(2, surface, position, color, fx);
			DrawNumberLine(4, surface, position, color, fx);
			DrawNumberLine(6, surface, position, color, fx);
			break;
		case 4:
			DrawNumberLine(1, surface, position, color, fx);
			DrawNumberLine(3, surface, position, color, fx);
			DrawNumberLine(4, surface, position, color, fx);
			DrawNumberLine(6, surface, position, color, fx);
			break;		
		case 5:
			DrawNumberLine(0, surface, position, color, fx);
			DrawNumberLine(1, surface, position, color, fx);
			DrawNumberLine(2, surface, position, color, fx);
			DrawNumberLine(3, surface, position, color, fx);
			DrawNumberLine(6, surface, position, color, fx);
			break;		
		case 6:
			DrawNumberLine(0, surface, position, color, fx);
			DrawNumberLine(1, surface, position, color, fx);
			DrawNumberLine(2, surface, position, color, fx);
			DrawNumberLine(3, surface, position, color, fx);
			DrawNumberLine(5, surface, position, color, fx);
			DrawNumberLine(6, surface, position, color, fx);
			break;
		case 7:
			DrawNumberLine(0, surface, position, color, fx);
			DrawNumberLine(4, surface, position, color, fx);
			DrawNumberLine(6, surface, position, color, fx);
			break;
		case 8:
			DrawNumberLine(0, surface, position, color, fx);
			DrawNumberLine(1, surface, position, color, fx);
			DrawNumberLine(2, surface, position, color, fx);
			DrawNumberLine(3, surface, position, color, fx);
			DrawNumberLine(4, surface, position, color, fx);
			DrawNumberLine(5, surface, position, color, fx);
			DrawNumberLine(6, surface, position, color, fx);
			break;
		case 9:
			DrawNumberLine(0, surface, position, color, fx);
			DrawNumberLine(1, surface, position, color, fx);
			DrawNumberLine(2, surface, position, color, fx);
			DrawNumberLine(3, surface, position, color, fx);
			DrawNumberLine(4, surface, position, color, fx);
			DrawNumberLine(6, surface, position, color, fx);
			break;
	
	}	//switch
}

/*
* void TUI_BodyPosition(int position, SDL_Surface *surface, TUI_pointStruct position, Uint32 color, TUI_SetPixelEffects fx){
* Desc: writes a numeral on surface 
* In:   bodyPos is the selected numeral
*       position is the position
*       color is the color of the pixel
*       surface is the destination surface
*       fx is the type of effect used to write a pixel
*/
void TUI_BodyPosition(int bodyPos, SDL_Surface *surface, TUI_pointStruct position, Uint32 color, TUI_SetPixelEffects fx){

	float x = position.x;
	float y = position.y;
	int i = 0;


	switch (bodyPos){
		case 0:		//standing
			DrawNumberLine(4, surface, position, color, fx);
			DrawNumberLine(6, surface, position, color, fx);			
			break;
		case 1:		//crouching
			DrawNumberLine(1, surface, position, color, fx);
			DrawNumberLine(4, surface, position, color, fx);			
			DrawNumberLine(5, surface, position, color, fx);	
			DrawNumberLine(6, surface, position, color, fx);
			break;
		case 2:		//lying
			DrawNumberLine(2, surface, position, color, fx);			
			break;

	}	//switch
}