// VectorDir.cpp : implementation file
//

#include "stdafx.h"
//#include "uisp server.h"
#include "VectorDir.h"
#include "Vector.hpp"
#include <math.h>


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern HWND VectWnd;

DrawStruct draw;
/////////////////////////////////////////////////////////////////////////////
// CVectorDir dialog


SoldStruct::SoldStruct()
{
	m_pWay = NULL;
	memset (&m_Vectors, 0, sizeof(info));
	m_iWaypointCount = 0;
}

SoldStruct::~SoldStruct()
{
	if (m_pWay) delete m_pWay;
	m_pWay = NULL;
	m_iWaypointCount = 0;
}


DrawStruct::DrawStruct()
{
	m_iCount = 0;
	m_pSoldiers = NULL;
}

DrawStruct::~DrawStruct()
{
	DeleteAll();
}

void DrawStruct::SetSoldCount(int count)
{
	DeleteAll();

	m_iCount = count;
//	m_pSoldiers = new SoldStruct[count];
	m_pSoldiers = new SoldStruct[20];
}

void DrawStruct::DeleteAll()
{	
/*	for (int i=0; i <m_iCount; i++)
	{
		m_pSoldiers[i].~SoldStruct();
	}*/
	if (m_pSoldiers) delete [] m_pSoldiers;
	m_pSoldiers = NULL;
	m_iCount = 0;

}

void DrawStruct::SetSoldierVectors(int index, Vector pos, Vector move, Vector v0, Vector v1, Vector v2, Vector coll, Vector vel)
{
	if (index >= m_iCount) return;

	m_pSoldiers[index].m_Vectors.pos = pos;
	m_pSoldiers[index].m_Vectors.move = move;
	m_pSoldiers[index].m_Vectors.v0 = v0;
	m_pSoldiers[index].m_Vectors.v1 = v1;
	m_pSoldiers[index].m_Vectors.v2 = v2;
	m_pSoldiers[index].m_Vectors.coll= coll;
	m_pSoldiers[index].m_Vectors.total = vel;
}

void DrawStruct::SetSoldierWaypoints (int index, Vector pos, Vector *way, int count)
{
	if (m_pSoldiers[index].m_pWay) delete m_pSoldiers[index].m_pWay;
	m_pSoldiers[index].m_pWay = new Vector [count+1];
	
	m_pSoldiers[index].m_pWay[0] = pos;
	for (int i=0; i < count; i++)	m_pSoldiers[index].m_pWay[i+1] = way[i];
	m_pSoldiers[index].m_iWaypointCount = count+1;
}

void DrawStruct::ClearSoldierWaypoints(int index)
{
	if (m_pSoldiers) delete m_pSoldiers[index].m_pWay;
	m_pSoldiers[index].m_pWay = NULL;
}

void DrawStruct::ClearSoldierVector (int index)
{
	m_pSoldiers[index].m_Vectors.coll.x = 0;
	m_pSoldiers[index].m_Vectors.coll.y = 0;
	m_pSoldiers[index].m_Vectors.move.x = 0;
	m_pSoldiers[index].m_Vectors.move.y = 0;
	m_pSoldiers[index].m_Vectors.pos.x  = 0;
	m_pSoldiers[index].m_Vectors.pos.y  = 0;
	m_pSoldiers[index].m_Vectors.total.x= 0;
	m_pSoldiers[index].m_Vectors.total.y= 0;
	m_pSoldiers[index].m_Vectors.v0.x = 0;
	m_pSoldiers[index].m_Vectors.v0.y = 0;
	m_pSoldiers[index].m_Vectors.v1.x = 0;
	m_pSoldiers[index].m_Vectors.v1.y = 0;
	m_pSoldiers[index].m_Vectors.v2.x = 0;
	m_pSoldiers[index].m_Vectors.v2.y = 0;
}


void Refresh()
{
	SendMessage(VectWnd, WM_PAINT, 0,0);
}

void DrawLine(HDC dc, int width, int color, int x1, int y1, int x2, int y2)
{
	int colors[] = { 255, 0, 0, 0, 255, 0, 0, 0, 255, 255, 255, 0, 255, 0, 255, 0, 255, 255 };
	if (color > 5) color = 5;
	
	HPEN pen = CreatePen(PS_SOLID, width, RGB(colors[color*3],colors[color*3 + 1], colors[color*3+2] ));
	HPEN prevPen = (HPEN) SelectObject(dc, pen);
	
	MoveToEx (dc, x1, y1, NULL);
	LineTo(dc, x1+x2, y1+y2);

	SelectObject(dc, prevPen);
	DeleteObject(pen);
}


LRESULT CALLBACK VectorDir(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	PAINTSTRUCT ps;
	HDC dc;

	switch (message) 
	{
		case WM_COMMAND:
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			// Parse the menu selections:
			switch (wmId)
			{
				case IDM_EXIT:
					DestroyWindow(hWnd);
					break;
				default:
					return DefWindowProc(hWnd, message, wParam, lParam);
			}
			break;
		case WM_DESTROY:
            PostQuitMessage(0);
            break;

		case WM_PAINT:
		{
			RECT r;
			GetClientRect(hWnd, &r);

			InvalidateRect(hWnd, &r, TRUE);
			dc = BeginPaint(hWnd, &ps);

			int mul = 10;

			for (int i=0; i < draw.m_iCount; i++)
			{

				if (draw.m_pSoldiers[i].m_pWay)
				{
					int j=0;
					while (j < draw.m_pSoldiers[i].m_iWaypointCount-1)
					{
						Vector curr = draw.m_pSoldiers[i].m_pWay[j];
						Vector next = draw.m_pSoldiers[i].m_pWay[j+1];
						DrawLine (dc, 2, 1, (int)floor(curr.x *mul), (int)floor(curr.y *mul), (int)floor((next.x-curr.x)*mul), (int)floor((next.y-curr.y) *mul));
						j++;				
					}
				}

				DrawLine (dc, 2, 4, (int)floor(draw.m_pSoldiers[i].m_Vectors.pos.x *mul), (int)floor(draw.m_pSoldiers[i].m_Vectors.pos.y *mul), (int)floor(draw.m_pSoldiers[i].m_Vectors.total.x *mul), (int)floor(draw.m_pSoldiers[i].m_Vectors.total.y *mul));
				DrawLine (dc, 1, 0, (int)floor(draw.m_pSoldiers[i].m_Vectors.pos.x *mul), (int)floor(draw.m_pSoldiers[i].m_Vectors.pos.y *mul), (int)floor(draw.m_pSoldiers[i].m_Vectors.move.x *mul), (int)floor(draw.m_pSoldiers[i].m_Vectors.move.y *mul));
				DrawLine (dc, 1, 1, (int)floor(draw.m_pSoldiers[i].m_Vectors.pos.x *mul), (int)floor(draw.m_pSoldiers[i].m_Vectors.pos.y *mul), (int)floor(draw.m_pSoldiers[i].m_Vectors.v0.x *mul), (int)floor(draw.m_pSoldiers[i].m_Vectors.v0.y *mul));
				DrawLine (dc, 1, 2, (int)floor(draw.m_pSoldiers[i].m_Vectors.pos.x *mul), (int)floor(draw.m_pSoldiers[i].m_Vectors.pos.y *mul), (int)floor(draw.m_pSoldiers[i].m_Vectors.v1.x *mul), (int)floor(draw.m_pSoldiers[i].m_Vectors.v1.y *mul));
				DrawLine (dc, 1, 3, (int)floor(draw.m_pSoldiers[i].m_Vectors.pos.x *mul), (int)floor(draw.m_pSoldiers[i].m_Vectors.pos.y *mul), (int)floor(draw.m_pSoldiers[i].m_Vectors.coll.x *mul), (int)floor(draw.m_pSoldiers[i].m_Vectors.coll.y *mul));
				DrawLine (dc, 1, 5, (int)floor(draw.m_pSoldiers[i].m_Vectors.pos.x *mul), (int)floor(draw.m_pSoldiers[i].m_Vectors.pos.y *mul), (int)floor(draw.m_pSoldiers[i].m_Vectors.v2.x *mul), (int)floor(draw.m_pSoldiers[i].m_Vectors.v2.y *mul));
			}
	
			EndPaint(hWnd, &ps);
		}
			break;

   }
	return DefWindowProc(hWnd, message, wParam, lParam);
}






