
// $Id: Viewport.hpp,v 1.8 2001/04/08 15:40:52 mitja Exp $

/* Viewports class
 *
 * A basic output for the program.  
 * Created by: Matija Jekovec
 * Created on: 22.11.2000
 */

#ifndef __VIEWPORT__
#define __VIEWPORT__

class Viewport;

#include "SDL.h"
#include "EnvironmentClient.hpp"
#include "TuiGraphics.hpp"
#include "Console.hpp"


const VIEWPORT_RECT_OUTSIDE  = 0;
const VIEWPORT_RECT_INSIDE   = 1;
const VIEWPORT_RECT_OVERLAPS = 2;

Uint32 MapColor(MyColor color, SDL_PixelFormat *fmt); // returns an integer using the appropriate mask

class Viewport
{
private:
	Console						*console;			//pointer to console
	SDL_Surface       *offScreen;   // offscreen surface, to which only the static environment is written
	float fZoom;                    // zoom on the current window

	/* Incorporates the logic for blitting the unchanged part of environment and
	 * redrawing the  newly exposed and therefore blank part of the viewport */
	void DrawBase(float oldX, float oldY, float newX, float newY);

	void DrawOverlay(SDL_Surface *surface);	// draws all overlay objects on surface

	// draws a drag rectangle from world coordinates
	void DrawScreenRect(SDL_Surface *surface, float sX, float sY, float eX, float eY);
	void DrawWaypoint(SDL_Surface *surface, int x, int y, int size);		//draws a waypoint marker
	void DrawAOD(SDL_Surface *surface, int x, int y, int r);		//draws a waypoint marker
	float startX, startY; // starting point of the drag
	float endX, endY;     // ending point of the drag
	
	bool drag; // mouse is in drag mode

public:
	// constructor, creates offScreen
	Viewport(Console *console, SDL_PixelFormat *fmt, int w, int h);
	~Viewport(); // destructor, also disposes of offScreen, detaches pointer to functions environment
	bool  active;        // active viewport for scrolling
	int   left, top;     // position of the viewport on screen
	int   height, width; // dimensions of the viewport
	float posX, posY;    // top left corner of the viewport relative to the environment
	
	float GetZoom(); // returns zoom
	void  SetZoom(float zoom, SDL_Surface *screen);	// sets zoom
	float GetStartX(); // get starting point of the drag
	float GetStartY();

	/* called when main event handling  procedure is invoked (used to convert the absolute
	 * coordinates on screen to environment coordinates)*/
	bool MouseButtonDown(SDL_Surface *screen, int x, int y, float *rX, float *rY, Uint8 Button); 

	/* called when main event handling  procedure is invoked (used to convert the absolute
	 * coordinates on screen to environment coordinates) */
	bool MouseButtonUp(SDL_Surface *screen, int x, int y, float *rX, float *rY, Uint8 Button);

	/* called when main event handling  procedure is invoked (used to convert the absolute
	 * coordinates on screen to environment coordinates) */
	bool MouseMotion(SDL_Surface *screen, int x, int y, float *rX, float *rY);

	/* calls DrawBase with the required parameters, then sets the new posX, posY and as
	 * last calls the DrawOverlay. It also sets the dirty rects */
	void Move(float x, float y,  SDL_Surface *screen);

	/* sets the new coordinates and then calls Move(x,y,Rects*,SDL_Surface). Moving 
	 * is stopped if the border is reached, any further movement is this
	 * direction is ignored */	
	void Move(Direction dir, int scrollStep, SDL_Surface *screen);	
	bool PointInViewport(float x, float y);    // checks if point in world is also in viewport
	int  RectInViewport(TUI_rectStruct r);     // checks if rect with world coords is in viewport
	void RefreshOverlay(SDL_Surface *surface); // blits base with surface and the calls DrawOverlay

	/* Resets the background of all the soldiers and bullets using their BoundingRect function
	 * to get their dimensions */											   
	//void ResetOverlayBackground(Rects *rects);
};

#endif // __VIEWPORT__
