// $Id: VisGraph.cpp,v 1.9 2001/04/17 07:22:48 damir Exp $

/* VisGraph.cpp, 1.2, 2001-04-15, Mitja Lustrek
 *
 * See header file.
 */

#include "stdafx.h"
#include <stdlib.h>
#include <math.h>
#include "Auxiliary.hpp"
#include "VisGraph.hpp"
#include "Square.hpp"
#include "Environment.hpp"
#include "List.hpp"
#include "VGVertex.hpp"
#include "Vector.hpp"

//#include <fstream.h> //!!!

#define WEIGHT 1
#define IMPASSABLE 50
#define EXPANSION 0.49f

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


////////////////////////////////////////////////////////////////////////////////////////////////////////

void VisGraph::aStar (VGVertex *start, VGVertex *end)
{
	VGVertex *best, *temp, *current;
	float newAsG;
	bool skip;
	Vector *newCoords;

//	fstream out ("TEST - aStar.txt", ios::out, filebuf::sh_none); //!!!

	// Create lists
	asOpen=new List;
	asClosed=new List;
	asPath=NULL;

	// Calculate A* functions for start of path and add it to asOpen
	start->asG=0;
	start->asH=hDistance (start, end);
	start->asF=start->asG+start->asH;
	start->asParent=NULL;
	asOpen->add (start);

	// Main loop
	while ((asOpen->getNumItems ()>0) && (asPath==NULL))
	{

		// Find best vertex
		best=(VGVertex *)(asOpen->getFirst ());
		temp=(VGVertex *)(asOpen->getFirst ());
		while (temp!=NULL)
		{
			if (temp->asF<=best->asF)
			{
				best=temp;
				asOpen->bookmark (0);
			}
			temp=(VGVertex *)(asOpen->getNext ());
		}
		asOpen->goTo (0);
		asOpen->remove ();

//		out<<"Best: "<<best->x<<", "<<best->y<<"\n"; //!!!

		// Check whether end of path is reached
		if (best==end)
		{
			asPath=new List;
			temp=best->asParent;
			while (temp!=start)
			{
				newCoords=new Vector (temp->x+squareBorder*0.5f, temp->y+squareBorder*0.5f);
				asPath->add (newCoords);
				temp=temp->asParent;
			}
		}
		else
		{

			// Process successors
			current=(VGVertex *)(best->visible->getFirst ());
			while (current!=NULL)
			{

//				out<<"Current: "<<current->x<<", "<<current->y<<"\n"; //!!!

				skip=false;
				newAsG=best->asG+gDistance (best, current);

				// Check whether current is in asClosed
				temp=(VGVertex *)(asClosed->getFirst ());
				while ((temp!=NULL) && (temp!=current))
					temp=(VGVertex *)(asClosed->getNext ());
				if (temp!=NULL)
				{

//					out<<"asClosed\n"; //!!!

					if (current->asG<=newAsG)
						skip=true;
					else
						asClosed->remove ();
				}

				// Check whether current is in asOpen
				if (!skip)
				{
					temp=(VGVertex *)(asOpen->getFirst ());
					while ((temp!=NULL) && (temp!=current))
						temp=(VGVertex *)(asOpen->getNext ()); // !! CRASH !! ??
					if (temp!=NULL)
					{
	
//						out<<"asOpen\n"; //!!!

						if (current->asG<=newAsG)
							skip=true;
					}
					else
						asOpen->add (current);
				}

				// Update current
				if (!skip)
				{
					current->asParent=best;
					current->asG=newAsG;
					current->asH=hDistance (current, end);
					current->asF=current->asG+current->asH;

//					out<<"Current asF: "<<current->asF<<"\n"; //!!!

				}

				asClosed->add (best);
				current=(VGVertex *)(best->visible->getNext ());
			}
		}
	}

	// Set A* values to default and destroy lists
	temp=(VGVertex *)(asOpen->getFirst ());
	while (temp!=NULL)
	{
		temp->asG=0;
		temp->asH=0;
		temp->asF=0;
		temp=(VGVertex *)(asOpen->getNext ());
	}
	temp=(VGVertex *)(asClosed->getFirst ());
	while (temp!=NULL)
	{
		temp->asG=0;
		temp->asH=0;
		temp->asF=0;
		temp=(VGVertex *)(asClosed->getNext ());
	}
	delete asOpen;
	delete asClosed;

/*	out<<"\nasPath:\nStart: "<<start->x<<", "<<start->y<<"; "; //!!!->
	temp=(VGVertex *)(asPath->getFirst ());
	while (temp!=NULL)
	{
		out<<temp->x<<", "<<temp->y<<"; ";
		temp=(VGVertex *)(asPath->getNext ());
	}
	out<<"End: "<<end->x<<", "<<end->y; //!!!<-*/

}

////////////////////////////////////////////////////////////////////////////////////////////////////////

float VisGraph::gDistance (VGVertex *start, VGVertex *end)
{
	return (float)(sqrt ((end->x-start->x)*(end->x-start->x)+(end->y-start->y)*(end->y-start->y)));
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

float VisGraph::hDistance (VGVertex *start, VGVertex *end)
{
	Vector vecStart, vecEnd;

	if (heu!=NULL)
	{
		vecStart.x=(float)(start->x);
		vecStart.y=(float)(start->y);
		vecEnd.x=(float)(end->x);
		vecEnd.y=(float)(end->y);
		return heu->distance (vecStart, vecEnd);
	}
	else
		return gDistance (start, end);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

VisGraph::VisGraph (Environment *env1, Heuristics *heu1)
{
	int i, j;
	VGVertex *start, *end;

//	fstream out ("TEST - koncna slika.txt", ios::out, filebuf::sh_none); //!!!

	// Store pointers to environment and heuristics
	env=env1;
	heu=heu1;

	// Create graph
	graph=new List;
	
	// Copy map from environment
	maxSx=env->getMaxSx ();
	maxSy=env->getMaxSy ();
	squareBorder=env->getSquareBorder ();
	matrix=new Square *[maxSx*maxSy];
	for (i=0; i<maxSx; i++)
		for (j=0; j<maxSy; j++)
			matrix [i*maxSy+j]=env->getSquare (i, j);

	// Process squares
	for (i=0; i<maxSx-1; i++)
		for (j=0; j<maxSy-1; j++)
			processSquare (i, j);

	// Find arcs between vertices
	start=(VGVertex *)(graph->getFirst ());
	while (start!=NULL)
	{
		graph->bookmark (0);
		end=(VGVertex *)(graph->getNext ());
		while (end!=NULL)
		{
			checkVisibility (start, end);
			end=(VGVertex *)(graph->getNext ());
		}
		graph->goTo (0);
		start=(VGVertex *)(graph->getNext ());
	}

/*	start=(VGVertex *)(graph->getFirst ()); //!!!->
	while (start!=NULL)
	{
		out<<start->x<<", "<<start->y<<":\n";
		end=(VGVertex *)(start->visible->getFirst ());
		while (end!=NULL)
		{
			out<<"  "<<end->x<<", "<<end->y<<"\n";
			end=(VGVertex *)(start->visible->getNext ());
		}
		start=(VGVertex *)(graph->getNext ());
	} //!!!<-*/

}


////////////////////////////////////////////////////////////////////////////////////////////////////////

VisGraph::~VisGraph ()
{
	graph->deleteAll(List::VERTEX);
	delete graph;
	delete matrix;
	delete heu;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void VisGraph::processSquare (int x, int y)
{
//	fstream out ("TEST - passability.txt", ios::app, filebuf::sh_none); //!!!

	if ((matrix [x*maxSy+y]->passability<=IMPASSABLE) && (matrix [x*maxSy+y+1]->passability>IMPASSABLE) && (matrix [(x+1)*maxSy+y]->passability>IMPASSABLE) && (matrix [(x+1)*maxSy+y+1]->passability>IMPASSABLE))
		addVertex (matrix [(x+1)*maxSy+y+1], x+1, y+1);
	if ((matrix [x*maxSy+y]->passability>IMPASSABLE) && (matrix [x*maxSy+y+1]->passability<=IMPASSABLE) && (matrix [(x+1)*maxSy+y]->passability>IMPASSABLE) && (matrix [(x+1)*maxSy+y+1]->passability>IMPASSABLE))
		addVertex (matrix [(x+1)*maxSy+y], x+1, y);
	if ((matrix [x*maxSy+y]->passability>IMPASSABLE) && (matrix [x*maxSy+y+1]->passability>IMPASSABLE) && (matrix [(x+1)*maxSy+y]->passability<=IMPASSABLE) && (matrix [(x+1)*maxSy+y+1]->passability>IMPASSABLE))
		addVertex (matrix [x*maxSy+y+1], x, y+1);
	if ((matrix [x*maxSy+y]->passability>IMPASSABLE) && (matrix [x*maxSy+y+1]->passability>IMPASSABLE) && (matrix [(x+1)*maxSy+y]->passability>IMPASSABLE) && (matrix [(x+1)*maxSy+y+1]->passability<=IMPASSABLE))
		addVertex (matrix [x*maxSy+y], x, y);

//	out<<matrix [x*maxSy+y]->passability<<'\n'; //!!!
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void VisGraph::addVertex (Square *square, int x, int y)
{
	VGVertex *newVertex;

//	fstream out ("TEST - vozlisca.txt", ios::app, filebuf::sh_none); //!!!

	newVertex=(VGVertex *)(graph->getFirst ());
	while ((newVertex!=NULL) && ((newVertex->x!=x) || (newVertex->y!=y))) newVertex=(VGVertex *)(graph->getNext ());
	if (newVertex==NULL)
	{
		newVertex=new VGVertex (square, x, y);
		graph->add (newVertex);

//		out<<x<<", "<<y<<'\n'; //!!!

	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

void VisGraph::checkVisibility (VGVertex *start, VGVertex *end)
{
	float k, n, y, x, yM, xM, factor;
	int i, yD, xD, j, limit;
	bool visible=true;

//	fstream out ("TEST - po preverjanju vidljivosti.txt", ios::app, filebuf::sh_none); //!!!

	// Check for crossing of vertical walls
	if (start->x!=end->x)
	{
		k=(float)(end->y-start->y)/(float)(end->x-start->x);
		n=start->y+squareBorder*0.5f-k*(start->x+squareBorder*0.5f);
		for (i=MIN (start->x, end->x); i<MAX (start->x, end->x); i++)
		{
			factor=1.0f-EXPANSION;
			if (i==MAX (start->x, end->x)-1)
				limit=1;
			else
				limit=2;
			for (j=0; j<limit; j++)
			{
				y=k*(i*squareBorder+squareBorder*factor)+n;
				yD=(int)(y/squareBorder);
				yM=y-yD*squareBorder;
				if (matrix [(i+1)*maxSy+yD]->getPassability()<=IMPASSABLE)
				{
					visible=false;
					break;
				}
				if (yM<EXPANSION)
					if (matrix [(i+1)*maxSy+yD-1]->getPassability()<=IMPASSABLE)
					{
						visible=false;
						break;
					}
				if (yM>(1.0f-EXPANSION))
					if (matrix [(i+1)*maxSy+yD+1]->getPassability()<=IMPASSABLE)
					{
						visible=false;
						break;
					}
				factor=2.0f+EXPANSION;
			}
		}
	}

	// Check for crossing of horizontal walls
	if ((visible==true) && (start->y!=end->y))
	{
		k=(float)(end->x-start->x)/(float)(end->y-start->y);
		n=start->x+squareBorder*0.5f-k*(start->y+squareBorder*0.5f);
		for (i=MIN (start->y, end->y); i<MAX (start->y, end->y); i++)
		{
			factor=1.0f-EXPANSION;
			if (i==MAX (start->y, end->y)-1)
				limit=1;
			else
				limit=2;
			for (j=0; j<limit; j++)
			{
				x=k*(i*squareBorder+squareBorder*factor)+n;
				xD=(int)(x/squareBorder);
				xM=x-xD*squareBorder;
				if (matrix [xD*maxSy+i+1]->getPassability()<=IMPASSABLE)
				{
					visible=false;
					break;
				}
				if (xM<EXPANSION)
					if (matrix [(xD-1)*maxSy+i+1]->getPassability()<=IMPASSABLE)
					{
						visible=false;
						break;
					}
				if (xM>(1.0f-EXPANSION))
					if (matrix [(xD+1)*maxSy+i+1]->getPassability()<=IMPASSABLE)
					{
						visible=false;
						break;
					}
				factor=1.0f+EXPANSION;
			}
		}
	}

	// Add arc to the graph if vertices are visible from each other
	if (visible)
	{
		start->addVisible (end);
		end->addVisible (start);

//		out<<start->x<<", "<<start->y<<"; "<<end->x<<", "<<end->y<<"\n";

	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

List *VisGraph::findPath (Vector start, Vector end, Vector corner1, Vector corner2)
{
	VGVertex *startV, *endV, *temp;
	Vector smaller, larger;

//	VGVertex *test1, *test2; //!!!
//	fstream out ("TEST - z dodanima start, end.txt", ios::out, filebuf::sh_none); //!!!

	// Add new vertices to the graph
	startV=new VGVertex (matrix [(int)(start.x)*maxSy+(int)(start.y)], (int)(start.x), (int)(start.y));
	endV=new VGVertex (matrix [(int)(end.x)*maxSy+(int)(end.y)], (int)(end.x), (int)(end.y));

	// This method was called directly
	if (corner1.x>-1)
	{
		smaller.x=MIN (corner1.x, corner2.x);
		smaller.y=MIN (corner1.y, corner2.y);
		larger.x=MAX (corner1.x, corner2.x);
		larger.y=MAX (corner1.y, corner2.y);
		temp=(VGVertex *)(graph->getFirst ());
		while (temp!=NULL)
		{
			if ((temp->x>smaller.x) && (temp->x<larger.x) && (temp->y>smaller.y) && (temp->y<larger.y))
				checkVisibility (startV, temp);	
			temp=(VGVertex *)(graph->getNext ());
		}
		temp=(VGVertex *)(graph->getFirst ());
		while (temp!=NULL)
		{
			if ((temp->x>smaller.x) && (temp->x<larger.x) && (temp->y>smaller.y) && (temp->y<larger.y))
				checkVisibility (endV, temp);	
			temp=(VGVertex *)(graph->getNext ());
		}
	}
	else

	// This method was called from same method with two parameters
	{
		temp=(VGVertex *)(graph->getFirst ());
		while (temp!=NULL)
		{
			checkVisibility (startV, temp);	
			temp=(VGVertex *)(graph->getNext ());
		}
		temp=(VGVertex *)(graph->getFirst ());
		while (temp!=NULL)
		{
			checkVisibility (endV, temp);	
			temp=(VGVertex *)(graph->getNext ());
		}
	}
	checkVisibility (startV, endV);

/*	test1=(VGVertex *)(graph->getFirst ()); //!!!->
	while (test1!=NULL)
	{
		out<<test1->x<<", "<<test1->y<<":\n";
		test2=(VGVertex *)(test1->visible->getFirst ());
		while (test2!=NULL)
		{
			out<<"  "<<test2->x<<", "<<test2->y<<"\n";
			test2=(VGVertex *)(test1->visible->getNext ());
		}
		test1=(VGVertex *)(graph->getNext ());
	}
	out<<startV->x<<", "<<startV->y<<":\n";
	test2=(VGVertex *)(startV->visible->getFirst ());
	while (test2!=NULL)
	{
		out<<"  "<<test2->x<<", "<<test2->y<<"\n";
		test2=(VGVertex *)(startV->visible->getNext ());
	}
	out<<endV->x<<", "<<endV->y<<":\n";
	test2=(VGVertex *)(endV->visible->getFirst ());
	while (test2!=NULL)
	{
		out<<"  "<<test2->x<<", "<<test2->y<<"\n";
		test2=(VGVertex *)(endV->visible->getNext ());
	}	//!!!<-*/
	
	aStar (startV, endV);

	// Remove new vertices from the graph
	startV->removeItself ();
	endV->removeItself ();

/*	out<<"Po removeItself:\n"; //!!!->
	test1=(VGVertex *)(graph->getFirst ());
	while (test1!=NULL)
	{
		out<<test1->x<<", "<<test1->y<<":\n";
		test2=(VGVertex *)(test1->visible->getFirst ());
		while (test2!=NULL)
		{
			out<<"  "<<test2->x<<", "<<test2->y<<"\n";
			test2=(VGVertex *)(test1->visible->getNext ());
		}
		test1=(VGVertex *)(graph->getNext ());
	}
	out<<startV->x<<", "<<startV->y<<":\n";
	test2=(VGVertex *)(startV->visible->getFirst ());
	while (test2!=NULL)
	{
		out<<"  "<<test2->x<<", "<<test2->y<<"\n";
		test2=(VGVertex *)(startV->visible->getNext ());
	}
	out<<endV->x<<", "<<endV->y<<":\n";
	test2=(VGVertex *)(endV->visible->getFirst ());
	while (test2!=NULL)
	{
		out<<"  "<<test2->x<<", "<<test2->y<<"\n";
		test2=(VGVertex *)(endV->visible->getNext ());
	}	//!!!<-*/
	
	delete startV;
	delete endV;

	return asPath;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

List *VisGraph::findPath (Vector start, Vector end)
{
	Vector minusOne;

	minusOne.x=-1;
	minusOne.y=-1;
	return findPath (start, end, minusOne, minusOne);
}