// $Id: VisGraph.hpp,v 1.6 2001/04/15 15:05:41 mitja Exp $

/* VisGraph.hpp, 1.2, 2001-04-15, Mitja Lustrek
 *
 * This class contains visibility graph, which is calculated when an object of this class is created.
 * It has a method for finding shortest path between two points (which are not part of the graph).
 * This is done with A* algorithm, so the class also contains some data structures used by it.
 */

#ifndef __VISGRAPH__
#define __VISGRAPH__

#include "Square.hpp"
#include "List.hpp"
#include "VGVertex.hpp"
#include "Heuristics.hpp"

class Environment;

class VisGraph
{
private:
	Square **matrix;                  // map
	int maxSx;                        // width of the map
	int maxSy;                        // height of the map
	float squareBorder;               // size of map square
	List *graph;                      // visibility graph itself
	Environment *env;                 // environment for which this is visibility graph
	Heuristics *heu;                  // heuristics for calculating hDistance
	List *asOpen, *asClosed, *asPath; // A* lists

	void processSquare (int x, int y);                     // process the square at given coordinates and add vertices to the graph
	void addVertex (Square *square, int x, int y);         // add given square at given coordinates to the graph, if it isn't already there
	void checkVisibility (VGVertex *start, VGVertex *end); // check whether startV and endV are visible form each other and add arc to the graph if they are
	void aStar (VGVertex *vertex, VGVertex *end);          // A* algorithm
	float gDistance (VGVertex *start, VGVertex *end);      // A* actual distance between given vertices
	float hDistance (VGVertex *start, VGVertex *end);      // A* heuristic distance between given vertices

public:
	VisGraph (Environment *env, Heuristics *heu);                              // construct graph for given environment's map with given heuristics
	~VisGraph ();                                                              // destructor
	List *findPath (Vector start, Vector end, Vector corner1, Vector corner2); // find path between given points if it lies within rectangle defined by given corners
	List *findPath (Vector start, Vector end);                                 // find path between given points
};

#endif //__VISGRAPH__