
// $Id: unpackC.cpp,v 1.6 2001/04/18 06:29:08 damir Exp $

/*this is a prototype for unpacking and updating on the client
 *some of the things are just pointed at, but I will make
 *them clearer when things start to get clearer to me
 *
 *Created by: Uros Cibej
 */

#include "stdafx.h"
#include <string.h>
#include <stdlib.h>
#include "EnvironmentClient.hpp"
#include "SoldierClient.hpp"
#include "BulletClient.hpp"
#include "Order.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


enum tMove { MOVEMENT_WALK, MOVEMENT_CROUCH, MOVEMENT_CRAWL }; // types of movement


/*first two functions update the Soldier and the Bullet with ID,
 *if it is not yet in the list it adds a new Soldier or Bullet.
 *
 *function UnpackC gets the packet from the server, unpacks it and
 *updates the lists in the Environment
 */
void updateSoldier(EnvironmentClient* env, int ID, float x, float y, float moveangle,
				   float viewAngle, tMove movetype, float FOV, bool friendly, int health)
{
	SoldierClient *temp, *newSoldier;
	
	temp = (SoldierClient*)env->soldiers->getFirst();
	
	while (temp != NULL && temp->GetSoldierId() != ID)
	{
		temp = (SoldierClient*)env->soldiers->getNext();
	}
	if (temp == NULL)
	{
		if (friendly)
			newSoldier = new SoldierClient(NULL, NULL, ID, clBlue);
		else
			newSoldier = new SoldierClient(NULL, NULL, ID, clRed);
		newSoldier->SetMoveAngle(moveangle);
		newSoldier->SetX(x);
		newSoldier->SetY(y);
		newSoldier->SetViewAngle(viewAngle);
		newSoldier->SetFOV(FOV);
		newSoldier->SetHealth(((float)health)/255.0f);
		newSoldier->SetBodyPosition(movetype);
		env->soldiers->add(newSoldier);
	}
	else 
	{
		temp->SetX(x);
		temp->SetY(y);
		temp->SetMoveAngle(moveangle);
		temp->SetViewAngle(viewAngle);
		temp->SetFOV(FOV);
		temp->SetHealth(((float)health)/255.0f);
		temp->updateFlag = true;
		temp->SetBodyPosition(movetype);
	}
}


void updateBullet(EnvironmentClient* env, int ID, float x, float y)
{
	BulletClient *temp, *newBullet;
	
	temp = (BulletClient*)env->bullets->getFirst();
	
	while (temp != NULL && temp->GetId() != ID)
	{
		temp = (BulletClient*)env->bullets->getNext();
	}
	if (temp == NULL)
	{
		newBullet = new BulletClient(NULL, NULL, x, y, ID);
		// newBullet->SetMoveAngle(moveangle);
		env->bullets->add(newBullet);
	}
	else 
	{
		temp->SetX(x);
		temp->SetY(y);
		temp->updated = true;
		// temp->SetMoveAngle(moveangle);
	}
}


void clearUpdateFlag(EnvironmentClient *env)
{
	SoldierClient *sol;
	sol =  (SoldierClient *) env->soldiers->getFirst();
	while (sol)
	{
		sol->updateFlag = false;
		sol = (SoldierClient *) env->soldiers->getNext();
	}
}


void removeOutdatedSoldiers(EnvironmentClient *env)
{
	SoldierClient *sol;
	sol = (SoldierClient *) env->soldiers->getFirst();
	while (sol)
	{
		if (!sol->updateFlag)
		{
			env->soldiers->remove();
			delete sol;
		}
		sol = (SoldierClient *) env->soldiers->getNext();
	}
}


void unpackC(EnvironmentClient* E, char* package, int friendlyID)
{
	int numS, numB, i, diff;
	SoldierPack tsoldier;
	BulletPack tbullet;
	BulletClient *bullet;
	
	memcpy(&numS, &package[0], sizeof(int));
	memcpy(&numB, &package[4], sizeof(int));

	clearUpdateFlag(E);
	
	for (i = 0; i < numS; i++)
	{
		memcpy(&tsoldier, &package[8 + i*sizeof(SoldierPack)], sizeof(SoldierPack));
		// search for the soldier with tsoldier.ID, and update
		updateSoldier(E, tsoldier.ID, 50*tsoldier.x, 50*tsoldier.y, tsoldier.moveAngle, tsoldier.viewAngle,
 			(enum tMove)tsoldier.moveType, tsoldier.fieldOfView, (tsoldier.aiID == friendlyID), tsoldier.health);
	}

	removeOutdatedSoldiers(E);
	
	bullet =(BulletClient *) E->bullets->getFirst();
	while (bullet) 
	{
		bullet->updated = false;
		bullet =(BulletClient *) E->bullets->getNext();
	}
	diff = 8 + numS*sizeof(SoldierPack);
	for (i = 0; i < numB; i++)
	{
		memcpy(&tbullet, &package[diff + i*sizeof(BulletPack)], sizeof(BulletPack));
		updateBullet(E, tbullet.ID, 50*tbullet.x, 50*tbullet.y);
	}
	bullet =(BulletClient *) E->bullets->getFirst();
	while (bullet) 
	{
		if (bullet->updated == false)
		{
			E->bullets->remove();
			delete bullet;
		}
		bullet =(BulletClient *) E->bullets->getNext();
	}
}
